CREATE TABLE street (
	id serial primary key,
	streettype text,
	street text,
	suburb text,
	city text,
	areacode character(12)
);

CREATE TABLE recordlastcard (
	id serial primary key,
	dummy integer,
	cardno text
);

CREATE TABLE footsentrykeylist (
	id serial primary key,
	cardno integer
);

CREATE TABLE flag (
	id serial primary key,
	door integer,
	command integer,
	cardno integer
);

CREATE TABLE programmegroup (
	id serial primary key,
	text name not null
);

CREATE TABLE paymenttype (
	id serial primary key,
	name text not null
);

CREATE TABLE programme (
	id serial primary key,
	programmegroupid integer NOT NULL references programmegroup (id),
	name text not null,
	director text,
	amount double precision default 0,
	enrollimit integer,
	waitinglistlimit integer,
	startdate date,
	enddate date,
	imembershiplength integer,
	membershiplengthtype integer,
	concession integer,
	programmetype integer,
	history boolean default 'f',
	quickadd boolean default 'f',
	oneoff boolean,
	notes text,
	emailcontact text,

	lastupdate date default (now()),  
	maxage integer,
  	minage integer,
   	emailcontact text,
    	header text,
     	description text             
     

);

CREATE TABLE member (
	id serial primary key,
	name text not null,
	gender character(1),
	email text,
	dob date,
	joiningdate date default date(now()),
	cardno integer,
	occupation text,
	notes text,
	medicalconditions text,

	emergencyname varchar(100),
	emergencyhome varchar(100),
	emergencywork varchar(100),
	emergencycell varchar(100),
	
	phonehome varchar(30),
	phonework varchar(30),
	phonecell varchar(30),
	
	addressstreet varchar(100),
	addresssuburb varchar(100),
	addresscity varchar(40),
	addressareacode varchar(30),

	receivemailout boolean default 't',
	receiveemail boolean default 't',

	lastupdate date
);

CREATE TABLE membership (
	id serial primary key,
	programmeid integer NOT NULL references programme (id),
	memberid integer NOT NULL references member (id),
	startdate date default date(now()),
	enddate date,
	cardno integer,
	concession integer default 0,
	numrenewals integer default 0,
	history boolean default 'f',
	waitinglist boolean,
	programmegroupid integer NOT NULL references programmegroup (id),
	notes text,

	lastupdate date default (now()),
	show_web boolean default false
);

CREATE TABLE payment (
	id serial primary key,
	paymenttypeid integer NOT NULL references paymenttype (id),
	memberid integer NOT NULL references member (id),
	membershipid integer constraint ref_membership references membership (id),
	paymentdate date default date(now()),
	amount double precision default 0,
	note text,

	lastupdate date
);

CREATE TABLE membershipamount (
	id serial primary key,
	membershipid integer NOT NULL references membership (id),
	duedate date default date(now()),
	amount double precision default 0
);

CREATE TABLE programmetype (
	id serial primary key,
	description text
);

CREATE TABLE membershiplengthtype (
	id serial primary key,
	description text
);

CREATE TABLE entryexit (
	id serial primary key,
	time timestamp DEFAULT (now()),
	cardno integer NOT NULL,
	door integer,
	access integer,
	checked boolean,
	membershipid integer
);

CREATE TABLE allowedentry (
	id serial primary key,
	programmeid integer NOT NULL references programme(id),
	starttime date DEFAULT date(now()),
	endtime date DEFAULT date(now()),
	door integer,
	dayofweek text
);

CREATE TABLE reports (
	id serial primary key,
	filename text,
	title text
);

CREATE FUNCTION update_last_date() RETURNS trigger AS '
 BEGIN
        NEW.lastupdate := ''now()'';
        RETURN NEW;
    END;
 ' LANGUAGE 'plpgsql';

CREATE TRIGGER update_last_date
BEFORE INSERT OR UPDATE ON member
     FOR EACH ROW EXECUTE PROCEDURE update_last_date();


CREATE TRIGGER update_last_date
BEFORE INSERT OR UPDATE ON membership
     FOR EACH ROW EXECUTE PROCEDURE update_last_date();

CREATE TRIGGER update_last_date
BEFORE INSERT OR UPDATE ON programme
     FOR EACH ROW EXECUTE PROCEDURE update_last_date();
      
      
CREATE TABLE venue (
	id  serial primary key,
	name text not null,
	phone varchar(20),
	mobile varchar(20),
	fax varchar(20),
	email text,
	website varchar(50),

	addressstreet varchar(100),
	addresssuburb varchar(100),
	addresscity varchar(40),
	addressareacode varchar(30),
	
	contactperson text,
	
	costperperson numeric(8,2) not null default 0,
	costpergroup numeric(8,2) not null default 0,
	costperleader numeric(8,2) not null default 0,
	paymentmethod int not null default 0,
	
	agesuitablemin int default 5,
	agesuitablemax int default 15,
	groupsizemin int default 1,
	groupsizemax int default 60,
	weather int default 0,
	summerseason boolean default 'f',
	autumnseason boolean default 'f',
	winterseason boolean default 'f',
	springseason boolean default 'f',
	timesavailiable text,
	timetoallow text,
	history boolean default 'f',
	
	toilets boolean default 'f',
	foodavailable boolean default 'f',
	ownlunch boolean default 'f',
	busstop text,
	activities text,
	descriptionofvenue text,
	safetyissues text,
	staffatvenue text,
	feedbackadvice text,
	gearrequired text,
	comments text
);

CREATE TABLE programme_venue (
	id serial primary key,
	programmeid integer NOT NULL references programme(id),
	venueid integer NOT NULL references venue(id),
	arrivaldatetime timestamp default '2004-01-01 10:00',
	departuredatetime timestamp default '2004-01-01 10:00',
	arrivalplace varchar(60),
	departureplace varchar(60),
	costincurred numeric(8,2),
	specialinstructions text
);

CREATE TABLE organisation (
       id serial primary key,
       name text,
       postaladdress text,
       physicaladdress text,
       contactname text,
       phonehome varchar(30),
       phonework varchar(30),
       phonecell varchar(30),
       email text
);

CREATE TABLE organisation_event (
       id serial primary key,
       description text
);

CREATE TABLE organisation_activity (
       id serial primary key,
       organisationid int references organisation(id),
       eventid int references organisation_event(id),
       location text,
       startdate date default date(now()),
       enddate date,
       term int,
       numberofclients int,
       priceperperson numeric(8,2),
       pricepergroup numeric(8,2),
       futurerecommendations text,
       comments text
);
