<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:output method="xml" indent="yes" 
    doctype-public="-//treshna.com//DTD Bond V2.0//EN" 
    doctype-system="http://bond.treshna.com/bond-2.0.dtd" />
  <xsl:strip-space elements="*" />
  
  <xsl:template match="glade-interface">
    <bond-intermediate>
      <xsl:apply-templates />
    </bond-intermediate>
  </xsl:template>
  
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <xsl:template match="widget">
    <!-- Get the widget name, as the substring after the first 3
         characters if those characters are 'Gtk', else the
         widget name will be equal to @class -->
    <xsl:variable name="widget-name">
      <xsl:call-template name="convert-widget-name" />
    </xsl:variable>
    
    <!-- Create the element using the variable $widget-name -->
    <xsl:element name="{$widget-name}">
      <xsl:attribute name="name">
        <xsl:value-of select="@id" />
      </xsl:attribute>
      <xsl:apply-templates />
      <xsl:if test="$widget-name='label'">
        <xsl:call-template name="get-label-property" />
      </xsl:if>
    </xsl:element>
  </xsl:template>
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->

  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <xsl:template match="widget[@class='GtkMenu']">
    <xsl:apply-templates />
  </xsl:template>


  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - --> 
  <xsl:template match="property">
    <xsl:variable name="property-name">
      <xsl:call-template name="get-property-name" />
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$property-name='x' or $property-name='y'">
        <xsl:call-template name="get-property-options">
          <xsl:with-param name="xy" select="$property-name" />
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$property-name!=''">
        <xsl:attribute name="{$property-name}">
          <xsl:call-template name="get-property" />
        </xsl:attribute>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->  

  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <!-- x_options splits up into three attributes: xfill, xexpand and
       xshrink. Similarly for y_options -->
  <xsl:template name="get-property-options">
    <xsl:param name="xy" />
    <xsl:if test="contains(text(), 'fill')">
      <xsl:attribute name="{concat($xy, 'fill')}">true</xsl:attribute>
    </xsl:if>
    <xsl:if test="contains(text(), 'expand')">
      <xsl:attribute name="{concat($xy, 'expand')}">true</xsl:attribute>
    </xsl:if>
    <xsl:if test="contains(text(), 'shrink')">
      <xsl:attribute name="{concat($xy, 'shrink')}">true</xsl:attribute>
    </xsl:if>
  </xsl:template>
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->


  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->  
  <xsl:template name="get-label-property">
    <xsl:value-of select="property[@name='label']" />
  </xsl:template>
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->  

  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->  
  <xsl:template match="packing">
    <xsl:variable name="packing-name">
      <xsl:call-template name="get-packing-name" />
    </xsl:variable>
    <xsl:element name="{$packing-name}">
      <xsl:apply-templates select="*"/>
      <xsl:apply-templates select="preceding-sibling::*[1][name()='widget']" />
    </xsl:element>
  </xsl:template>
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->


  <!-- A little messy this one... -->
  <xsl:template match="packing[property/@name='new_group']">
    <separator />
    <xsl:apply-templates select="preceding-sibling::*[1][name()='widget']" />
  </xsl:template>


  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <xsl:template match="signal">
    <xsl:variable name="event">
      <xsl:call-template name="get-event" />
    </xsl:variable>
    <xsl:variable name="api-call" select="substring-after(@handler, 'bond_')"/>
    <xsl:if test="$event!=''">
      <trigger event="{$event}">
        <xsl:if test="$api-call!=''">
          <api call="{$api-call}">
            <widget><xsl:value-of select="@object" /></widget>
          </api>
        </xsl:if>
      </trigger>
    </xsl:if>
  </xsl:template>
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->

  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <xsl:template match="accelerator">
    <xsl:copy-of select="." />
  </xsl:template> 
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
     
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <xsl:template match="child">
    <xsl:apply-templates select="packing"/>
    <xsl:apply-templates select="widget[name(following-sibling::*[1])!='packing']"/>
  </xsl:template>
  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->

  <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
       NAMED TEMPLATES - - - - - - - - - - - - - - - - - - - - - - - -
       - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
  <!-- Convert our widget @class. Strip the suffix 'Gtk', translate to 
       lower case, convert elements still not reflecting our syntax. -->
  <xsl:template name="convert-widget-name">
    <xsl:variable 
      name="class" 
      select="translate(substring-after(@class, 'Gtk'),
              'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
              'abcdefghijklmnopqrstuvwxyz')" />

    <!-- There are three widgets in glade (that I know of) that don't
         have the 'Gtk' suffix. Add a comment for unknown ones -->
    <xsl:choose>
      <xsl:when test="not($class)">
        <xsl:if test="not(@class='toggle'
                        or @class='radio' 
                        or @class='button')">
          <xsl:comment>
            <xsl:text>Not too sure about this widget!</xsl:text>
          </xsl:comment>
        </xsl:if>
        <xsl:value-of select="@class" />
      </xsl:when>
      <xsl:when test="$class='table'">box</xsl:when>
      <xsl:when test="$class='checkbutton'">check</xsl:when>
      <xsl:when test="$class='radiobutton'">radio</xsl:when>
      <xsl:when test="$class='spinbutton'">spin</xsl:when>
      <xsl:when test="$class='hbuttonbox'">hbox</xsl:when>
      <xsl:when test="$class='vbuttonbox'">vbox</xsl:when>
      <xsl:when test="$class='imagemenuitem'">menuitem</xsl:when>
      <xsl:when test="$class='menuitem' and 
                      child/widget/@class='GtkMenu'">menu</xsl:when>
      <xsl:when test="$class='combo'">dropdown</xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$class" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="get-property-name">
    <xsl:choose>
      <xsl:when test="@name='shadow_type'">shadow</xsl:when>
      <xsl:when test="@name='visibility'">hide-input</xsl:when>
      <xsl:when test="@name='n_rows'">rows</xsl:when>
      <xsl:when test="@name='n_columns'">columns</xsl:when>
      <xsl:when test="@name='left_attach'">xpos</xsl:when>
      <xsl:when test="@name='right_attach'">xspan</xsl:when>
      <xsl:when test="@name='top_attach'">ypos</xsl:when>
      <xsl:when test="@name='bottom_attach'">yspan</xsl:when>
      <xsl:when test="@name='x_padding'">xpad</xsl:when>
      <xsl:when test="@name='y_padding'">ypad</xsl:when>
      <xsl:when test="@name='x_options'">x</xsl:when>
      <xsl:when test="@name='y_options'">y</xsl:when>
      <xsl:when test="@name='border_width'">border</xsl:when>
      <xsl:when test="@name='toolbar_style'">style</xsl:when>
      <xsl:when test="@name='label' and 
                      contains(./text(), 'gtk-')">stock</xsl:when>
      <xsl:when test="@name='use_stock' and text()='True' or 
                      @name='label' and ../@class='GtkLabel'" />
      <!-- there's got to be a better way... -->
      <xsl:when test="@name='text'">
        <xsl:choose>
          <xsl:when test="not(text())" />
          <xsl:otherwise>
            <xsl:if test="../@class='GtkEntry'">defaultvalue</xsl:if>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="@name" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- Convert properties to our naming convention -->
  <xsl:template name="get-property">
    <xsl:choose>
      <xsl:when test="@name='visibility' and text()='True'">
        <xsl:value-of select="'false'" />
      </xsl:when>
      <xsl:when test="text()='True'">true</xsl:when>
      <xsl:when test="text()='False'">false</xsl:when>
      <xsl:when test="text()='GTK_ORIENTATION_HORIZONTAL'">
        <xsl:text>horizontal</xsl:text>
      </xsl:when>
      <xsl:when test="text()='GTK_JUSTIFY_LEFT'">left</xsl:when>
      <xsl:when test="text()='GTK_JUSTIFY_RIGHT'">right</xsl:when>
      <xsl:when test="text()='GTK_JUSTIFY_CENTER'">center</xsl:when>
      <xsl:when test="text()='GTK_JUSTIFY_FILL'">fill</xsl:when>
      <xsl:when test="text()='GTK_SHADOW_NONE'">none</xsl:when>
      <xsl:when test="text()='GTK_SHADOW_IN'">in</xsl:when>
      <xsl:when test="text()='GTK_SHADOW_OUT'">out</xsl:when>
      <xsl:when test="text()='GTK_SHADOW_ETCHED_IN'">etched-in</xsl:when>
      <xsl:when test="text()='GTK_SHADOW_ETCHED_OUT'">etched-out</xsl:when>
      <xsl:when test="text()='GTK_ORIENTATION_VERTICAL'">
        <xsl:text>vertical</xsl:text>
      </xsl:when>
      <xsl:when test="text()='GTK_TOOLBAR_BOTH'">both</xsl:when>
      <xsl:when test="text()='GTK_TOOLBAR_ICONS'">icons</xsl:when>
      <xsl:when test="text()='GTK_TOOLBAR_TEXT'">text</xsl:when>
      <xsl:when test="text()='GTK_TOOLBAR_BOTH_HORIZ'">both-beside</xsl:when>
      <xsl:when test="@name='right_attach'">
        <xsl:value-of select="text()-(../property[@name='left_attach'])" />
      </xsl:when>
      <xsl:when test="@name='bottom_attach'">
        <xsl:value-of select="text()-(../property[@name='top_attach'])" />
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="text()" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="get-packing-name">
    <xsl:choose>
      <xsl:when test="preceding-sibling::*[1]
                      [@class='GtkNotebook']">page</xsl:when>
      <xsl:when test="preceding-sibling::*[1]
                      [@class='GtkButton'] and 
                      ./property
                      [@name='new_group']/text()='True'">toolbarspace</xsl:when>
      <xsl:otherwise>cell</xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="get-event">
    <xsl:choose>
      <xsl:when test="@name='changed'">change</xsl:when>
      <xsl:when test="@name='focus_in_event'">entry</xsl:when>
      <xsl:when test="@name='focus_out_event'">exit</xsl:when>
      <xsl:when test="@name='activate'">click</xsl:when>
      <xsl:when test="@name='clicked'">click</xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="@name" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
