#include <stdio.h>
#include <stdlib.h>

#include <gnome.h>

#include "general.h"

static char *cur_text = NULL;
static int cur_reply = 0;

static void string_reply (gchar *string, gpointer data)
   {
   if (!string)
      exit(0);
   
   cur_text = realloc (cur_text, strlen (string) + 1);
   strcpy (cur_text, string);
   gtk_main_quit ();
   }

static void yesno_reply (gint reply, gpointer data)
   {
   cur_reply = !reply;
   gtk_main_quit ();
   }

char *gen_today (void)
   {
   return "31/12/2001";
   }

char *gen_query_string (char *prompt, char *def)
   {
   GtkWidget *dialog;
   dialog = gnome_request_dialog (FALSE, prompt, def, 80,
				  string_reply, NULL, NULL);     
   gtk_widget_show_all (dialog);
   gtk_main ();
   return cur_text;
   }

char *gen_query_password (char *prompt)
   {
   GtkWidget *dialog;
   dialog = gnome_request_dialog (TRUE, prompt, NULL, 80,
				  string_reply, NULL, NULL);     
   gtk_widget_show_all (dialog);
   gtk_main ();
   return cur_text;
   }

int gen_query_yesno (char *prompt)
   {
   GtkWidget *dialog;
   dialog = gnome_question_dialog (prompt, yesno_reply, NULL);
   gtk_widget_show_all (dialog);
   gtk_main ();
   return cur_reply;
   }

int gen_query_integer (char *prompt, int *def, int *min, int *max)
   {
   GtkWidget *dialog, *label, *spin;
   GtkObject *adj;
   int button;
   int value;
   
   dialog = gnome_dialog_new ("Query",
			      GNOME_STOCK_BUTTON_OK,
			      GNOME_STOCK_BUTTON_CANCEL,
			      NULL);

   label = gtk_label_new (prompt);
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, 
		       TRUE, TRUE, 0);
   adj = gtk_adjustment_new (*def, *min, *max, 1.0, 5.0, 5.0);
   spin = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1.0, 0);
   gtk_widget_ref (spin);
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), spin,
		       TRUE, TRUE, 0);
   
   gtk_widget_show_all (dialog);

   button = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
   
   if (button == -1 || button == 1)
      exit (0);

   value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
   gtk_widget_unref (spin);
   
   return value;
   }

double gen_query_double (char *prompt, double *def, double *min, double *max)
   {
   GtkWidget *dialog, *label, *spin;
   GtkObject *adj;
   int button;
   double value;
   
   dialog = gnome_dialog_new ("Query",
			      GNOME_STOCK_BUTTON_OK,
			      GNOME_STOCK_BUTTON_CANCEL,
			      NULL);

   label = gtk_label_new (prompt);
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, 
		       TRUE, TRUE, 0);
   adj = gtk_adjustment_new (*def, *min, *max, 0.1, 1.0, 1.0);
   spin = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1.0, 5);
   gtk_widget_ref (spin);
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), spin,
		       TRUE, TRUE, 0);
   
   gtk_widget_show_all (dialog);

   button = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
   
   if (button == -1 || button == 1)
      exit (0);

   value = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (spin));
   gtk_widget_unref (spin);
   
   return value;
   }

char *gen_query_date (char *prompt, char *def)
   {
   GtkWidget *dialog, *label, *date;
   int button;
   time_t t;
   
   dialog = gnome_dialog_new ("Date Query",
			      GNOME_STOCK_BUTTON_OK,
			      GNOME_STOCK_BUTTON_CANCEL,
			      NULL);

   label = gtk_label_new (prompt);
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, 
		       TRUE, TRUE, 0);

   date = gnome_date_edit_new (time (NULL), 0, 0);
   gtk_widget_ref (date);
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), date, 
		       TRUE, TRUE, 0);
   
   gtk_widget_show_all (dialog);

   button = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
   
   if (button == -1 || button == 1)
      exit (0);

   t = gnome_date_edit_get_date (GNOME_DATE_EDIT (date));

   gtk_widget_unref (date);
   
   return ctime (&t);
   }
