#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <gnome.h>
#include <glib.h>
#include <libgnomeprint/gnome-print.h>
#include <libgnomeprint/gnome-printer-dialog.h>
#include <libgnomeprint/gnome-print-master-preview.h>

#include "print.h"

#define min(a,b) ((a)<(b))?(a):(b)
#define max(a,b) ((a)>(b))?(a):(b)

int print_current_page = 0;
static int is_preview = 0;
static GnomePrintMaster *gpm = NULL;
static char *master_papername;

char *get_paper_details (char *papername)
   {
   FILE *fp;
   int pos;
   char line[121];
   char tmppaper[50];
   static char details[121];
   
   if ((fp = fopen ("/etc/paper.config", "r")) == NULL)
      return NULL;

   while (fgets (line, 120, fp) != NULL)
      {
      if (strstr (line, "=") != NULL)
	 {	 
	 pos = strstr(line, "=") - line;
	 strncpy (tmppaper, line, pos);
	 tmppaper[pos] = '\0';
/*	 printf ("Got paper name: %s\n", tmppaper); */
	 }
      else
	 continue;
      
      if (strcasecmp (papername, tmppaper) == 0)
	 {
	 strcpy (details, &line[pos+2]);
	 details[strlen(details)-2]='\0';
	 return details;
	 }
      }
      
   return NULL;
   }

int print_paper_width (void)
   {
   char *details = get_paper_details (master_papername);
   return atoi(details);
   }

int print_paper_height (void)
   {   
   char *details = get_paper_details (master_papername);
   return atoi (&details[strstr(details, " ") - details]);
   }

int print_start_file (char *filename, char *papername)
   {
   GnomePrinter *printer;
   GnomePrintContext *context;
   
   master_papername = papername;

   printer = gnome_printer_new_generic_ps (filename);
   context = gnome_print_context_new_with_paper_size (printer, papername);
   
   return (int)context;
   }

int print_start_preview (char *papername)
   {
   GnomePrintContext *context;
   
   master_papername = papername;

   is_preview = 1;
   
   gpm = gnome_print_master_new ();
   gnome_print_master_set_paper (gpm, gnome_paper_with_name (papername));

   context = gnome_print_master_get_context (gpm);

   return (int)context;
   }

int print_start_printer (char *papername)
   {
   GnomePrinter *printer;
   GnomePrintContext *context;

   master_papername = papername;
   
   printer = gnome_printer_dialog_new_modal ();
   context = gnome_print_context_new_with_paper_size (printer, papername);
   
   return (int)context;
   }

void print_setcolour (int *printer, double *red, double *green, double *blue)
   {
   gnome_print_setrgbcolor ((void *)*printer, *red, *green, *blue);
   }

int print_getfont (char *name, double *size)
   {
   return (int)gnome_font_new (name, *size);
   }

void print_setfont (int *printer, int *font)
   {
   gnome_print_setfont ((void *)*printer, (void *)*font);
   }

int print_lineheight (int *fontp)
   {
   GnomeFont *font = (void*)*fontp;
   
   return gnome_font_get_ascender (font) + 
      gnome_font_get_descender (font);
   }

int print_linewidth (int *fontp, char *text)
   {
   return gnome_font_get_width_string ((void *)*fontp, text);
   }

int print_show (int *printer, int *fontp, char *text)
   {
   print_setfont (printer, fontp);
   gnome_print_show ((void *)*printer, text);
   
   return print_lineheight (fontp);
   }

int print_showat (int *printer, int *fontp, int *x, int *y, char *text)
   {
   print_moveto (printer, x, y);
   return print_show (printer, fontp, text);
   }

int print_show_centered (int *printer, int *fontp, int *x0p, int *x1p,
			 int *y, char *text)
   {
   int xpos, x0=*x0p, x1=*x1p, width;
   
   width = print_linewidth (fontp, text);
   
   xpos = ((x1 - x0) / 2) - (width / 2) + x0;
   
   fprintf (stderr, "print_show_centered: width: %d, x0: %d, x1: %d, xpos: %d\n",
	    width, x0, x1, xpos);	    
   
   return print_showat (printer, fontp, &xpos, y, text);
   }

int print_show_paragraph (int *printer, int *fontp, int *para_width,
			   int *xp, int *yp, char *text)
   {
   char *upto = text;
   int curlen, width = *para_width, x = *xp, y = *yp, i;
   double lineheight, totalheight = 0;
   GnomeFont *font = (void *)*fontp;
   GnomePrintContext *context = (void*)*printer;
   
   gnome_print_setfont (context, font);
   
   while (*upto != '\0')
      {
      for (curlen = 0; curlen < strlen (upto); curlen++)
	 {
	 if (upto[curlen] == '\n' ||
	     gnome_font_get_width_string (font, upto) < width ||
	     (gnome_font_get_width_string_n (font, upto, curlen) > width &&
	      gnome_font_get_width_string_n (font, upto, curlen - 1) < width))
	    {
	    if (gnome_font_get_width_string (font, upto) < width)
	       curlen = strlen (upto) + 1;
	    else if (upto[curlen] == '\n')
	       curlen ++;
	    else
	       curlen = min(curlen - 1, strlen (upto));
	    
	    while (upto[curlen - 1] != '\0' && upto[curlen - 1] != ' ' &&
		   upto[curlen - 1] != '\n')
	       curlen--;
	    
	    for (i = 0; i < curlen; i++)
	       if (upto[i] == '\n')
		  upto[i] = ' ';
	    
	    gnome_print_moveto (context, x, y);
	    gnome_print_show_sized (context, upto, curlen);
	    lineheight = 1.2 * print_lineheight (fontp);
	    y -= lineheight;
	    totalheight += lineheight;
	    upto += curlen;
	    break;
	    }
	 }
      }
   return totalheight;
   }

void print_image (int *printer, int *x, int *y,  char *filename)
   {
   double scale = 1.0;
   print_image_scaled (printer, x, y, &scale, filename);
   }

void print_image_scaled (int *printer, int *x, int *y, double *scale, char *filename)
   {
   GdkPixbuf *pixbuf;
   
   pixbuf = gdk_pixbuf_new_from_file (filename);
   
   if (!pixbuf) 
      {
      fprintf (stderr, "Unable to load file: %s", filename);
      return;
      }
   
   gnome_print_gsave((void *)*printer);
   gnome_print_translate ((void *)*printer, *x, *y);
   gnome_print_scale ((void *)*printer, 
		      *scale * gdk_pixbuf_get_width (pixbuf), 
		      *scale * gdk_pixbuf_get_height (pixbuf));
   gnome_print_pixbuf ((void *)*printer, pixbuf);    
   gnome_print_grestore((void *)*printer);
   
   gdk_pixbuf_unref (pixbuf); 
   }

void print_newpage (int *printer)
   {
   char pagename[10];
   print_current_page++;
   
   sprintf (pagename, "%-9d", print_current_page);
   
   gnome_print_beginpage ((void *)*printer, pagename);
   }

void print_showpage (int *printer)
   {
   gnome_print_showpage ((void *)*printer);
   }

void print_newpath (int *printer)
   {
   gnome_print_newpath ((void*)*printer);
   }

void print_moveto (int *printer, int *x, int *y)
   {
   gnome_print_moveto ((void*)*printer, *x, *y);
   }

void print_lineto (int *printer, int *x, int *y)
   {
   gnome_print_lineto ((void *)*printer, *x, *y);
   }

void print_box (int *printerp, int *xp, int *yp, int *widthp, int *heightp,
		int *fill)
   {
   int x = *xp, y = *yp, width = *widthp, height = *heightp;
   GnomePrintContext *printer = (void *)*printerp;
   
   gnome_print_newpath (printer);
   gnome_print_moveto (printer, x, y);
   gnome_print_lineto (printer, x + width, y);
   gnome_print_lineto (printer, x + width, y - height);
   gnome_print_lineto (printer, x, y - height);
   gnome_print_lineto (printer, x, y);
   
   if (*fill)
      gnome_print_fill (printer);
   
   gnome_print_stroke (printer);
   }

void print_stroke (int *printer)
   {
   gnome_print_stroke ((void *) *printer);
   }

void print_setlinewidth (int *printer, double *width)
   {
   gnome_print_setlinewidth ((void *)*printer, *width);
   }

/* This is a nasty function which is called every now and then to see if the
 * window is closed, so it can close the application. There should be a way 
 * to do it using signals, but I couldn't find it. I'm sure its really 
 * easy, so someone else should find it and fix this :)
 */ 

static gboolean preview_close_check (gpointer data)
   {
#warning "data is a junk pointer, so this works. BUT IT IS SOOO BAD"   
   if (!GTK_IS_WIDGET (data))
      {
      gtk_main_quit ();
      return FALSE;
      }

   return TRUE;   
   }


void print_stop (int *printer)
   {
   GnomePrintContext *context;
   GnomePrintMasterPreview *preview;

   context = (void*)*printer;

   if (is_preview)
      {
      gnome_print_master_close (gpm);
      preview = gnome_print_master_preview_new (gpm, "Preview");

      g_timeout_add (400, preview_close_check, preview);
      
      gtk_widget_show_all (GTK_WIDGET (preview));
      gtk_main ();
      }
   
   gnome_print_context_close (context);   
   }

int print_init (void)
   {
   char *arg="";
   
   gnome_init ("bond-report",
	       "0.0.1",
	       1,
	       &arg);
      
   return 0;
   }

int print_cleanup (void)
   {
   if (gpm)
      gnome_print_master_close (gpm);
   return 0;
   }

