/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <glib/gconvert.h>

#include "ui.h"
#include "db.h"
#include "bc.h"

#include "bond.h"
#include "bondmain.h"
#include "biglists.h"
#include "watchtower.h"

/* get field and table list used in the columns */
/* The field list is twice as long as the number of fields requested.
 * It's first column is the field, second column is the table it occures
 * and so on. It contents are stored as strings */
static gint
biglist_getfields(BondObject * parentbobj, gchar *** fieldlist)
	{
	BondObject *bobj;
	GList *walk;
	gchar *parenttable;
	gint numfield = 0;

	(*fieldlist) = NULL;
	if (parentbobj->obj)
		parenttable = parentbobj->obj->basetable;
	else
		parenttable = NULL;
	for (walk = g_list_first(parentbobj->children); walk != NULL; 
			walk = walk->next)
		{
		bobj = walk->data;
		g_assert(bobj->oprnode);

		(*fieldlist) = mem_realloc((*fieldlist), sizeof(void *) * 
					   (numfield + 3));
		((*fieldlist)[numfield]) = NULL;
		((*fieldlist)[numfield + 1]) = NULL;

		if (bobj->oprnode->field)
			{
			((*fieldlist)[numfield]) = bobj->oprnode->field;
			if (bobj->oprnode->table)
				((*fieldlist)[numfield + 1]) = 
					bobj->oprnode->table;
			else
				((*fieldlist)[numfield + 1]) = parenttable;
			((*fieldlist)[numfield + 2]) = (gchar*) bobj;
			/* m.essage("Looking for %s.%s",
			 ((*fieldlist)[numfield + 1]) ,
			 ((*fieldlist)[numfield]) ); */
			}
		numfield = numfield + 3;
		}
	return numfield / 3;
	}

gulong
biglist_gensecretid(Object * obj)
	{
	return obj->row;
	}

static gint
biglist_fill(BondObject * bobj, gchar ** fieldlist, gint numfield, 
		gint * rowselect)
	{
	static gchar blankstr[] = "";	/* Blank entry for list */
	gint i, j;
	gint numitems, oldpos;
	gint rowcount, colcount, x, y;
	gint jlimit;
	gulong secretid;
	gsize tmpa, tmpb;
	GList *vallist = NULL;
	GList *glistfield = NULL;
	BondObject *columnbobj;
	Registration *reg;
	void **resultarray;			  /* Returned array */

	/* Plugin function - its cached to save finding it each time. */
	typedef void (*function) (UiXML * uixml, UiWidget *, GList *, gulong,
			gboolean);
	UiXML *uixml;
	PluginFunction *f;
	function func;

	g_assert(bobj->obj);
	*rowselect = -1;
	uixml = bobj->bond->uixml;
	numitems = db_numrecord(bobj->obj);
	oldpos = db_currentrow(bobj->obj);
	/* remove old registration */
	registration_remove_bybobj(bobj);
	f = plugin_findfunction(uixml, "list_append");
	func = f->ptr;
	/* Build a list of what is to be sent to the cache_extract. */
	for (i = 0; i < (numfield * 3); i += 3)
		if (fieldlist[i])
			{
			/* m.essage("Add %s.%s",fieldlist[i+1],fieldlist[i]); */
			glistfield = g_list_append(glistfield, fieldlist[i]);
			glistfield = g_list_append(glistfield, 
					fieldlist[i + 1]);
			}
	/* this code was the faster cooler version. Unforuntly it is sloower 
	 * and less cool I still dont understand why it is slower .??? I 
	 * should look in detail at it at some time. */
	if (glistfield)
		resultarray = db_cache_extract(bobj->obj, glistfield, 
				&rowcount, &colcount);
	else
		{
		errormsg("No column fields are specified on grid %s. Aborting "
			"fill.",bobj->widgetname);
		return -1;
		}
	
	y = rowcount * colcount;
	jlimit = (numfield * 3) - 3;
	/* m.essage("Draw: %dx%d items found %d len %d nofield %d c", rowcount,
	 colcount, g_list_length(glistfield), numfield, jlimit);  */
	if (colcount > 0)
		{
		x = colcount;
		/* Check if there isn't a dead field there */
		/* jlimit is the number of fields*/
		for (j = jlimit; j >= 0; j -= 3)
			{
			if (fieldlist[j] == NULL)
				{
				vallist = g_list_prepend(vallist, blankstr);
				continue;
				}
			x--;
			if (x < 0)
				{
				errormsg("Unable able to extract a field, will"
					" abort biglist draw.");
				g_assert(NULL);
				return -1;
				}
			}
		/* Now the actual work. Go though each row adding content */
		for (i = rowcount - 1; i >= 0; i--)
			{
			vallist = NULL;
			x = colcount;
			y = y - colcount;
			/* Add each column in the row */
			for (j = jlimit; j >= 0; j -= 3)
				{
				if (fieldlist[j] == NULL)
					{
					vallist = g_list_prepend(vallist, NULL);
					continue;
					}
				x--;
				/* m.essage("y:%d x: %d - %s %s.%s",
				 y/colcount,x,resultarray[y+x], fieldlist[j+1],
				 fieldlist[j]); */
				/* quickly addes a registration entry for this 
				 * record. */
				db_moveforce(bobj->obj,i);
				columnbobj = (BondObject *)fieldlist[j + 2];
				reg = registration_add(columnbobj, bobj->obj, 
					fieldlist[j + 1], fieldlist[j], TRUE);
				columnbobj->registrationlist = g_list_prepend(
					columnbobj->registrationlist, reg);
				/* This is possible MEMORY LEAK with utf8!? */
				/* If only it was better documented in gnome 
				 * so i knew if i am to free or not */
				if (resultarray[y + x])
					vallist = g_list_prepend(vallist, 
					g_locale_to_utf8(resultarray[y + x], 
					-1, &tmpa, &tmpb, NULL));
				else
					vallist = g_list_prepend(vallist, 
					resultarray[y + x]);
				}
			secretid = biglist_gensecretid(bobj->obj);
			if (i == oldpos)
				*rowselect = secretid;
			/* add to the list */
			func(uixml, bobj->widget, vallist, i, FALSE);

			if (vallist != NULL)
				g_list_free(vallist);
			}
		}
	if (resultarray)
		mem_free(resultarray);
	if (numitems > 0)
		db_moveto(bobj->obj, oldpos);
	return 0;
	}

/**
 * biglist_draw:
 * bobj: Bond object to draw
 * 
 * 
 * Returns: non-zero on error.
 */
gint
biglist_draw(BondObject * bobj, gboolean literefresh)
	{
	gint numfield = 0, rowselect = -1;
	gchar **fieldlist = NULL;

	g_assert(bobj->widget);
	if (bobj->obj == NULL)
		{
		ui_list_clear(bobj->bond->uixml, bobj->widget);
		return -1;
		}

	if (literefresh == FALSE)
		{
		numfield = biglist_getfields(bobj, &fieldlist);
		ui_list_clear(bobj->bond->uixml, bobj->widget);
		biglist_fill(bobj, fieldlist, numfield, &rowselect);
		}
	else
		rowselect = biglist_gensecretid(bobj->obj);

	if (rowselect != -1)
		ui_list_setselection(bobj->bond->uixml, bobj->widget, 
				rowselect); 

	if (fieldlist)
		mem_free(fieldlist);
	return 0;
	}

gint
biglist_selectrow(BondObject * bobj)
	{
	static gulong lastsecretid;
	gulong secretid;
	GList *a;

	g_assert(bobj);
	secretid = ui_list_getselection(bobj->bond->uixml, bobj->widget, 
			TRUE, &a);
	if (secretid < 0)
		{
		errormsg("Invalid row position of %d", secretid);
		return -1;
		}
	db_moveto(bobj->obj, secretid);
	/* for (;a!=NULL;a=a->next) message(": %s",a->data); */
	debuglog(80, "Moving to position %d in biglist %s.", secretid, 
			bobj->widgetname);
	
	return 0;
	}

/**
 * biglist_updatecell:
 * @widget: clist widget to update
 * @value:
 * 
 * Something in my eye. Anyway this will update a specific cell. This is
 * for another field is edited, and it will update whats in the list. 
 * 
 * Returns: non-zero on error.
 */
gint
biglist_updatecell(BondObject * bobj, gulong secretid, gchar * table, 
		gchar * field, gchar * value)
	{
	GList *walk;
	gint fieldpos = -1, counter = 0;

	for (walk = g_list_first(bobj->parent->children); walk != NULL;
			walk = walk->next)
		{
		if (walk->data == bobj)
			{
			fieldpos = counter;
			break;
			}
		counter++;
		}
	g_assert(bobj);
	g_assert(bobj->parent);
	if (fieldpos == -1)
		{
		errormsg("Can't find field.");
		return -1;
		}
	/* m.essage("Setting %s to %s pos %dx%d",bobj->parent->widgetname,
			value,fieldpos,secretid); */
	return ui_list_setcellvalue(bobj->bond->uixml,
			bobj->parent->widget, fieldpos, secretid, value);
	}

static BondObject * 
biglist_savevalue(BondObject *bobj, glong rowid, guint colnumber, gchar *value)
	{
	gint colcount = 1, oldrow=-2;
	gboolean found = FALSE;
	GList *walk;
	BondObject *childbobj;

	for (walk = g_list_first(bobj->children);walk!=NULL;walk=walk->next)
		{
		childbobj = walk->data;
		if (childbobj->oprnode->widgettype == UiGridColumn)
			{
			if (colcount == colnumber)
				{
				/* Found the right column */
				found = TRUE;
				break;
				}
			colcount ++;
			}
		}
	/* If dont find anything leave */
	if (found == FALSE)
		return NULL;
	if (childbobj->obj)
		{
		oldrow = db_currentrow(childbobj->obj);
		db_moveto(childbobj->obj,rowid);
		}
	bond_save_writedb(childbobj,value,TRUE);
	/* m.essage("record has been saved!"); */
	/* move back to old row if needed - this shouldnt normally happen */
	if (oldrow != -2 && oldrow != rowid)
		db_moveto(childbobj->obj,oldrow);
	return childbobj;
	}	
/* This code is for when a cell is edited. It is called remotely by uiplugin 
 * code. */
void
biglist_sig_gridedited(UiXML *uixml, UiWidget *widget, 
		glong rowid, guint colnumber, 
		gchar *newtext)
	{
	Bond *bond;
	BondObject *bobj, *childbobj;
	gchar *widgetname;
	
	g_assert(uixml);	
	bond = uixml->parent;
	
	widgetname = ui_getname(uixml,widget);
	if (widgetname == NULL)
		{
		errormsg("Unable to find a widgetname for this cell editing "
			"event. This isn't normal");
		return;
		}
	/* m.essage("render: THis cell has been edited. %d, %d, %s\n%s",
		rowid,colnumber,newtext,widgetname); */
	
	bobj = bond_getbyname(bond,widgetname);
	if (bobj == NULL)
		{
		errormsg("Unable to find a bondobject associated with "
			"%s.",widgetname);
		return;
		}
	/* m.essage("Bobj is %s",bobj->widgetname); */
	if ((childbobj = biglist_savevalue(bobj, rowid, colnumber, newtext)))
		bond_setchanged(childbobj,TRUE,newtext);
	else
		warningmsg("Unable to find the column in order to record "
			"change on bobj %s",bobj->widgetname);
	}


