/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef bond_h
#define bond_h

#include <glib.h>

typedef struct _BondObject BondObject;
typedef struct _SignalId SignalId;

#include "db.h"
#include "ui.h"
#include "bc.h"

#include "bondxml.h"
#include "bondrelation.h"
#include "bondmain.h"
#include "postit.h"
#include "uicommonplugin.h"


struct _SignalId
	{
	gint signalid;
	void *ptrindex;
	};

/*
 * A bond representation for a widget.  It could be any type of widget,
 * like a simple entry box or a complex window containing many other widgets.
 */
struct _BondObject
	{
	Bond *bond;	/* Global bond object that stores all BondObject */
	/* general information */
	/* =============================================== */
	/* widget information */
	gchar *widgetname; /* full name of widget. ie window3/box/entry23 */
	UiWidget *widget; /* generic pointer to the widget. This could be a GtkWidget depending on user interface */
	UiWidgetType widgettype; /* type of widget it is, ie window/entry/list */
	UiSignalKey *uisignalkey;	/* If key signals are attached to the widget they are here */
	GList *signallist;
	/* relationships to other bondobjects */
	GList *dependon;	/* who we depend on if our values change */
	GList *dependents;	/* bond objects that are dependent on us */
	gboolean ready;		/* are we currently painting this object? */
	gboolean changed;	/* have we changed? */
	gboolean defaultsset; 	/* have the default values been applied */
	gboolean editable; 	/* can you edit this widget?? */
	gboolean refreshneeded; /* do i need to refresh this - NOT USED */
	gboolean dbobjactive; 	/* is there a datasource? */
	gboolean fieldactive;	/* if field information is present */
	gboolean active;	/* if field isn't active, but active component anyway */
	gboolean signalsset;	/* if signals have been set on the object */
	gboolean cousinsset;	/* if we have found our cousins */
	gboolean beenwatched;	/* if this bond object is being watched by another bond object */
	
	/* alias widget specific information */
	/* =============================================== */
	OperationsNode *oprnode;/* database information & reference back to original node for which we were born */
	GList *functions;	/* information about method calls and where i get my values from */
	GList *cousins;		/* where do i share my values with when i change? */

	/* dataroof specific information */
	/* =============================================== */
	DbObject *obj;	/* Database object. This is NULL for most widgets */
	BondRelation *brel; 	/* how we came about existing in the data sense */
	BondObject *parent;	/* parent is a bond object who we base ourselves on */
	BondObject *dbobjectsrc;/* dbobjectsrc parent */

	/* if this is for the lookup tag, you need to store a hash list of primary keys */
	/* =============================================== */
	GHashTable *primarylookupkey;
	GHashTable *primarylookupvalue;
	GHashTable *primarylookuprowindex;
	GList *children;	/* children are other bond objects that are based on us */
	GList *registrationlist;/* A list of all the registrations this object has made. A key is used here for look ups */
	GList *watchtowerlist;	/* list of watchtowers we currently have */
	MemGarbage *garbage;	/* List of items to free. */
	};

gint bond_inithash(Bond *bond);
void bond_cleanuphash();
BondObject *bond_create(Bond *bond, OperationsNode * oprnode);
BondObject *bond_getbyname(Bond *bond, gchar * widgetname);
BondObject *bond_getbydbobjectsrc(Bond * bond, gchar * dbobjectsrc);
void bond_free(void *bobj);

void bond_show(BondObject *bobj, gboolean recusiveinprogress);
gint bond_close(BondObject *bobj, gboolean save);
gint bond_save_writedb(BondObject * bobj, gchar * value, gboolean mark);
gint bond_save(BondObject * bobj, gboolean mark, gboolean recusive);
gint bond_refresh(BondObject * bobj);
gint bond_paint(BondObject * bobj, gboolean literefresh, gboolean recusive);
void bond_paint_setui(BondObject * bobj, gchar * value, gboolean literefresh);
void bond_repaint_bychange(BondObject * bobj, BondObject *paintby, gulong row, 
		gchar *tablename, gchar *fieldname, gchar *newtext);
void bond_save_getui(BondObject *bobj, gchar **value); /* Low level support functions */
gboolean bond_isparentobj(BondObject *bobj);
gboolean bond_isfieldactive(BondObject * bobj);
gboolean bond_ischanged(BondObject * bobj);
gint bond_setchanged(BondObject * bobj, gboolean changed, gchar *newtext);
BondObject *bond_spawn(Bond *bond, BondObject * parentbobj, gchar * widgetname, BondRelation * brel, gboolean spawnnew);
	


gint bond_registerobj(BondObject *bobj);
gint bond_registerobjchange(BondObject *parentbobj, gint (*objchange) (BondObject *),gboolean literefresh);
gint bond_unregisterobj(BondObject * bobj);

/* debuging and testing */
gint bond_test(Bond *bond, gint testnum);
void bond_debug(BondObject * bobj);

#endif
