/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "bc.h"
#include "db.h"

#include "bond.h"
#include "fieldpath.h"
#include "bondargument.h"
#include "watchtower.h"

/** on second element, take c->column[0] and c->table[0], take obj from first
 * element and generate a key.  Create watch on key to internal function. This
 * function will eventually rerun trigger.  When watch picks up something, you
 * first of all read its value, and store it in livevalue. it goes in the second
 * element. The bobj is added to its glist of bobjs. A callback is set on this
 * bobj so that whenever its value changes it updates the argument livevalue.
 * The database value (dbvalue) is set anyway, this is a just a
 * db_getvalue(first->obj,c->column[0],c->table[0]).  The first element will
 * have dbvalue and livevalue set by its ui_getvalue if its bondobject is
 * visible, and db_getvalue(first->obj,first->field,first->table);
 * 
 * When it comes to third element; you have to work out if the previous element
 * and if the livevalue is set.  If the livevalue is set you take the same
 * approach, dcreate a watch etc. If its not live do a
 * db_getvalue(prev->obj,c->column[0],c->table[0]); bobjs are in a list, so you
 * always take the first value if it exists.  When no bobj's exist you have to
 * create an object anyway. SELECT field,next->c->column[0] FROM table WHERE
 * field=value (livevalue if exist else dbvalue).
 *
 * Watchs need to be destroyed and recreated everytime the argument is
 * requested. and they need to active on changes carried out by there keys.
 *
 * To generate the key you need arg->obj and arg->path->tablename. But you need
 * the key to get the dbobj. Oh what a visous circle.
 *
 * Last item, you need 2 watchs because you watch both the linkvalue and the
 * argument value. or you create the watchs in the item before hand.
 *
 */

  
static BondArgument*
bondargument_create_argument(FieldPath *path)
	{
	BondArgument *arg;
	arg = mem_calloc(sizeof(BondArgument));
	arg->path = path;
	arg->initial = TRUE;
	return arg;
	}

static gchar *
bondargument_init_getkey(BondArgument *arg)
	{
	gchar *key;
	
	key = registration_getkey(arg->prev->obj, arg->path->c->table[0], 
		arg->path->c->column[0]);

	return key;
	}

static void
bondargument_free_watch(BondArgument *arg)
	{
	watchtower_unregister(arg->parentbobj);
	}

static GList *
bondargument_init_watch(BondArgument *arg)
	{
	WatchTower *watch;
	GList *retlist = NULL;
	
	g_assert(arg->key);
	
	watch = watchtower_create(arg->parentbobj,bondargument_watchrun);
	watch->key = arg->key;
	watch->ptr[0] = arg;
	retlist = watchtower_register(watch);
	
	return retlist;
	}

static Object *
bondargument_init_generateobj(BondArgument *arg)
	{
	Object *obj;
	gchar *value, *query;
	if (arg->livevalue)
		value = arg->livevalue;
	else if (arg->dbvalue)
		value = arg->dbvalue;
	else
		{
		errormsg("No livevalue or dbvalue exists on argument %s.%s so i can not proceed to get its arguments",arg->path->tablename,arg->path->fieldname);
		return NULL;
		}
	query = mem_strdup_printf("SELECT * FROM %s WHERE %s=%s",arg->path->tablename, 
		arg->path->fieldname,value);
	obj = db_sql(NULL,query);
	mem_free(query);
	return obj;
	}

/* Find key, livevalue and dbvalue */
static gint
bondargument_init_argumentkey(BondArgument *arg)
	{
	if (arg->key)
		mem_free(arg->key);
	/* first item in list doens't need anything done */
	if (arg->first == arg)
		return 0;
	/* get key */
	arg->key = bondargument_init_getkey(arg);
	
	/* register watchtower to get live value */
	arg->bobjsmatchkey = bondargument_init_watch(arg);
	
	/* get dbvalue */
	db_getvalue(arg->prev->obj,arg->path->c->column[0],arg->path->c->table[0],&arg->dbvalue);
	
	/* set database object to any bobjs that match the key */
	if (arg->bobjsmatchkey)
		arg->obj = ((BondObject*)(arg->bobjsmatchkey->data))->obj; 
	/* get database object based on this table if one doesn't exist. */
	if (!arg->obj)
		arg->obj = bondargument_init_generateobj(arg);
	if (arg->obj && arg->dbvalue && arg->key)
		return 0;
	return -1;
	}

static GList *
bondargument_init_argumentfrompath(BondObject *parentbobj, GList *pathlist)
	{
	GList *walk, *retlist = NULL;
	FieldPath *path;
	BondArgument *arg, *prev = NULL, *first = NULL;
	
	for (walk=g_list_first(pathlist);walk!=NULL;walk=walk->next)
		{
		path = walk->data;
		arg = bondargument_create_argument(path);
		if (prev)
			prev->next = arg;
		if (!first)
			{
			first = arg;
			arg->parentbobj = parentbobj;
			arg->bobjsmatchkey = g_list_prepend(arg->bobjsmatchkey,parentbobj);
			}
		arg->first = first;
		arg->prev = prev;
		arg->parentbobj = parentbobj;
		if (bondargument_init_argumentkey(arg))
			{
			bondargument_free(retlist);
			return NULL;
			}
		prev = arg;
		retlist = g_list_append(retlist,arg);
		}
	/* Mark in the arguments that we have completed the task */
	for (walk=g_list_first(retlist);walk!=NULL;walk=walk->next)
		{
		arg = walk->data;
		arg->initial = FALSE;
		}
	return retlist;
	}

void
bondargument_free(GList *arglist)
	{
	BondArgument *arg;
	GList *walk;
	for (walk=g_list_first(arglist);walk!=NULL;walk=walk->next)
		{
		arg = walk->data;
		if (arg->bobjsmatchkey)
			g_list_free(arg->bobjsmatchkey);
		mem_free(arg);
		}
	g_list_free(arglist);
	}

gint 
bondargument_init_field(BondObject *bobj)
	{
	g_assert(bobj);
	
	return 0;
	}


void 
bondargument_watchrun(void *ptr1, void *ptr2)
	{
	BondArgument *arg;
	BondObject *bobj;
	WatchTower *watch;
	watch = ptr1;
	bobj = ptr2;
	arg = watch->ptr[0];
	
	bond_save_getui(bobj, &arg->livevalue);
	/* If called while creating watch, then just grab value */
	message("have change on %s effecting us",bobj->widgetname);
	if (arg->initial == TRUE)
		return;
	/* else now we have to rebuild the whole watch keys from this point on */
	message("Not in inital stage so rebuilding argument list");
	}
