/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef bondrelation_h
#define bondrelation_h

typedef struct _BondRelation BondRelation;

typedef enum
{
    RelationNone,
    RelationAdd,
    RelationLoadAll,
    RelationSQL,
    RelationAuto
}
RelationType;

#include "db.h"
#include "bond.h"
#include "bondxml.h"

struct _BondRelation
	{
	/* Bond object this is attached to. Can be NULL. This is who i am owned by. */
	BondObject *bobj;
	/* if the Object* defined in the structure really just a clone of someone elses parent */
	gboolean clonedobj;
	/* to inform if i save in this object or save in its parent (normally parent) */
	gboolean savetoparent;
	/* do i run this immediately */
	gboolean dontrunyet;
	gboolean rerun;
	/* Node information from which we were born */
	OperationsNodeDb *dbobj;
	/* How the object is loaded */
	RelationType type;
	/* If SQL statement attached to it */
	gchar *sql;
	/* when do i free myself? */
	gint refcounter;
	/* Database object we filter by if need be (a list of dbobj) */
	GList *parentobj;
	gboolean parentobjloaded;
	/* for been stored in repository - these are strings */
	gchar *targetwidget;
	GList *loadbywidget;
	};

BondRelation *bondrelation_create(RelationType type);
gint bondrelation_free(BondRelation * brel);
DbObject *bondrelation_load(Bond * bond, DbObject * obj, BondRelation * brel);
gint bondrelation_addpending_brel(Bond *bond, BondRelation *brel);
gint bondrelation_addpending(Bond *bond, gchar *widgetname, gchar *loadby);
BondRelation * bondrelation_checkpending(Bond *bond, gchar *widgetname, BondRelation *orginalbrel);
gint bondrelation_anaylise(BondObject * bobj);
DbObject *bondrelation_findobjfromobjsrc(Bond * bond, gchar * dbobjectsrc);
gint bondrelation_refresh(BondObject * bobj);
gint bondrelation_saveeffected(Bond *bond, GList *objlist);

#endif
