/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>

#include "bc.h"

#include "bond.h"
#include "fieldpath.h"
#include "livepath.h"
#include "watchtower.h"
#include "bondapi.h"

/*static gint
livepath_check_visible
*/

static gint
livepath_comparekeys(gchar * nextkey, gchar * prevkey)
	{
	return 0;
	}

LivePath *
livepath_create(Bond * bond, GList *fieldpathlist)
	{
	GList *walk;
	LivePath *headpath = NULL, *spawn, *prev = NULL;

	g_assert(fieldpathlist);
	/* creates a link list from a fieldpathlist. */
	for (walk = g_list_first(fieldpathlist); walk != NULL; walk = walk->next)
		{
		spawn = mem_calloc(sizeof(LivePath));
		spawn->fieldpath = walk->data;
		if (headpath == NULL)
			headpath = spawn;
		else
			{
			/* Set previous and next values in linked list */
			prev->next = spawn;
			spawn->prev = prev;
			}
		spawn->parent = headpath;
		prev = spawn;
		}
	return headpath;
	}

static GList *
livepath_evaulate_getbobjlist_link(Bond * bond, FieldPath * fieldpath)
	{
	gchar *fieldkey;
	GList *retlist;

	g_assert(fieldpath);
	g_assert(fieldpath->c);
	
	fieldkey = mem_strdup_printf("%s.%s", fieldpath->c->table[0], fieldpath->c->column[0]);
	retlist = g_hash_table_lookup(bond->globalreg->reghashbyfield, fieldkey);
	mem_free(fieldkey);

	return retlist;
	}

static GList *
livepath_evaulate_getbobjlist(Bond * bond, FieldPath * fieldpath)
	{
	gchar *fieldkey;
	GList *retlist;

	g_assert(fieldpath);

	fieldkey = mem_strdup_printf("%s.%s", fieldpath->tablename, fieldpath->fieldname);
	retlist = g_hash_table_lookup(bond->globalreg->reghashbyfield, fieldkey);
	mem_free(fieldkey);

	return retlist;
	}

/* static gint
livepath_evaulate_checklink(Bond *bond, LivePath *livepath, BondObject *prev)
	{
	return 0;
	} */

static void 
livepath_evaulate_reset(LivePath *current)
	{
	current->present = FALSE;
	current->bobj = NULL;
	current->linkvalue = NULL;
	current->bobjlink = NULL;
	}

/**
 * livepath_evaulate:
 * @bond: Global bond object.
 * @livepath: Livepath to work with.
 * @from: BondObject to start from.
 * 
 * 
 * 
 * Returns: Negitive on error. 0 on success, 1 if a clear live path doesn't exist.
 */

gint
livepath_evaulate(Bond * bond, LivePath * livepath, BondObject * from)
	{
	GList *walk, *bobjlist;
	gchar *nextkey, *prevkey;
	LivePath *current;
	BondObject *bobj;
	
	/* Error checking. */
	g_assert(bond);
	g_assert(livepath);
	g_assert(from);
	if (!from->obj)
		{
		errormsg("Bond object %s is missing dbobject or dbobjectsrc. Can not" "run function.", from->widgetname);
		return -1;
		}
	if (!from->oprnode->table || !from->oprnode->field)
		{
		errormsg("A tablename and fieldname need to be specified on %s for "
			"it to be used as an argument to a function.", from->widgetname);
		return -2;
		}
	
	/* Initilisation */
	/* Get the existing key from @from. */
	prevkey = registration_getkey(from->obj, 
		from->oprnode->table, from->oprnode->field);

	for (current = livepath->parent; current != NULL; current = current->next)
		{
		/* The for loop gets the next current link in livepath. 
		 * current is then reset back to its bondobject non-existanst state.
		 */
		livepath_evaulate_reset(current);
		/* ================================================================== */
		/* Gets the link table.field. Check if any bond objects for it exist. 
		 * It is the prev item in fieldpath, the fieldpath->c[0] table.field. 
		 *
		 * This works on getting the get ui value. This is the live follow the pathconf
		 * values.
		 */
		bobjlist = livepath_evaulate_getbobjlist_link(bond, current->fieldpath);
		for (walk = bobjlist; walk != NULL; walk = walk->next)
			{
			bobj = walk->data;
			if (!bobj->obj || !bobj->oprnode->table || !bobj->oprnode->field)
				continue;
			/* Get the link value. */
			bond_save_getui(bobj, &current->linkvalue);
			if (!current->linkvalue)
				{
				/* If link value doesn't extract properly continue the search */
				warningmsg("Unable to extract a widget value for bond object %s",
					bobj->widgetname);
				continue;
				}
			/* Only grab the first link bond object. */
			current->bobjlink = bobj;
			break;
			}

		/* ================================================================== */
		/* Get any bondobject pointing to the same table->field.  */
		bobjlist = livepath_evaulate_getbobjlist(bond, current->fieldpath);
		for (walk = bobjlist; walk != NULL; walk = walk->next)
			{
			bobj = walk->data;
			if (!bobj->obj || !bobj->oprnode->table || !bobj->oprnode->field)
				continue;
			/* Get the next values key */
			nextkey = registration_getkey(bobj->obj, bobj->oprnode->table, bobj->oprnode->field);

			/* Compare the keys to see if they can be linked */
			if (livepath_comparekeys(nextkey, prevkey))
				{
				
				
				}
			else
				continue;
			}
		/* ================================================================== */
		/* Have been unable to find a bond object open. Aborting search */
		if (walk == NULL)
			{
			/* No match to continue; Reset all next livepath's and return 1*/
			for (;current!=NULL;current=current->next)
				{
				/* Reseting the trailing items in livepath link list */
				livepath_evaulate_reset(current);
				}
			return 1;
			}
		
		/* ================================================================== */
		/* If can be linked, mark livepath->bond . Store link widget if there is 
		 * one 
		 */
		current->present = TRUE;
		current->bobj = bobj;
		}
	return 0;
	}
/**
 * livepath_free:
 * @livepath: Live path.
 *
 * Frees all memory created by livepath_create().
 */
void
livepath_free(LivePath *livepath)
	{
	LivePath *walk;
	for (walk = livepath->parent; walk != NULL; walk = walk->next)
		{
		
		mem_free(walk);
		}	
	}
	
gint
livepath_test(gint testnum, Bond *bond)
	{
	gint retval = 0;
	BondObject *from;
	LivePath *live;
	GList *fieldpathlist;
	
	bond_api_open(bond,"mainwindow");
	fieldpathlist = fieldpath_init("soldier", "kills", "rank");
	fieldpath_debug(fieldpathlist);

	from = bond_getbyname(bond, "kills");
	
	live = livepath_create(bond, fieldpathlist); 
	livepath_evaulate(bond, live, from);

	livepath_free(live);
	fieldpath_free(fieldpathlist);

	return retval;
	}
