/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <glib.h>

#include "bc.h"
#include "db.h"
#include "sql_parser.h"

#include "search.h"
#include "bond.h"
#include "bondxml.h"

/**
 * search_addslashes:
 * @value: Value to be modified. Must be allocated with mem_ functions.
 *
 * puts / infront of all ''s and /'s in front of all /'s. 
 * It will free @value and allocate a new variable.
 */
gint
search_addslashes(gchar **value)
        {
        gint i, slen, j;
        gchar *retstr;

        if (*value == NULL)
                return -1;
        slen = strlen(*value);
        if (slen == 0)
                return -2;
        retstr = (char *)mem_calloc((slen * 2) + 4);
		
		j = 0;
        for (i = 0; i < slen; i++)
                {
                if (((*value)[i]) == 39)                         /* ' */
                        {
                        retstr[j] = 92;
                        j++;
                        }
                else if (((*value)[i]) == 92)            /* \ ... what an evil                 */
					/* with backslashes */
                        {
                        retstr[j] = 92;
                        j++;
                        }
                retstr[j] = (*value)[i];
                j++;
                }
		retstr[j] = 0;
        if (j + 1 > (slen * 2) + 3)
				{
                errormsg("Overflow occurred!!");
				return -1;
				}
        mem_free(*value);
        *value = retstr;
		
		return 0;
		}

static GList *
search_parse_checkfieldlist(GList * fieldlist, OperationsNodeSearch * searchnode)
	{
	OperationsNodeSearch *tmpsearch;
	GList *walk;

	/* list x2 array.  First item is the searchnode, the second item is a GList of search nodes used in or statement */
	for (walk = g_list_first(fieldlist); walk != NULL; walk = walk->next)
		{
		tmpsearch = walk->data;
		if (searchnode->widgetname && tmpsearch->widgetname)
			{
			/* m.essage("comparing %s to %s",searchnode->widgetname,tmpsearch->widgetname); */
			if (strcmp(searchnode->widgetname, tmpsearch->widgetname) == 0)
				return walk->next;
			}
		/* move to next tiem */
		walk = walk->next;
		}
	return NULL;
	}

static gchar *
search_parse_getvalue(Bond *bond, OperationsNodeSearch * search)
	{
	gchar *retstr = NULL, *value = NULL, *avalue;
	gchar *widgetname;
	BondObject *bobj;
	g_assert(search);
	if (search->widgetname)
		{
		widgetname = bondxml_getwidgetlongname(bond, search->widgetname);
		if (!widgetname)
			{
			warningmsg("Unable to find widget %s used in searching.",search->widgetname);
			return NULL;
			}
		bobj = bond_getbyname(bond,widgetname);
		if (bobj)
			bond_save_getui(bobj,&value);
		else
			ui_getvalue(bond->uixml,ui_getwidget(bond->uixml,widgetname), &value);
		}
	if (value && strlen(value) > 0)
		{
		/* Escape it to make it nice */
		avalue = mem_strdup(value);
		search_addslashes(&avalue);
		/* Add it now */
		if (search->opr == SQL_like)
			{
			retstr = mem_strdup_printf("'%c%s%c'",'%',avalue,'%');
			}
		else
			retstr = mem_strdup_printf("'%s'",avalue);
		
		mem_free(avalue);
		}
	return retstr;
	}
	
static gint
search_parse_modifysql(Bond *bond, sql_statement *statement, GList * content)
	{
	gchar *value, *field, *tmpstr;
	gint count = 0, casesensitive;
	OperationsNodeSearch *search;
	GList *walk;
	search = content->data;
	for (walk=content;walk!=NULL;walk=walk->next)
		{
		search = walk->data;
		casesensitive = 0;
		
		if ((value = search_parse_getvalue(bond, search)) == NULL)
			{
			/* m.essage("value is null on %s",search->widgetname); */
			continue;
			}
		
		if (search->casesensitive)
			field = search->field;
		else
			{
			field = mem_strdup_printf("lower(%s)",search->field);
			tmpstr = value;
			value = mem_strdup_printf("lower(%s)",tmpstr);
			mem_free(tmpstr);
			casesensitive = 1;
			}
		message("Opr :%d",search->opr);
		if (count >= 1)
			{
			message("modifiny sql to include OR %s = %s",field,value);
			/* Opening brackets for OR statement */
			sql_statement_append_where(statement,field,value,SQL_or,search->opr);
			}
		else
			{
			message("modifiny sql to include AND %s = %s",field,value);
			/* Opening brackets for OR statement */
			sql_statement_append_where(statement,field,value,SQL_and,search->opr);
			}
		/* if case senstiive free the allocated field */
		if (casesensitive)
			mem_free(field);
		/* count determines if its a and or and statement */
		count ++;
		mem_free(value);
		}
	return count;
	}

/**
 * search_parse:
 *
 * Activates the search request on a object.
 * 
 */
gint
search_parse(Bond *bond, BondRelation * brel, Object *obj)
	{
	gint count = 0;
	gchar *tmpstr;
	GList *walk;
	OperationsNodeSearch *searchnode, *n;
	GList *fieldlist = NULL, *content;
	sql_statement *statement;

	/* Constraints Checks */
	g_assert(brel);
	g_assert(obj);
	if (!brel->dbobj)
		{
		errormsg("This BondRelation does not contain a dbobject.");
		return -1;
		}
	debuglog(70,"Running search on %s",brel->dbobj->widgetname);
	if (!brel->dbobj->search)
		{
		warningmsg("No search requirements listed under dbobject %s", brel->dbobj->widgetname);
		return -2;
		}
	if (!obj->query)
		{
		errormsg("Evil BondObject has no query.");
		return -3;
		}

	
	/* Now this is a bit complicated. We create a list of twice the size as the orginal search node list which is
	   sorted. The list is organised in a AND type strucutre, so there will be one occurance of table.field in every
	   item of list, followed by a GList of all occurances of that table.field in the orginal searchnode. */
	for (walk = g_list_first(brel->dbobj->search); walk != NULL; walk = walk->next)
		{
		searchnode = walk->data;
		/* check if this is and or a or request */
		if ((content = search_parse_checkfieldlist(fieldlist, searchnode)) != NULL)
			{
			n = ((GList*)(content->data))->data;
			/* m.essage("Appending item %s to %s",searchnode->field,n->field); */
			((GList *) (content->data)) = g_list_append(((GList *) (content->data)), searchnode);
			}
		else
			{
			/* m.essage("Start item %s ",searchnode->field); */
			fieldlist = g_list_append(fieldlist, searchnode);
			fieldlist = g_list_append(fieldlist, g_list_append(NULL, searchnode));
			}
		}
	/* Modify the SQL statement */
	message("Orginal query is %s",obj->query);
	statement = sql_parse(obj->query);
	for (walk = g_list_first(fieldlist); walk != NULL; walk = walk->next)
		{
		searchnode = walk->data;
		debuglog(40,"Item %s found for search on %s", searchnode->widgetname, searchnode->field);
		
		count += search_parse_modifysql(bond, statement, walk->next->data);
		
		walk = walk->next;
		}
	/* if more than one item is selected then do the search */
	if (count > 0)
		{
		mem_free(obj->query);
		tmpstr = sql_stringify(statement);
		obj->query = mem_strdup(tmpstr);
		debuglog(70,"Generated a new SQL statement %s",tmpstr);
		g_free(tmpstr);
		sql_destroy(statement);
		}

	/* free glist and its little children */
	for (walk = g_list_first(fieldlist); walk != NULL; walk = walk->next)
		{
		walk = walk->next;
		g_list_free((GList *) (walk->data));
		}
	g_list_free(fieldlist);
	/* return 1 if empty set */
	if (count)
		return 0;
	return 1;
	}
