
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "db.h"
#include "bc.h"
#include "bond.h"
#include "search.h"

/* Files to run tests from */
#include "fieldpath.h"
#include "livepath.h"

/* stuff used for testing */
int
main(int argc, char *argv[])
	{
	Bond *bond;
	gchar *retstr, *towaitfor;
	gint test = 0, retval;
	gint pos;						  /* for dropdown boxes mostly */
	gint pass = 0;					  /* count tests passed */
	GList *walk, *list;
	Object *obj = NULL;
	UiWidget *widget = NULL;

	setbuf(stdout, (char *)0);

	bond_init(argc, argv, &bond);

	if (argc <= 1)
		{
		/* run your a specific test here on default */
		test = 0;
		}
	else
		test = atoi(argv[1]);
	if (argc >= 3)
		setdebuglinenum(1);
	if (test > 9)
		{
		debug_output("test not supported\n");
		return 0;
		}
	switch (test)
		{
	case 0:
		printf("bond test suite.\n"
		       "Developed by treshna Enterprises Ltd.\n\n"
		       "You need to supply a test number to run. ie testbond 1.\n"
		       "If you havn't added the test database you will need to do so\n"
		       "else all the tests will fail. Make sure you install the bonddb\n"
		       "soldier sql database.\n");
		break;
		/* check to see if parsing a form and assigning it to datasources
		   works fine */
	case 1:
			{
			/* bond_test(0); bond_test(1); bond_test(2); */
			bond_test(bond, 3);
			break;
			}
		/* check if searching works fine */
	case 2:
			{
			bond_test(bond, 4);
			break;
			}
	case 3:
			{
			bond_test(bond, 5);
			break;
			}
	case 4:
			{
			fieldpath_test(0);
			break;
			}

	case 5:
			{
			bond_evaluate_test(bond, NULL);
			break;
			}
	case 6:
			{
			bond_test(bond, 2);
			break;
			}
	case 7:
			{
			fieldpath_test(0);
			break;
			}
	case 8:
			{
			livepath_test(0, bond);
			break;
			}

#if 0
	case 2:
			{
			widget = ui_getwidget("namesearch");
			g_assert(widget);
			dataroof_specifyclass("soldiersearch", "soldier");
			dataroof_specifyclass("input", "soldier");
			debugmsg("Testing soldier search capibalities");
			ui_setvalue(widget, "Avis");
			obj = buildsearch("input");
			g_assert(obj);
			debugmsg("%d records retreived", obj->num);
			db_getvalue(obj, "name", NULL, &retstr);
			if (strcmp(retstr, "Avis") == 0)
				debugmsg("TEST PASSED");
			else
				debugmsg("TEST FAILED");
			db_freeobject(obj);

			break;
			}
		/* Drop down box testing */
	case 3:
			{
			// fr:dropdown_fb_set(BUFFER_BEGIN);
			Dropdown *dropdown;

			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "3");
			debugmsg("Testing drop down box");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			g_assert(roof);

			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);
			printf
			("======================= Building Dropdown Box ==============================\n");
			// fr:dropdown_displaydropdown("soldier.vbox2.vbox3.table2.rankid",
			// "soldier",&towaitfor);
			// fr:dropdown_fb_set(BUFFER_FLUSH);
			dropdown_new("soldier.vbox2.vbox3.table2.rankid", 	/* dropdown
			             																						   widget
			             																						   name */
			             "soldier", 		 /* formname */
			             &dropdown);
			dropdown_display(dropdown, FALSE	/* FALSE means no dependency
			                 																   with other */ );
			ui_getvalue(ui_dropdown_getentrybox
			            (ui_getwidget("soldier.vbox2.vbox3.table2.rankid")),
			            &retstr);
			debugmsg("Text in dropdown box entry box is %s", retstr);

			if (strcmp(retstr, "UiGeneral") != 0)
				errormsg("TEST FAILED");
			else
				debugmsg("TEST PASSED");
			break;
			}
	case 4:
			{
			Dropdown *dropdown;

			/* drop box test without fb_set support */
			// fr:dropdown_fb_set(BUFFER_FLUSH);
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "3");
			debugmsg("Testing drop down box");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			g_assert(roof);

			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);
			printf
			("======================= Building Dropdown Box ==============================\n");
			// fr:dropdown_displaydropdown("soldier.vbox2.vbox3.table2.rankid",
			// "soldier",&towaitfor);
			// fr:dropdown_fb_debug();
			dropdown_new("soldier.vbox2.vbox3.table2.rankid", "soldier",
			             &dropdown);
			dropdown_display(dropdown, NULL	/* NULL means no dependency
			                 																   with other */ );
			ui_getvalue(ui_dropdown_getentrybox
			            (ui_getwidget("soldier.vbox2.vbox3.table2.rankid")),
			            &retstr);
			debugmsg("Text in dropdown box entry box is %s", retstr);

			if (strcmp(retstr, "UiGeneral") != 0)
				errormsg("TEST FAILED");
			else
				debugmsg("TEST PASSED");
			break;
			}
	case 5:
			{
			// fr:dropdown_fb_set(BUFFER_BEGIN);
			Dropdown *dropdown;

			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "3");
			debugmsg("Testing drop down box, setting a value");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);
			printf
			("======================== Building Dropdown Box ==============================\n");
			// fr:dropdown_displaydropdown("soldier.vbox2.vbox3.table2.rankid",
			// "soldier",&towaitfor);
			// fr:dropdown_fb_set(BUFFER_FLUSH);
			dropdown_new("soldier.vbox2.vbox3.table2.rankid", "soldier",
			             &dropdown);
			dropdown_display(dropdown, NULL	/* NULL means no dependency
			                 																   with other */ );
			ui_setvalue(ui_dropdown_getentrybox
			            (ui_getwidget("soldier.vbox2.vbox3.table2.rankid")),
			            "Private");

			dropdown_readlookupvalue(dropdown);
			retstr = dropdown->lookupvalue;

			debugmsg("Text retrieved from query %s, %d", retstr,
			         dropdown_getselectedpos(dropdown));

			if (retstr == NULL || strcmp(retstr, "5") != 0)
				errormsg("TEST FAILED");
			else
				debugmsg("TEST PASSED");
			dataroof_cleanup();
			break;
			}
		/* Test cross depdenceses of drop down boxes */
	case 6:
			{
			gchar *str2;
			Dropdown *dropdown;

			// fr:dropdown_fb_set(BUFFER_BEGIN);
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "3");
			debugmsg("Testing drop down box, lots of dependent stuff");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);
			alias_debugdisplay(roof->alias);
			printf
			("======================== Building Dropdown Box ==============================\n");
			// fr:dropdown_displaydropdown("soldier.vbox2.vbox3.table2.rankid",
			// "soldier",&towaitfor);
			widget = ui_getwidget("soldier.vbox2.vbox3.table2.rankid");
			// fr:dropdown_fb_set(BUFFER_FLUSH);
			dropdown_new("soldier.vbox2.vbox3.table2.rankid", "soldier",
			             &dropdown);
			dropdown_display(dropdown, NULL	/* NULL means no dependency
			                 																   with other */ );
			ui_getvalue(ui_dropdown_getentrybox(widget), &retstr);
			debugmsg("%s appears in dropdown box", retstr);
			dropdown_readlookupvalue(dropdown);
			retstr = dropdown->lookupvalue;
			db_getvalue(obj, "soldier.vbox2.vbox3.table2.rankid", NULL,
			            &str2);
			debugmsg
			("Text in dropdown box is %s, pos is %d, should be value %s",
			 retstr, pos, str2);
			if (strcmp(retstr, str2) != 0)
				errormsg("TEST FAILED");
			else
				debugmsg("TEST PASSED");
			break;
			}
	case 7:
			{
			GList *walk2, *retlist;
			AliasDepend *d;

			obj = db_createobject("soldier");
			debugmsg("Testing if widget dependences are created ok");
			retlist =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);
			alias_debugdisplay(retlist);
			walk = retlist;
			while (walk != NULL)
				{
				if (walk->data != NULL)
					{
					walk2 = ((AliasDataSource *) (walk->data))->depend;
					debugmsg("Widget: %s",
					         ((AliasDataSource *) (walk->data))->
					         widgetshortname);
					while (walk2 != NULL)
						{
						d = walk2->data;
						debugmsg("Widget does depend on: %s",
						         d->dependwidgetlongname);
						walk2 = walk2->next;
						}
					}
				walk = walk->next;
				}
			alias_freedatasources(retlist);
			db_freeobject(obj);
			debugmsg("TEST PASSED");
			break;
			}
	case 8:
		debugmsg("Laughing Test	. HAHAHA");
		debugmsg("\n\n\n\nTEST PASSED");
		break;
	case 9:
			{
			gchar *comment = mem_alloc(15);
			gint type;
			gchar *table;
			gchar *field;

			strcpy(comment, "1;liam.name");
			debugmsg("Using string %s\n", comment);
			type = db_casual_type(comment);
			field = db_casual_field(comment);
			table = db_casual_table(comment);
			debugmsg("Got type: %d\nGot field: %s\nGot table: %s\n", type,
			         field, table);
			if (strcmp(field, "name") != 0 || strcmp(table, "liam") != 0
			        || type != 1)
				errormsg("TEST FAILED");
			else
				debugmsg("TEST PASSED");
			mem_free(comment);
			break;
			}
	case 10:
			{
			/* test to see if we break stuff on load all */
			bond_specifyclass("soldier", "soldier");
			bond_formloadall("soldier");
			debugmsg
			("===================================================================\n"
			 "I've finished loading the form");

			pass = debug_checksoldier_fornewdropdown();

			if (pass == 4)
				debugmsg("TEST PASSED");
			else
				{
				debugmsg("passed %d of 4", pass);
				errormsg("TEST FAILED");
				}
			break;
			}
	case 11:
			{
			/* Test loading of dynamic libraries */
			Plugin *plugin;

			plugin = plugin_create("uigtk", NULL);
			break;
			}

		/* No i'm really going to test dependences of dropdown boxes now
		   ... */
	case 12:
			{
			Dropdown *dropdown;

			// fr:dropdown_fb_set(BUFFER_BEGIN);
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "2");
			debugmsg("Testing drop down box, lots of dependent stuff");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);
			alias_debugdisplay(roof->alias);
			form_createsignals("soldier");
			printf
			("======================== Building Dropdown Box ==============================\n");
			// fr:dropdown_displaydropdown("soldier.vbox2.vbox3.table2.rankid",
			// "soldier",&towaitfor);
			// fr:dropdown_displaydropdown("soldier.vbox2.vbox3.table2.countryid",
			// "soldier",&towaitfor);
			// fr:dropdown_fb_set(BUFFER_FLUSH);
			dropdown_new("soldier.vbox2.vbox3.table2.countryid", "soldier",
			             &dropdown);
			g_assert(dropdowngroup_registertogroup(dropdown) == 0);

			dropdown_new("soldier.vbox2.vbox3.table2.rankid", "soldier",
			             &dropdown);
			g_assert(dropdowngroup_registertogroup(dropdown) == 0);
			dropdowngroup_displayall();
			dropdowngroup_listall();
			printf("\n\n\n\n\nContents flushed\n");

			ui_getvalue(ui_dropdown_getentrybox
			            (ui_getwidget
			             ("soldier.vbox2.vbox3.table2.countryid")), &retstr);
			debugmsg("%s appears in dropdown box country", retstr);
			if (strcmp(retstr, "'New Zealand'") == 0)
				pass++;

			widget = ui_getwidget("soldier.vbox2.vbox3.table2.rankid");
			ui_getvalue(ui_dropdown_getentrybox
			            (ui_getwidget("soldier.vbox2.vbox3.table2.rankid")),
			            &retstr);
			debugmsg("%s now appears in dropdown box rank", retstr);
			if (strcmp(retstr, "Corporal") == 0)
				pass++;

			roof = dataroof_getbyname("soldier.vbox2.vbox3.table2.rankid");
			if (db_numrecord(roof->obj) == 4)
				pass++;
			else
				errormsg("Show %d of %d results instead of 4",
				         db_numrecord(roof->obj), roof->obj->num);
			// fr:dropdown_fb_debug();
			widget = ui_getwidget("soldier.vbox2.vbox3.table2.countryid");
			printf
			("\n\n\n\n\nSetting value in widget\n_____________________________________________________________\n");
			// fr:ui_setvalue(widget,"3");
			dropdown =
			    dropdown_getinstancebyname
			    ("soldier.vbox2.vbox3.table2.countryid");
			g_assert(dropdown);

			// {
			// gint pos;
			// gchar *itemstr;
			// extern gchar frezzechanges;
			//
			// dropdown_setselectedpos(dropdown, 3);
			// dropdown_readlookupvalue(dropdown);
			// itemstr = dropdown->lookupvalue;
			// printf ("******************************************\n");
			// printf ("1: [set sel 3] Got %s at %d\n", itemstr,
			// dropdown_getselectedpos(dropdown));
			//
			// frezzechanges = 1;
			// dropdown_setselectedpos(dropdown, 0);
			// dropdown_readlookupvalue(dropdown);
			// itemstr = dropdown->lookupvalue;
			// printf ("******************************************\n");
			// printf ("2: [set sel 0] Got %s at %d\n", itemstr,
			// dropdown_getselectedpos(dropdown));
			//
			// dropdown_setselectedpos(dropdown, 1);
			// dropdown_readlookupvalue(dropdown);
			// itemstr = dropdown->lookupvalue;
			// printf ("******************************************\n");
			// printf ("3: [set sel 1] Got %s at %d\n", itemstr,
			// dropdown_getselectedpos(dropdown));
			//
			// dropdown_setselectedpos(dropdown, 2);
			// dropdown_readlookupvalue(dropdown);
			// itemstr = dropdown->lookupvalue;
			// printf ("******************************************\n");
			// printf ("4: [set sel 2] Got %s at %d\n", itemstr,
			// dropdown_getselectedpos(dropdown));
			//
			// frezzechanges = 0;
			// }

			dropdown_setselectedpos(dropdown, 2);
			// dropdowngroup_displayall();
			dropdown =
			    dropdown_getinstancebyname
			    ("soldier.vbox2.vbox3.table2.rankid");
			g_assert(dropdown);
			printf("Num rec: %d\n", db_numrecord(dropdown->roof->obj));
			// dropdown_readlookupvalue(dropdown);
			// printf ("Rank id after set to USA %s\n",
			// dropdown->lookupvalue);

			dropdown =
			    dropdown_getinstancebyname
			    ("soldier.vbox2.vbox3.table2.rankid");
			// retstr = dropdown_getwidgetvalue(dropdown);
			retstr = dropdown->lookupvalue;
			printf
			("\nSetting widget changed\n_______________________________________________________________\n\n\n\n");
			/* postit_setwidgetchanged(widget); */

			if (db_numrecord(dropdown->roof->obj) == 1)
				pass++;
			else
				errormsg("Show %d of %d results instead of 1",
				         db_numrecord(dropdown->roof->obj),
				         dropdown->roof->obj->num);

			widget = ui_getwidget("soldier.vbox2.vbox3.table2.rankid");
			debugmsg("getting value now");
			ui_getvalue(ui_dropdown_getentrybox
			            (ui_getwidget("soldier.vbox2.vbox3.table2.rankid")),
			            &retstr);
			// fr:dropdown_fb_debug();
			test_result(retstr, "The dog shite stuck to my shoe...", &pass);

			test_total(pass, 5);
			break;
			}
		/* No i'm really going to test dependences of dropdown boxes now
		   ... */
	case 13:
			{
			Dropdown *dropdown;

			// fr:dropdown_fb_set(BUFFER_BEGIN);
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "4");
			debugmsg("Testing drop down box, lots of dependent stuff");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);
			alias_debugdisplay(roof->alias);
			printf
			("======================= Building Dropdown Box ==============================\n");
			// fr:dropdown_displaydropdown("soldier.vbox2.vbox3.table2.rankid",
			// "soldier",&towaitfor);
			// fr:dropdown_fb_set(BUFFER_FLUSH);

			dropdown_new("soldier.vbox2.vbox3.table2.countryid", "soldier",
			             &dropdown);
			g_assert(dropdowngroup_registertogroup(dropdown) == 0);

			dropdown_new("soldier.vbox2.vbox3.table2.rankid", "soldier",
			             &dropdown);
			g_assert(dropdowngroup_registertogroup(dropdown) == 0);

			dropdown =
			    dropdown_getinstancebyname
			    ("soldier.vbox2.vbox3.table2.rankid");
			dropdown_display(dropdown, FALSE);
			widget = ui_getwidget("soldier.vbox2.vbox3.table2.rankid");
			ui_getvalue(ui_dropdown_getentrybox
			            (ui_getwidget("soldier.vbox2.vbox3.table2.rankid")),
			            &retstr);
			debugmsg("%s appears in dropdown box rank", retstr);
			if (strcmp(retstr, "The dog shite stuck to my shoe...") == 0)
				pass++;

			dropdown_readlookupvalue(dropdown, &pos);
			retstr = dropdown->lookupvalue;
			debugmsg
			("%s is the return value for dropdown box rank, pos is %d",
			 retstr, pos);
			if (retstr != NULL && strcmp(retstr, "1") == 0)
				pass++;

			// fr:dropdown_displaydropdown("soldier.vbox2.vbox3.table2.countryid",
			// "soldier",&towaitfor);
			dropdown =
			    dropdown_getinstancebyname
			    ("soldier.vbox2.vbox3.table2.countryid");
			dropdown_display(dropdown, TRUE);
			widget = ui_getwidget("soldier.vbox2.vbox3.table2.countryid");
			g_assert(widget);

			ui_getvalue((ui_dropdown_getentrybox(widget)), &retstr);
			debugmsg("%s appears in dropdown box country", retstr);
			if (strcmp(retstr, "'USA'") == 0)
				pass++;
			/* strange comments lurking here... bewarned */
			widget = ui_getwidget("soldier.vbox2.vbox3.table2.rankid");
			ui_getvalue((ui_dropdown_getentrybox(widget)), &retstr);
			debugmsg("%s now appears in dropdown box rank", retstr);
			if (strcmp(retstr, "The dog shite stuck to my shoe...") == 0)
				pass++;

			if (pass == 4)
				debugmsg("\nTEST PASSED.");
			else
				{
				printf("Only passed %d of 4", pass);
				errormsg("\nTEST FAILED.");
				}

			break;
			}
		/* testing for showing a form a empty form */
	case 14:
			{
			debugmsg("checking the showing of a blank form");
			bond_specifyclass("soldier", "soldier");
			form_displaybyname("soldier");
			break;
			}
		/* testing for showing a form from a previous record */
	case 15:
			{
			bond_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "2");
			dataroof_update(ui_getwidget("soldier"), obj);

			form_displaybyname("soldier");
			/* pass = debug_checksoldier(); */
			pass = debug_checksoldier_fornewdropdown();
			if (pass != 4)
				errormsg("TEST FAILED");
			else
				debugmsg("TEST PASSED");
			break;
			}
		/* biglists */
	case 16:
			{
			bond_specifyclass("rank", "rank");
			obj = db_createobject("rank");
			db_loadall(obj);
			dataroof_update(ui_getwidget("rank"), obj);
			form_displaybyname("rank");
			break;
			}
		/* Testing for idefault upon you poor soul. Master version */
#endif
#if 0
	case 17:
			{
			gchar *pay;

			debugmsg("iDefault. Proceed");
			obj = db_createobject("rank");
			db_getobjectbyfield(obj, "id", "4");
			idefault_masteredit_updatedb(obj, obj->name, "pay", "5");
			db_freeobject(obj);

			obj = db_createobject("rank");
			db_getobjectbyfield(obj, "id", "4");
			db_getvalue(obj, "description", NULL, &retstr);
			db_getvalue(obj, "pay", NULL, &pay);
			debugmsg("%s has pay of %s", retstr, pay);
			test_result(pay, "5", &pass);

			db_freeobject(obj);

			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "1");
			db_getvalue(obj, "name", NULL, &retstr);
			db_getvalue(obj, "kills", NULL, &pay);
			debugmsg("%s has kills of %s", retstr, pay);
			test_result(pay, "5", &pass);
			db_freeobject(obj);

			obj = db_createobject("rank");
			db_getobjectbyfield(obj, "id", "4");
			idefault_masteredit_updatedb(obj, obj->name, "pay", "2");
			db_freeobject(obj);

			test_total(pass, 2);

			break;
			}
		/* test a broader range of idefault stuff. ... just how do i test
		   this? */
	case 18:
			{
			gint retval;

			debugmsg("iDefault, testing lots of stuff");
			bond_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_getobjectbyfield(ui_getwidget("soldier"), "id", "1");
			form_displaybyname("soldier");
			printf
			("======================== Setting iDefault values ==============================\n");
			widget = ui_getwidget("soldier.vbox2.vbox3.table2.kills");
			retval = idefault_enterwidget(roof, widget);
			ui_getvalue(widget, &retstr);
			debugmsg("entering string is %s, retval is %d", retstr, retval);

			/* setwidgetvalue(widget,"0"); retval =
			   idefault_enterwidget(roof, widget); retstr =
			   getwidgetvalue(widget); debugmsg("entering string is %s,
			   retval is %d",retstr,retval); */

			if (strcmp(retstr, "2") == 0)
				debugmsg("TEST PASSED.");
			else
				debugmsg("TEST FAILED.");

			break;
			}
#endif
#if 0
		/* The following are tests on the high end of the API, designed to
		   simulate direct user action */
	case 19:
			{
			Dropdown *dropdown;

			bond_formadd("soldier");
			ui_setvaluebyname("soldier.vbox2.vbox3.table2.id", "13");
			ui_setvaluebyname("soldier.vbox2.vbox3.table2.name", "Francis");
			ui_setvaluebyname("soldier.vbox2.vbox3.table2.dob",
			                  "1934-12-03");
			ui_setvaluebyname("soldier.vbox2.vbox3.table2.location",
			                  "Spain");
			ui_setvaluebyname("soldier.vbox2.vbox3.table2.kills", "2");
			ui_setvaluebyname("soldier.vbox2.vbox3.table2.table8.active",
			                  "1");
			// ui_setvaluebyname("soldier.vbox2.vbox3.table2.rankid", "3");
			//
			// dropdown_new (
			// "soldier.vbox2.vbox3.table2.rankid", /* dropdown widget name
			// */
			// "soldier", /* formname */
			// &dropdown);
			dropdown =
			    dropdown_getinstancebyname
			    ("soldier.vbox2.vbox3.table2.rankid");
			// dropdown_setselectedpos (dropdown, 1);
			dropdown_setwidgetvalue(dropdown, "Corporal");
			// dropdown_setwidgetvalue (dropdown, "Corporal");
			bond_formsave("soldier");
			bond_formclose("soldier");
			bond_flushall();
			obj = db_createobject("soldier");
			db_loadall(obj);
			db_movelast(obj);

			db_getvalue(obj, "id", NULL, &retstr);
			test_result(retstr, "13", &pass);
			db_getvalue(obj, "name", NULL, &retstr);
			test_result(retstr, "Francis", &pass);
			db_getvalue(obj, "dob", NULL, &retstr);
			test_result(retstr, "1934-12-03", &pass);
			db_getvalue(obj, "active", NULL, &retstr);
			test_result(retstr, "t", &pass);
			db_getvalue(obj, "countryid", NULL, &retstr);
			test_result(retstr, "1", &pass);
			db_getvalue(obj, "rankid", NULL, &retstr);
			test_result(retstr, "3", &pass);
			db_freeobject(obj);
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "13");
			db_deleteobject(obj);
			db_freeobject(obj);
			debugmsg("Finished test, now checking results");
			test_total(pass, 6);
			break;
			}
	case 20:
		imethod_test(1, TRUE);
		/*
		   { GList *itr; DataRoof *roof;

		   for (itr = g_list_first(globaldataroof); itr; itr =
		   g_list_next(itr)) {

		   roof = itr->data; printf ("_______________________\n"); printf
		   ("name:%s\n", roof->widgetlongname); if (roof->obj) { printf
		   ("obj:%s query:%s\n", roof->obj->name, roof->obj->query); printf 
		   ("birth: %p\n", roof->obj->birth); } } } */
		break;
	case 21:
		bond_specifyclass("soldier", "soldier");
		roof = dataroof_getbyname("soldier");
		// fr: uixml_getimmediatechildren("soldier");
		break;
			{
			gchar *retstr2;
			DataRoof *roof;
			UiWidget *widget;
			UiWidget *linkwidget;

			debugmsg("iDefault, testing a bit more of iDefault");
			hidedebuginfo();
			dataroof_init();
			bond_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_getobjectbyfield(ui_getwidget("soldier"), "id", "1");
			form_displaybyname("soldier");
			printf
			("======================== Setting iDefault values ==============================\n");
			widget = ui_getwidget("soldier.vbox2.vbox3.table2.kills");
			linkwidget = ui_getwidget("soldier.vbox2.vbox3.table2.rankid");
			showdebuginfo();
			ui_setvalue(widget, "");
			ui_setvalue(linkwidget, "4");

			/* simple test on entry */
			retval = idefault_enterwidget(roof, widget);
			ui_getvalue(widget, &retstr);
			debugmsg("entering string is %s, retval is %d", retstr, retval);
			test_result(retstr, "2", &pass);
			/* test change of link */
			printf("\n\n");
			ui_setvalue(linkwidget, "Corporal");
			retval = idefault_enterwidget(roof, linkwidget);
			ui_getvalue(widget, &retstr);
			debugmsg("entering string is %s, retval is %d", retstr, retval);
			test_result(retstr, "50000", &pass);
			/* test change of link again */
			printf("\n\n");
			ui_setvalue(linkwidget, "Private");
			retval = idefault_enterwidget(roof, linkwidget);
			ui_getvalue(widget, &retstr);
			debugmsg("entering string is %s, retval is %d", retstr, retval);
			test_result(retstr, "1000", &pass);
			/* test when unique value in to make sure it dont update */
			printf("\n\n");
			ui_setvalue(widget, "3431");
			ui_setvalue(linkwidget, "UiGeneral");
			retval = idefault_enterwidget(roof, linkwidget);
			ui_getvalue(widget, &retstr);
			debugmsg("entering string is %s, retval is %d", retstr, retval);
			test_result(retstr, "3431", &pass);
			printf("\n\n");
			/* enter a widget already having a value */
			retval = idefault_enterwidget(roof, widget);
			ui_getvalue(widget, &retstr);
			debugmsg("entering string is %s, retval is %d", retstr, retval);
			test_result(retstr, "3431", &pass);
			printf("\n\n");
			/* a bit more to expand */
			ui_setvalue(widget, "10000");
			ui_setvalue(linkwidget, "2");
			retval = idefault_enterwidget(roof, linkwidget);
			ui_getvalue(widget, &retstr);
			debugmsg("entering string is %s, retval is %d", retstr, retval);
			test_result(retstr, "10000", &pass);
			printf("\n\n");
			ui_setvalue(linkwidget, "5");

			ui_getvalue(linkwidget, &retstr2);
			if (strcmp(retstr2, "5") != 0)
				errormsg
				("FAILED HERE, unable to set widget to id 5, it returned %s",
				 retstr2);
			retval = idefault_enterwidget(roof, linkwidget);

			debugmsg("=============================================\n"
			         "entering string is %s, retval is %d, %s", retstr,
			         retval, retstr2);
			ui_getvalue(linkwidget, &retstr);
			if (strcmp(retstr, "5") != 0)
				errormsg("Failed here");

			ui_getvalue(widget, &retstr2);
			test_result(retstr2, "1000", &pass);

			ui_setvalue(linkwidget, "4");
			retval = idefault_enterwidget(roof, linkwidget);

			ui_getvalue(linkwidget, &retstr2);
			debugmsg("=============================================\n"
			         "entering string is %s, retval is %d, %s", retstr,
			         retval, retstr2);
			test_result(retstr2, "2", &pass);
			/* otherroof = dataroof_getbywidget(linkwidget);
			   filterit_debugall(otherroof); */
			/* print results */
			debugmsg("%d of 6", pass);
			if (pass == 6)
				debugmsg("\nTEST PASSED.");
			else
				errormsg("\nTEST FAILED.");
			break;
			}
	case 21:
			{
			DataRoof *roof;
			gfloat stuff;

			dataroof_init();
			hidedebuginfo();
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "3");
			debugmsg("Testing drop down box, setting a value");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(roof->obj, ui_getwidget("soldier"),
			                          NULL);
			showdebuginfo();
			printf
			("======================== Testing Spin UiButton ==============================\n");
			gtk_spin_button_set_value(GTK_SPIN_BUTTON
			                          (ui_getwidget
			                           ("soldier.vbox2.vbox3.table2.table8.kills2")),
			                          5);
			stuff =
			    gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON
			                                       (ui_getwidget
			                                        ("soldier.vbox2.vbox3.table2.table8.kills2")));

			debugmsg("Text retrieved from query %f", stuff);
			if (stuff != 5)
				errormsg("TEST FAILED");
			dataroof_cleanup();
			break;
			}
#endif
#if 0
	case 22:
			{
			DataRoof *roof;
			gfloat stuff, test;
			gchar *stuff2, *retstr2;

			dataroof_init();
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "3");
			debugmsg("Testing drop down box, setting a value");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);
			printf
			("======================== Testing Spin UiButton ==============================\n");
			gtk_spin_button_set_value(GTK_SPIN_BUTTON
			                          (ui_getwidget
			                           ("soldier.vbox2.vbox3.table2.table8.kills2")),
			                          5.5);
			stuff =
			    gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON
			                                       (ui_getwidget
			                                        ("soldier.vbox2.vbox3.table2.table8.kills2")));
			debugmsg("Text retrieved from query %f", stuff);

			ui_getvalue(ui_getwidget("soldier.vbox2.vbox3.table2.kills"),
			            &stuff2);
			debugmsg("Text from stuff2 goodness %s", stuff2);
			ui_setvaluebyname("soldier.vbox2.vbox3.table2.kills", "6.5");
			ui_getvaluebyname("soldier.vbox2.vbox3.table2.kills", &retstr2);
			test = atof(retstr2);
			debugmsg("Text from stuff2 goodness the second time round %f",
			         test);

			if (stuff != 5.5)
				errormsg("Test failed");
			dataroof_cleanup();
			break;

			}
	case 23:
			{
			/* test idefaultmaster set. */
			DataRoof *roof;
			AliasDataSource *alias;
			UiWidget *widget;

			hidedebuginfo();
			dataroof_init();
			obj = db_createobject("rank");
			db_getobjectbyfield(obj, "id", "1");
			debugmsg("iDefault, testing lots of stuff");
			bond_specifyclass("soldier", "soldier");
			bond_specifyclass("rank", "rank");
			dataroof_update(ui_getwidget("rank"), obj);
			dataroof_getobjectbyfield(ui_getwidget("soldier"), "id", "4");
			form_displaybyname("soldier");

			form_displaybyname("rank");
			roof = dataroof_getbyname("rank");
			alias = dataroof_searchforwidget(roof, "rank.vbox1.table1.pay");
			g_assert(alias);
			g_assert(roof);
			showdebuginfo();

			printf
			("======================== Testing iDefault Values ==============================\n");
			widget = ui_getwidget("rank.vbox1.table1.pay");
			g_assert(widget);
			ui_setvalue(widget, "12000");
			alias->changed = TRUE;
			retval = idefault_entermaster(roof, alias);

			widget = ui_getwidget("soldier.vbox2.vbox3.table2.kills");
			g_assert(widget);
			ui_getvalue(widget, &retstr);
			debugmsg("entering string is now %s, retval is %d", retstr,
			         retval);
			if (strcmp(retstr, "12000") == 0)
				pass++;
			else
				debugmsg("FAILED section");

			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "4");
			db_getvalue(obj, "kills", NULL, &retstr);
			if (strcmp(retstr, "12000") == 0)
				pass++;
			else
				debugmsg("FAILED section");

			/* clean up so it works next time */
			db_setvalue(obj, "kills", NULL, "13");
			db_freeobject(obj);

			obj = db_createobject("rank");
			db_getobjectbyfield(obj, "id", "1");
			db_setvalue(obj, "pay", NULL, "13");
			db_freeobject(obj);

			debugmsg("kills is now %s", retstr);

			dataroof_cleanup();

			debugmsg("%d of 2", pass);
			if (pass == 2)
				debugmsg("\nTEST PASSED.");
			else
				debugmsg("\nTEST FAILED.");
			break;
			}
	case 24:
			{
			/* display the form and check to see if all the values appear in
			   the clist form. This involves doing a compare with what should 
			   be in there. Note need to check gtk documentation to find the
			   values of a row in the clist. */
			Object *obj;
			gint rows, cols, counta, counta2;
			GtkCList *clist;
			gchar *word;

			hidedebuginfo();
			dataroof_init();
			bond_specifyclass("rank", "rank");
			obj = db_createobject("rank");
			db_loadall(obj);
			dataroof_update(ui_getwidget("rank"), obj);
			clist = GTK_CLIST(ui_getwidget("clist2"));
			g_assert(clist);
			rows = obj->num;
			cols = ui_list_numcolumn(widget);
			form_displaybyname("rank");
			showdebuginfo();
			printf("\n|");
			for (counta = 0; counta < rows; counta++)
				{
				for (counta2 = 0; counta2 < cols; counta2++)
					{
					gtk_clist_get_text(clist, counta, counta2, &word);
					printf("  %15s  |", word);
					mem_free(word);
					}
				printf("\n|");
				}

			dataroof_cleanup();
			break;
			}
#endif
#if 0
	case 25:
			{
			/* Test to see if autoattaching signals is all go */

			DataRoof *roof;
			GList *test_list = NULL;
			UiWidget *widget;

			dataroof_init();
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "3");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);

			printf
			("======================== Testing UiButton Signal==============================\n");

			widget =
			    ui_getwidget
			    ("soldier.vbox2.vbox3.handlebox4.toolbar2.nextbutton");
			test_list = g_list_append(test_list, widget);
			widget =
			    ui_getwidget("soldier.vbox2.vbox3.handlebox4.toolbar2.c");
			test_list = g_list_append(test_list, widget);

			hitme_findsources("soldier", test_list);

			dataroof_cleanup();

			break;
			}
	case 26:
			{
			/* Test apicalls - Show form by form */

			DataRoof *roof;

			dataroof_init();
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "2");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);

			printf
			("==================Testing Show UiForm by UiForm==========================\n");

			bond_showformbyform("weapon", "soldier");

			ui_getvalue(ui_getwidget("weapon.vbox7.table6.name"), &retstr);
			debugmsg("name: %s", retstr);

			dataroof_cleanup();

			break;
			}

	case 27:
			{
			/* Test apicalls */
			DataRoof *roof;

			dataroof_init();
			obj = db_createobject("soldier");
			db_getobjectbyfield(obj, "id", "1");
			dataroof_specifyclass("soldier", "soldier");
			roof = dataroof_getbyname("soldier");
			dataroof_update(ui_getwidget("soldier"), obj);
			roof->alias =
			    alias_finddatasources(obj, ui_getwidget("soldier"), NULL);

			printf
			("==================Testing Show UiForm by UiForm==========================\n");

			bond_showformbyform("weapon", "soldier");

			dataroof_cleanup();

			break;
			}
	case 28:
			{
			/* test idefaultmaster set. */
			DataRoof *roof;
			iDefault *def;
			iDefault_Link *link;

			hidedebuginfo();
			obj = db_createobject("rank");
			db_getobjectbyfield(obj, "id", "3");
			debugmsg("iDefault, testing lots of stuff");
			bond_specifyclass("soldier", "soldier");
			bond_specifyclass("rank", "rank");
			form_displaybyname("soldier");
			dataroof_update(ui_getwidget("rank"), obj);
			printf
			("======================== Testing iDefault Values ==============================\n");

			form_displaybyname("rank");
			roof = dataroof_getbyname("rank");

			showdebuginfo();

			def = idefault_getidefault(roof, ui_getwidget("pay"));
			walk = idefault_findwidgetsref(def);
			if (walk == NULL)
				debugmsg("TEST FAILED");
			while (walk != NULL)
				{
				link = walk->data;
				debugmsg("widgetname of relation is %s",
				         link->alias->widgetlongname);
				walk = walk->next;
				}
			idefault_freelinkref(walk);
			break;
			}
#endif
	default:
			{
			debugmsg("Test hasn't been written yet");
			}
		}
	bond_cleanup(bond);
	/* g_mem_profile(); */
	return 0;
	}
