/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef ui_h
#define ui_h

/* void* pointer to Uiwidget declared in the dynamically loaded library */
typedef void UiWidget;
typedef void UiEvent;

#include "uiplugin.h"
#include "uidropdown.h"

/* Some functions mentoned in here may not work or be present anymore.
 * This is the left overs from 0.4.0 release which had all of this stuff.
 * I've tried to simplify ui.c to make porting easier.
 */

UiXML * ui_init(gchar * library, gchar * localargs, gchar * xmlfile, gint argc, gchar * argv[]);
gint ui_cleanup(UiXML *uixml);
gint ui_main(UiXML *uixml);
gint ui_mainquit(UiXML *uixml);

UiWidget *ui_getwidget(UiXML *uixml, gchar * widgetname);
gint ui_getvalue(UiXML *uixml, UiWidget * widget, gchar ** value);
gchar *ui_getname(UiXML *uixml, UiWidget * widgetname);
gchar *ui_getname_short(UiXML *uixml, UiWidget * widgetname);
gint ui_getvaluebyname(UiXML *uixml, gchar * widgetname, gchar ** value);
UiWidgetType ui_gettype(UiXML *uixml, UiWidget * widget);
void ui_parse(UiXML *uixml);
gchar *ui_getparentformname(UiXML *uixml, UiWidget * widget);

gint ui_setvalue(UiXML *uixml, UiWidget * widget, gchar * value);
gint ui_setvaluebyname(UiXML *uixml, gchar * widgetname, gchar * value);
GList *ui_widgetchildren(UiXML *uixml, gchar * widgetlongname);

/*fr: added */
gchar *ui_widgetlongname(UiXML *uixml, gchar * widgetlongname);
void ui_showwidget(UiXML *uixml, UiWidget * widget);
void ui_hidewidget(UiXML *uixml, UiWidget * widget);
gboolean ui_isvisible(UiXML *uixml, UiWidget * widget);
gint ui_issignalattached(UiXML *uixml, UiWidget * widget, gchar * name);

GList *ui_getcontainerchildren(UiXML *uixml, UiWidget * widget);

void ui_signalemitstopbyname(UiXML *uixml, UiWidget * object, const gchar * name);
void ui_signalemitbyname_arg_pUiEvent_pgboolean(UiXML *uixml, UiWidget * object, const gchar * name, UiEvent * event,
        gboolean * retval);
void ui_event_handlekeyevent(UiXML *uixml, UiEvent * event, void *user_data);
void ui_event_setvalue(UiXML *uixml, UiEvent * event, UiEventType type, UiEventValue value);

void ui_editable_inserttext(UiXML *uixml, UiWidget * editable, const gchar * new_text, gint new_text_length, gint * position);
void ui_editable_deletetext(UiXML *uixml, UiWidget * editable, gint start_pos, gint end_pos);
void ui_editable_setposition(UiXML *uixml, UiWidget * editable, gint position);
void ui_togglebutton_setactive(UiXML *uixml, UiWidget * toggle_button, gboolean is_active);

UiWidget *ui_getwidgetinparent(UiXML *uixml, gchar * name, gchar * parentname);

gchar *ui_findxmlfile(gchar * gladefile);

gint ui_get_selected_row_number(UiXML *uixml, UiWidget * clist);
void ui_assign_position(UiXML *uixml, gchar * widgetname, gint x, gint y);

/* other */
void ui_seteditable(UiXML *uixml, UiWidget * widget, gboolean state);
gboolean ui_geteditable(UiXML *uixml, UiWidget * widget);
gchar *ui_event_getstringfromevent(UiXML *uixml, UiWidget * widget);
void ui_event_setstringfromevent(UiXML *uixml, UiWidget * widget, gchar * str);
void ui_setsensitive(UiXML *uixml, UiWidget * widget, gboolean val);

gboolean ui_isformcapable(UiXML *uixml, UiWidget * widget);

void ui_list_append(UiXML *uixml, UiWidget * widget, GList * values, gulong secretid, gboolean append);
void ui_list_clear(UiXML *uixml, UiWidget * widget);
gboolean ui_list_setselection(UiXML *uixml, UiWidget * widget, gulong secretid);
gulong ui_list_getselection(UiXML *uixml, UiWidget * widget, gboolean getvalues, GList ** values);
gint ui_list_setcellvalue(UiXML *uixml, UiWidget * widget, gint col, gulong secretid, gchar * value);
void ui_list_setrow(UiXML *uixml, UiWidget * widget, gint row);

/* Signal code */
gint ui_signalconnect(UiXML *uixml, UiWidget * object, const gchar * name, void *(*)(), void *data);
void ui_signaldisconnect(UiXML * uixml, UiWidget * object, gint signalid);
void ui_signaldisconnectbyfunc(UiXML *uixml, UiWidget * object, void (*argfunc) (), void *data);
UiSignalKey * ui_signalconnect_keypress(UiXML *uixml, UiWidget * widget, gint (*callback) (void *, gchar *str), void *userdata);
void ui_signaldisconnect_keypress(UiXML *uixml,UiWidget * widget, UiSignalKey *signalkey);

gint ui_display_message(UiXML *uixml, gchar *message,gchar *title, gint type);
gint ui_raisewindow(UiXML * uixml, UiWidget *widget);


/* OBSOLETE */
#endif
