#ifndef uicommonplugin_h
#define uicommonplugin_h

#include <libxml/parser.h>
#include <time.h>

#include "bc.h"

#define IsEqual(a, b) !strcmp(a, b)
#define IsTrue(a) IsEqual(a, "true")
#define IsFalse(a) IsEqual(a, "false")

typedef struct _UiXML UiXML;
typedef struct Plugin Plugin;

/* not sure on this one, some explain this for me */
typedef int (*plugin_init_cb) (Plugin * plugin);

/* list of known libraries i'm referencing */
typedef enum
{ gtk, unknown }
PluginRef;

typedef enum
{
    UiNull,
    UiGeneral,
    UiLabel,
    UiEntryBox,
    UiDropDown,
    UiGrid,
    UiGridColumn,
    UiToggleButton,
    UiButton,
    UiCheckButton,
    UiRadioButton,
    UiSpinButton,
    UiForm,
    UiFrame,
    UiTextBox,
    UiScrolledWindow,
    UiNotebook,
    UiTree,
    UiMenuItem
}
UiWidgetType;

/* Francis added the following key types. */
typedef enum
{
    UI_KP_RIGHT,
    UI_KP_DOWN,
    UI_KP_LEFT,
    UI_KP_UP
}
UiEventValue;

typedef enum
{
    UI_KEY,
    UI_BUTTON
}
UiEventType;

typedef struct
	{
	void *ptr;		  /* pointer to the function */
	gchar *name;		  /* function name */
	}
PluginFunction;

struct Plugin
	{
	PluginRef type;
	gchar *name;
	gchar *args;
	void *dlhandle;

	/* Some functions */
	plugin_init_cb sym;
	/* an array of functions */
	PluginFunction **func;
	gint numfunctions;
	/* functions that connect back into bond -> in the other direction */
	PluginFunction *func_gridedit;
	/* to set i'mbusygoaway  */
	PluginFunction *func_blockemit;
	};

/* Public */
struct _UiXML
	{
	/* This is BondXML object */
	void *parent;
	/* Plugin that loads the library */
	Plugin *plugin;

	guint widgetcounter;
	/* Hash tables containing information on widgets etc */
	GHashTable *widgets;
	GHashTable *longname;
	GHashTable *shortname;
	/* do windows default to visible */
	gboolean defaultvisible;
	gint defaultentrywidth;
	/* xml document */
	xmlDoc *doc;
	/* structure for remembering callbacks */
	GList *uisignalkeylist;
	/* General garbage to be freed on exit. */
	MemGarbage *garbage;

	gchar *sha;
	time_t creation_time;
	};

typedef struct
	{
	UiXML *uixml;
	void *widget;
	void *userdata;
	gint(*func) (void *, gchar * str);
	void (*parentcallback) ();
	guint signal_id;
	}
UiSignalKey;

#endif
