/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>

#include "ui.h"
#include "uidropdown.h"
#include "uiplugin.h"
#include "bc.h"

void
ui_dropdown_populate(UiXML * uixml, UiWidget * widget, GList * values)
	{
	PluginFunction *f;
	typedef UiWidget *(*function) (UiXML * uixml, UiWidget * widget,
	                               GList * values);
	function func;

	f = plugin_findfunction(uixml, "dropdown_populate");
	func = f->ptr;
	func(uixml, widget, values);
	}

UiWidget *
ui_dropdown_getentrybox(UiXML * uixml, UiWidget * widget)
	{
	PluginFunction *f;
	typedef UiWidget *(*function) (UiXML * uixml, gchar * widget);
	function func;

	f = plugin_findfunction(uixml, "dropdown_getentrybox");
	func = f->ptr;
	return func(uixml, widget);
	}

/**
 * ui_dropdown_getpopulatelist:
 *
 * Get the listbox inside a dropdown widget
 */
GList *
ui_dropdown_getpopulatelist(UiXML * uixml, xmlNodePtr node)
	{
	PluginFunction *f;
	typedef GList *(*function) (UiXML * uixml, xmlNode *);
	function func;

	f = plugin_findfunction(uixml, "dropdown_getpopulatelist");
	func = f->ptr;
	return func(uixml, node);
	}

/**
 * ui_dropdown_getparent:
 *
 * Find the parent dropdown widget from a entrybox or a list.  It will move
 * one up in the XML file and return that widget.
 */

UiWidget *
ui_dropdown_getparent(UiXML * uixml, UiWidget * widget)
	{
	PluginFunction *f;
	typedef UiWidget *(*function) (UiXML * uixml, UiWidget * widget);
	function func;

	f = plugin_findfunction(uixml, "dropdown_getlistbox");
	func = f->ptr;
	return func(uixml, widget);
	}

/**
 * ui_dropdown_gettype:
 *
 * This needs properly coding, but it would return a number based upon
 * the properties of the dropdown box.  Ie, if the user can type anything or if
 * it is hard linked.
 */
gint
ui_dropdown_gettype(UiXML * uixml, UiWidget * widget)
	{
	PluginFunction *f;
	typedef gint(*function) (UiXML * uixml, UiWidget * widget);
	function func;

	f = plugin_findfunction(uixml, "dropdown_gettype");
	func = f->ptr;
	return func(uixml, widget);
	}

/**
 * ui_dropdown_getselection:
 * @widget:
 * 
 * Get a GList of all the currently selected items in a dropdown box, this would
 * normally be 1, but sometimes you might have selected more than one item.
 */
GList *
ui_dropdown_getselection(UiXML * uixml, UiWidget * widget)
	{
	errormsg("DISABLED");
	return NULL;
	}

/**
 * ui_dropdown_setselection:
 * @widget:
 * @pos:
 *
 * Set the selection in the dropdown box.
 * 
 */
gint
ui_dropdown_setselection(UiXML * uixml, UiWidget * widget, gint pos)
	{
	errormsg("DISABLED");
	return 0;
	}

/**
 * ui_dropdown_draw:
 * 
 * Draw the widget.
 * Note: this needs to moved, well the code, out of here to somewhere else.
 */
gint
ui_dropdown_draw(UiXML * uixml, UiWidget * widget, GList * values)
	{
	PluginFunction *f;
	typedef void (*function) (UiXML * uixml, UiWidget * widget, GList * values);
	function func;

	f = plugin_findfunction(uixml, "dropdown_draw");
	func = f->ptr;

	func(uixml, widget, values);

	return 0;
	}

gint
ui_dropdown_clear(UiXML * uixml, UiWidget * widget)
	{
	PluginFunction *f;
	typedef gint(*function) (UiXML * uixml, UiWidget * widget);
	function func;

	f = plugin_findfunction(uixml, "dropdown_clear");
	func = f->ptr;

	return func(uixml, widget);
	}
