/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#ifndef watchtower_h
#define watchtower_h

#include "bond.h"
#include "dbuniqueid.h"
#include "ui.h"

typedef struct _Registration Registration;
typedef struct _WatchTower WatchTower;

struct _Registration
	{
	BondObject *registraint;
	gchar *key;
	gchar *fieldname;
	gchar *tablename;
	gchar *dbobjectsrc;
	/* if its a list then i need to know. */
/*	gboolean wholedbobj; */
	/* Convert'ing id's to rows */
/*	GHashTable *id2row; */
	
	/* Older stuff --- Not so much matters */
	/* Link list of registrations for dependencies */
/*	GList *nextlist;
	Registration *prev;	 */
	gint row;
	};

/**
 * Watchtowers watch. They are added watchers which look for certain conditions
 * then run there designated action on spotting.
 *
 */
struct _WatchTower
	{
	/* object doing the watching */
	BondObject *watching;
	/* things to look for. Can be NULL on general stuff */
	gchar *tablename;
	gchar *fieldname;
	gchar *key;
	gchar *dbobjectsrc;
	/* widget to watch */
	UiWidget *widget;
	/* do I trigger a change event on successfully watching someone */
	gboolean triggerchange;
	/* if it is active */
	gboolean ready;
	/* if functiononchange has been activated yet. */
	gboolean functionhasbeenrun;
	/* function to call when the event happens. */
	void (*runonchange) (void *, void *);
	/* general pointers used in internal functions */
	void *ptr[3];
	};

void registration_init(Bond *bond);
gchar *registration_getkey(Object *obj, gchar *tablename, gchar *fieldname);
Registration *registration_add(BondObject * reg, Object * obj, gchar * tablename, gchar * fieldname, gboolean quickadd);
void registration_remove_bybobj(BondObject * bobj);
void registration_cleanup(Bond *globalreg);
void registration_debug(Registration * reg);
void registration_debugall(GList *reglist);
int registration_hash_length(GHashTable *table);
void registration_debugtotal(Bond *bond);
GList *registration_check(BondObject * bobj);
GList *registration_getbykey(GlobalRegistor *globalreg, gchar *key);

WatchTower *watchtower_create(BondObject * watching, void (*runonchange) (void *, void *));
GList *watchtower_register(WatchTower * watch);
void watchtower_unregister(BondObject * bobj);
gint watchtower_forcerun(BondObject * bobj);
gint watchtower_change(BondObject * bobj);
gint watchtower_run(WatchTower * watch, BondObject * bobj);
void watchtower_cleanup(Bond *globalreg);
void watchtower_debug(GlobalRegistor *globalreg);

gint watch_test(gint testnum, BondObject * bobj1, BondObject * bobj2, BondObject * bobj3);
void watch_set_id(WatchTower * watch, DbUniqueId * id);
void watch_set_obj(WatchTower * watch, Object * obj);
void watch_set_widget(WatchTower * watch, gchar * widgetname);
void watchtower_set_field(WatchTower * watch, gchar * table, gchar * field);

#endif
