#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libxml/parser.h>

#include "uixml.h"
#include "../src/uiplugin.h"
#include "../src/ui.h"

#include <debug.h>
#include <dbmem.h>

/**
 * Global XML document, similar, or the same to globalxml but without gladexml 
 * touching it.
 */
xmlDoc *globalxmldoc = NULL;

/**
 * IT BEGINS HERE ...
 *
 *
 */

static gchar *
uixml_searchnode(xmlNode * node, gchar * attrname)
   {
   xmlNode *child;

   g_assert(node);
   g_assert(attrname);
   child = node->childs;
   while (child != NULL)
      {
      if (child->childs != NULL)
         if (child->childs->content != NULL)
            if (strcmp(child->name, attrname) == 0)
               return child->childs->content;
      child = child->next;
      }
   return NULL;
   }

static gint
uixml_notebook_istab(xmlNode * node)
   {
   gchar *str;

   g_assert(node);
   str = uixml_searchnode(node, "child_name");
   if (str == NULL)
      ;
   else if (strcmp(str, "Notebook:tab") == 0)
      return 1;
   return 0;
   }

/* yes i know there are lots of static functions, no andre did not write this. for
 once i get to write a small module */
static void
uixml_notebook_free_xml(GList * retlist)
   {
   XmlNotebook *page;
   GList *walk;

   for (walk = g_list_first(retlist); walk != NULL; walk = walk->next)
      {
      page = walk->data;
      mem_free(page->widgetname);
      mem_free(page);
      }
   g_list_free(retlist);
   }

static xmlNode *
get_node(xmlNode * node, char *name)
   {
   xmlNode *child;

   if (!node)
      return NULL;

   /* debugmsg ("Searching for %s node in %s", name, node->name); */
#ifdef _LIBXML2

   for (child = node->children; child != NULL; child = child->next)
#else

   for (child = node->childs; child != NULL; child = child->next)
#endif

      {
      if (strcmp(child->name, name) == 0)
         return child;
      }

   return NULL;
   }

static char *
get_node_content(xmlNode * node, char *name)
   {
   xmlNode *child;

   child = get_node(node, name);
   if (child)
      return child->content;
   return NULL;
   }

/**
 * uixml_init:
 * 
 * Parse the XML file and keep the structure in memory.
 */
gint
uixml_init(gchar * xmlfile)
   {
   g_assert(xmlfile);
   globalxmldoc = xmlParseFile(xmlfile);
   if (globalxmldoc != NULL)
      printf("XML parsing successful. %s\n", xmlfile);
   else
      printf("XML parsing failed.\n");

   return 0;
   }

void
uixml_cleanup(void)
   {
   if (globalxmldoc!=NULL)
	xmlFreeDoc(globalxmldoc);
   }

static gint
uixml_notebook_getchildren_getlist(gchar * widgetname, gchar ** prevname, xmlNode * node, gint save, GList ** retlist, gint limit)
   {
   XmlNotebook *page;
   gint tosave = 0;
   gchar *tmpstr, *name, *whatwas = NULL;
	if (limit >= 2)
		return 0;

   if (*prevname != NULL)
      whatwas = mem_strdup(*prevname);
	
   if (strcmp(node->name, "widget") == 0)
      {
      name = uixml_searchnode(node, "name");
      if (name != NULL)
         {
         if (prevname != NULL && *prevname != NULL)
            {
            tmpstr = mem_strdup(*prevname);
            mem_free(*prevname);
            *prevname = mem_strdup_printf("%s.%s", tmpstr, name);
            mem_free(tmpstr);
            }
         else
            *prevname = mem_strdup(name);
         /* printf("Comparing %s and %s\n",*prevname,widgetname); */
         if (strcmp(*prevname, widgetname) == 0)
            {
            tosave = 1;
				save = 1;
            }
         if (save == 1)
            {
				if (limit != -1)
					limit ++;
				
            page = (XmlNotebook *) mem_alloc(sizeof(XmlNotebook));
            page->node = node;
            page->widgetname = mem_strdup(*prevname);
            if (uixml_notebook_istab(node) == 1)
               page->notebooklabel = 1;
            else
               page->notebooklabel = 0;
            (*retlist) = g_list_append((*retlist), page);
            }
         }
      }

   /* go recusive */
   if (node->childs != NULL)
      uixml_notebook_getchildren_getlist(widgetname, prevname, node->childs, save, retlist,limit);
   /* finish saving sub children, abort */
   if (tosave == 1)
      save = -1;
   if (prevname != NULL && *prevname != NULL)
      {
      mem_free(*prevname);
      if (whatwas != NULL)
         {
         *prevname = mem_strdup(whatwas);
         mem_free(whatwas);
         }
      else
         *prevname = NULL;
      }
   if (node->next != NULL && save >= 0)
      uixml_notebook_getchildren_getlist(widgetname, prevname, node->next, save, retlist,limit);
   return 0;
   }

static GList *
uixml_notebook_getchildren_build(GList * retlist, gint start, gint end)
   {
   XmlNotebook *page;
   gint widgetcounter = 0, save = 0;
   GList *thelist = NULL, *walk;

   for (walk = g_list_first(retlist); walk != NULL; walk = walk->next)
      {
      page = walk->data;
      if (widgetcounter == start)
         save = 1;
      if (save == 1)
         thelist = g_list_append(thelist, mem_strdup(page->widgetname));
      if (widgetcounter == end)
         break;
      widgetcounter++;
      }
   return thelist;
   }

/**
 * uixml_notebook_getchildren:
 * @widgetname: A label of a notebook tab.
 * 
 * Get a list of all widgets inside a tab, specified in @widgetname.
 *
 * Returns: A GList you need to free using uixml_notebook_freelist() or %NULL on error or no data.
 */

GList *
uixml_notebook_getchildren(gchar * widgetname)
   {
   XmlNotebook *page;
   gint lasttabpos = 1, widgetcounter;
   gchar *prevname = NULL, *notebookname;
   xmlNode *child;
   GList *retlist = NULL, *thelist = NULL, *walk;

   /* XML Document stuff */
   g_assert(globalxmldoc);
   /* Get name your actually looking for */
   notebookname = uixml_widget_getoneup(widgetname); /* BUG HERE IN TABLE.FIELD CASE */

   child = globalxmldoc->root;
   for (; child != NULL; child = child->next)
      uixml_notebook_getchildren_getlist(notebookname, &prevname, child, 0, &retlist,-1);

   /* printf("%d items returned inside widget %s\n",g_list_length(retlist),notebookname); */
   widgetcounter = 0;
   for (walk = g_list_first(retlist); walk != NULL; walk = walk->next)
      {
      page = walk->data;
      /* printf("%d debug %s :%d\n",widgetcounter,(gchar*)page->widgetname,page->notebooklabel); */
      if (strcmp(page->widgetname, widgetname) == 0)
         {
         /* printf("Building list from %d to %d\n",lasttabpos,widgetcounter);  */
         thelist = uixml_notebook_getchildren_build(retlist, lasttabpos, widgetcounter);
         break;
         }
      if (page->notebooklabel == 1)
         lasttabpos = widgetcounter;
      widgetcounter++;
      }

   uixml_notebook_free_xml(retlist);

   /* for (walk = g_list_first(thelist);walk!=NULL;walk=walk->next)
   		printf("Returning Widget %s\n",(gchar*)(walk->data));  */
   return thelist;
   }

/**
 * uixml_widget_getoneup:
 * @widgetname: name of widget
 * 
 * Gets the name of the widget one parent up, so if the widget name was 
 * window.frame.frame.frame.box it would return window.frame.frame.frame.
 *
 * Returns: %NULL on error, else a newly allocated string (which you need to free)
 */
gchar *
uixml_widget_getoneup(gchar * widgetname)
   {
   gchar *retstr;
   gint i, slen, j;

   if (widgetname == NULL)
      return NULL;
   slen = strlen(widgetname);
   for (i = slen - 1; i >= 0; i--)
      {
      if (widgetname[i] == '.')
         {
         retstr = mem_alloc(i + 1);
         for (j = 0; j < i; j++)
            retstr[j] = widgetname[j];
         retstr[j] = 0;
         /* printf("returned %s from %s:uxxml_widget_getoneup\n",retstr,widgetname); */
         return retstr;
         }
      }
   return NULL;
   }

/**
 * uixml_notebook_freelist:
 * @thelist: GList to be freed.
 * 
 * This frees up the list returned by uixml_notebook_getchildren() as well
 * as from uixml_notebook_gettabs().
 */
void
uixml_notebook_freelist(GList * thelist)
   {
   GList *walk;

   if (thelist == NULL)
      return ;
   walk = g_list_first(thelist);
   for (; walk != NULL; walk = walk->next)
      mem_free(walk->data);
   g_list_free(thelist);
   }

/**
 * uixml_notebook_gettabs:
 * @widgetname: Name of a notebook.
 * 
 * Get a list of tabs in notebook @widgetname, and return the label for each of these.
 * You can use uixml_notebook_getchildren() to find out widgets are in the tab.
 *
 * Returns: an allocated list of widget names that needs freeing by uixml_notebook_freelist()
 */
GList *
uixml_notebook_gettabs(gchar * widgetname)
   {
   XmlNotebook *page;
   gchar *prevname = NULL;
   xmlNode *child;
   GList *retlist = NULL, *thelist = NULL, *walk;

   g_assert(globalxmldoc);
   child = globalxmldoc->root;
   for (; child != NULL; child = child->next)
      uixml_notebook_getchildren_getlist(widgetname, &prevname, child, 0, &retlist,-1);

   /* printf("%d items returned\n",g_list_length(retlist)); */
   for (walk = g_list_first(retlist); walk != NULL; walk = walk->next)
      {
      page = walk->data;
      if (page->notebooklabel == 1)
         {
         thelist = g_list_append(thelist, mem_strdup(page->widgetname));
         }
      }
   uixml_notebook_free_xml(retlist);

   /*	for (walk = g_list_first(thelist);walk!=NULL;walk=walk->next)
   		printf("Returning Widget %s\n",(gchar*)(walk->data)); */ 
   return thelist;
   }


GList *
uixml_getimmediatechildren(gchar * widgetname)
	{
	gint first=1;
   XmlNotebook *page;
   gchar *prevname = NULL;
   xmlNode *child;
   GList *retlist = NULL, *thelist = NULL, *walk;

   g_assert(globalxmldoc);
   child = globalxmldoc->root;
   for (; child != NULL; child = child->next)
      uixml_notebook_getchildren_getlist(widgetname, &prevname, child, 0, &retlist,0);

   /* printf("%d items returned\n",g_list_length(retlist)); */
   for (walk = g_list_first(retlist); walk!=NULL; walk=walk->next)
      {
      page = walk->data;
		if (first == 0)
			thelist = g_list_prepend(thelist, mem_strdup(page->widgetname));
		else
			first = 0;
      }
   uixml_notebook_free_xml(retlist);

 /*   for (walk = g_list_first(thelist);walk!=NULL;walk=walk->next)
   		printf("Returning Widget %s\n",(gchar*)(walk->data)); */
   return thelist;
	}



 