#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_display.h"
#include "bc.h"
#include "uibutton.h"
#include "parse.h"

/**
 * ui_button_create:
 * @parentnode: an xml node in the bond xml file
 * @parentwidget: parent widget of this node
 *
 * Creates a button based on values of label and stock. 
 * If only label is set: and the label is a stock id, then the button
 *   created is a stock button, otherwise gtk will create a button with
 *   a label text the value of the label.
 * If stock and label are set: then a stock property will take precedence over the label property.
 * If only stock is set: same as if only label was set.
 * If neither are set: A button with no label is created.
 *
 * Attributes:
 * label: sets label of button
 * stock: stock id to use. eg "gtk-add"
 * 
 * Returns: pointer to the widget created.
 */

GtkWidget *
uigtk_button_create(xmlNode * parentnode, GtkWidget * parentwidget)
	{
	GtkWidget *widget;
	GtkWidget *alignment;
	GtkWidget *hbox;
	GtkWidget *image;
	GtkWidget *label;
	gchar *content = NULL;

	content = xmlGetProp(parentnode, "stock");
	if (content)
		widget = gtk_button_new_from_stock(content);
	content = xmlGetProp(parentnode, "image");
	if (content)
		{
		widget = gtk_button_new();
		alignment = gtk_alignment_new(0.5, 0.5, 0, 0);
		gtk_widget_show(alignment);
		gtk_container_add(GTK_CONTAINER(widget), alignment);

		hbox = gtk_hbox_new(FALSE, 2);
		gtk_widget_show(hbox);
		gtk_container_add(GTK_CONTAINER(alignment), hbox);

		image = gtk_image_new_from_stock(content, GTK_ICON_SIZE_BUTTON);
		gtk_widget_show(image);
		gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);

		if ((content = xmlGetProp(parentnode, "label")) != NULL)
			label = gtk_label_new_with_mnemonic(content);
		else
			label = gtk_label_new_with_mnemonic("");
		gtk_widget_show(label);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		}
	else if ((content = xmlGetProp(parentnode, "label")))
		widget = gtk_button_new_with_mnemonic(content);
	debuglog(30, "Creating button: %s with label: %s",
	         xmlGetProp(parentnode, "name"), content);

	gtk_widget_show(widget);
	return widget;
	}
