/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_display.h"
#include "bc.h"
#include "uidropdown.h"
#include "parse.h"

GList *
uigtk_dropdown_populatelist(void *bond, xmlNodePtr poplist)
	{
	GList *list = NULL;
	xmlNodePtr node = NULL;

	/* Assume the DTD is correct and just parse the children of <populatelist>.
	   xmlNodeGetContent calls xmlStrdup() to allocate space for the content */
	for (node = poplist->children; node; node = node->next)
		{
		if (IsEqual(node->name, "listitem"))
			list = g_list_append(list, xmlNodeGetContent(node));
		}

	if (!list)
		debugmsg("Nothing in the populated list! Line number: %d",
		         xmlGetLineNo(poplist));

	return list;
	}

static void
uigtk_dropdown_buildlistfree(GList * list)
	{
	GList *walk;

	for (walk = g_list_first(list); walk; walk = walk->next)
		g_free(walk->data);
	g_list_free(list);
	}

GtkWidget *
uigtk_dropdown_create(xmlNode * parentnode, GtkWidget * parentwidget)
	{
	xmlNodePtr node;
	GList *popdownlist;

	/* GList *walk; */
	gchar *content = NULL;
	GtkWidget *widget;

	widget = gtk_combo_new();
	/* widget = gtk_combo_box_new_text(); */
	gtk_widget_show(widget);
	debuglog(50, "Creating dropdown box %s", parentnode->name);

	content = xmlGetProp(parentnode, "visible");
	if (content && IsFalse(content))
		gtk_widget_hide(widget);

	for (node = parentnode->children; node; node = node->next)
		{
		if (IsEqual(node->name, "populatelist"))
			{
			popdownlist = uigtk_dropdown_populatelist(NULL, node);
			gtk_combo_set_popdown_strings(GTK_COMBO(widget), popdownlist);

#if 0
			/* append each string in the list to the dropdown box */
			for (walk = g_list_first(popdownlist); walk; walk = walk->next)
				gtk_combo_box_append_text(GTK_COMBO(widget), walk->data);
#endif

			uigtk_dropdown_buildlistfree(popdownlist);
			}
		}

	return widget;
	}

void
uigtk_dropdown_populate(UiXML * uixml, GtkWidget * widget, GList * list)
	{
	gtk_combo_set_popdown_strings(GTK_COMBO(widget), list);

#if 0

	GList *walk;

	for (walk = g_list_first(list); walk; walk = walk->next)
		gtk_combo_box_append_text(GTK_COMBO(widget), walk->data);
#endif
	}

GtkWidget *
uigtk_dropdown_getentrybox(UiXML * uixml, GtkCombo * widget)
	{
	return widget->entry;
	}
/**
 * uigtk_dropdown_clear:
 * 
 * Empties the contents of the dropdown box.
 * 
 * 
 */ 
void
uigtk_dropdown_clear(UiXML * uixml, GtkCombo * widget)
	{
	GtkList *child;

	child = GTK_LIST(GTK_COMBO(widget)->list);
	gtk_list_clear_items(child, 0, -1);
	}
