
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "bc.h"
#include "uilist.h"
#include "parse.h"
#include "xml_funcs.h"

static GtkTreeViewColumn *
uigtk_list_createsecretcolumn()
	{
	struct UiColumn uic;
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;
	gboolean visible = TRUE;

	renderer = gtk_cell_renderer_text_new();
	visible = FALSE;
	uic.sortable = FALSE;
	uic.clickable = FALSE;
	uic.resizable = FALSE;
	uic.reorderable = FALSE;
	uic.spacing = 0;
	uic.sizing = GTK_TREE_VIEW_COLUMN_FIXED;
	column =
	    gtk_tree_view_column_new_with_attributes("", renderer, "text", 0, NULL);

	gtk_tree_view_column_set_visible(column, visible);
	gtk_tree_view_column_set_clickable(column, uic.clickable);
	gtk_tree_view_column_set_resizable(column, uic.resizable);
	gtk_tree_view_column_set_reorderable(column, uic.reorderable);
	gtk_tree_view_column_set_spacing(column, uic.spacing);
	gtk_tree_view_column_set_sizing(column, uic.sizing);
	return column;
	}

/*G_TYPE_CHAR,
  G_TYPE_UCHAR,
  G_TYPE_BOOLEAN,
  G_TYPE_INT,
  G_TYPE_UINT,
  G_TYPE_LONG,
  G_TYPE_ULONG,
  G_TYPE_ENUM,
  G_TYPE_FLAGS,
  G_TYPE_FLOAT,
  G_TYPE_DOUBLE,
  G_TYPE_STRING,
  G_TYPE_POINTER,
  G_TYPE_BOXED,
  G_TYPE_PARAM,
  G_TYPE_OBJECT */
static GType
uigtk_list_getcolumn_gtype(xmlNode * widgetnode)
	{
	gchar *content;
	GType retval = G_TYPE_STRING;

	content = xmlGetProp(widgetnode, "type");
	if (content)
		{
		if (IsEqual(content, "char") || IsEqual(content, "g_type_char"))
			retval = G_TYPE_CHAR;
		else if (IsEqual(content, "boolean") || IsEqual(content, "bool")
		         || IsEqual(content, "g_type_boolean"))
			retval = G_TYPE_BOOLEAN;
		else if (IsEqual(content, "int") || IsEqual(content, "number")
		         || IsEqual(content, "g_type_int"))
			retval = G_TYPE_INT;
		else if (IsEqual(content, "uint") || IsEqual(content, "g_type_uint"))
			retval = G_TYPE_UINT;
		else if (IsEqual(content, "long") || IsEqual(content, "g_type_long"))
			retval = G_TYPE_LONG;
		else if (IsEqual(content, "ulong") || IsEqual(content, "g_type_ulong"))
			retval = G_TYPE_ULONG;
		else if (IsEqual(content, "float") || IsEqual(content, "g_type_float"))
			retval = G_TYPE_FLOAT;
		else if (IsEqual(content, "double")
		         || IsEqual(content, "g_type_double"))
			retval = G_TYPE_DOUBLE;
		else if (IsEqual(content, "object")
		         || IsEqual(content, "g_type_object"))
			retval = G_TYPE_OBJECT;
		else if (IsEqual(content, "pointer")
		         || IsEqual(content, "g_type_pointer"))
			retval = G_TYPE_POINTER;
		}
	return retval;
	}

void
uilist_celledited_callback(GtkCellRendererText * cell,
                           gchar * path_string, gchar * new_text,
                           gpointer user_data)
	{
	GValue *a;
	GtkWidget *view;
	GtkTreeModel *model;
	GtkListStore *store;
	GtkTreeIter iter;
	guint column_number;
	glong secretid;
	UiXML *bondxml;

	/* Function for callback to bond library */
	typedef void (*function) (UiXML * uixml, UiWidget * widget,
	                          glong secretid, guint colnumber, gchar * text);
	function func;

	/* Figure out some details about what is actually going on */
	column_number = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(cell),
	                                 "my_column_num"));
	bondxml = g_object_get_data(G_OBJECT(cell), "bondxml");

	message("render: edit %s %s %d %d\n", path_string, new_text,
	        column_number, bondxml->widgetcounter);
	/* work out position in list and find the relevant structers */
	view = user_data;
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
	gtk_tree_model_get_iter_from_string(model, &iter, path_string);
	gtk_tree_model_get(model, &iter, 0, &secretid, -1);

	store = GTK_LIST_STORE(model);
	/* Change contents which is in the list */
	a = g_malloc0(sizeof(GValue));
	g_value_init(a, G_TYPE_STRING);
	g_value_set_string(a, new_text);

	gtk_list_store_set_value(store, &iter, column_number, a);
	g_value_unset(a);
	g_free(a);

	/* Call back to bond the changes */
	func = bondxml->plugin->func_gridedit->ptr;
	func(bondxml, view, secretid, column_number, new_text);
	}

/**
 * uigtk_list_createcolumn:
 * @widgetnode: a xml node in the bond XML file.
 * @colnumber: Reference number for the column.
 * @coltype: What type of data is stored in column.
 * @sortable: Specifiy if this is sortable.
 * 
 * This is a internal function, used by uigtk_list_create()
 * to generate the columns in a list.
 *
 * Options:
 * <title> Name of header that appears.
 * <visible> If column is hidden or not.
 * <sortable> specify if the column can be sortable.
 * <clickable> If the column can be clicked on or not.
 * <resizable> If the column is resiable or not.
 * <reorderable> If columns can be moved around or not.
 * <spacing> Pixels between columns.
 * <sizing> Can be growonly, autoresize, or fixed.
 *
 * Example:
 * <label>label txt</label>
 *
 * Returns: Pointer to the widget created.
 */
GtkTreeViewColumn *
uigtk_list_createcolumn(xmlNode * widgetnode, struct UiColumn uic,
                        UiXML * bondxml, GtkWidget * widget, gint * editable)
	{
	GtkTreeViewColumn *column = NULL;
	GtkCellRenderer *renderer;
	gboolean visible = TRUE;
	gchar *content, *title = NULL;
	xmlNodePtr parent = NULL;

	/* Get the title property from the column element */
	content = xmlGetProp(widgetnode, "title");
	if (content)
		title = content;

	/* Get the parent of the column */
	parent = widgetnode->parent;
	content = xmlGetProp(widgetnode, "sortable");
	if (content && IsEqual(content, "true"))
		uic.sortable = TRUE;

	content = xmlGetProp(widgetnode, "visible");
	if (content && IsFalse(content))
		visible = FALSE;

	content = xmlGetProp(widgetnode, "clickable");
	if (content && IsFalse(content))
		uic.clickable = FALSE;

	content = xmlGetProp(widgetnode, "resizable");
	if (content && IsFalse(content))
		uic.resizable = FALSE;
	else if (content)
		uic.resizable = TRUE;

	content = xmlGetProp(widgetnode, "reorderable");
	if (content && IsEqual(content, "true"))
		uic.reorderable = TRUE;
	else if (content)
		uic.reorderable = FALSE;

	content = xmlGetProp(widgetnode, "spacing");
	if (content)
		uic.spacing = atoi(content);

	content = xmlGetProp(widgetnode, "sizing");
	if (content)
		{
		if (IsEqual(content, "growonly"))
			uic.sizing = GTK_TREE_VIEW_COLUMN_GROW_ONLY;
		else if (IsEqual(content, "autosize"))
			uic.sizing = GTK_TREE_VIEW_COLUMN_AUTOSIZE;
		else if (IsEqual(content, "fixed"))
			uic.sizing = GTK_TREE_VIEW_COLUMN_FIXED;
		}

	content = xmlGetProp(widgetnode, "fixedwidth");
	if (content)
		uic.fixedwidth = atoi(content);

	content = xmlGetProp(widgetnode, "maxwidth");
	if (content)
		uic.maxwidth = atoi(content);

	content = xmlGetProp(widgetnode, "minwidth");
	if (content)
		uic.minwidth = atoi(content);
	   
	content = xmlGetProp(widgetnode, "editable");
	if (content && IsEqual(content, "true"))
		{
		uic.editable = TRUE;
		(*editable)++;
		}
	else if (content && IsEqual(content, "false"))
		uic.editable = FALSE;

	renderer = gtk_cell_renderer_text_new();
	/* Set up callback for editing if needed */
	/* m.essage("render : set editable = %d : %s :
	   %s",uic.editable,title,content); */
	if (uic.editable == TRUE)
		g_object_set(renderer, "editable", TRUE, NULL);

	g_object_set_data(G_OBJECT(renderer), "my_column_num",
	                  GUINT_TO_POINTER(uic.colnumber));
	g_object_set_data(G_OBJECT(renderer), "bondxml", bondxml);
	if (uic.editable == TRUE)
		g_signal_connect(renderer, "edited",
		                 (GCallback) uilist_celledited_callback, widget);
	/* Needs to be coded, for handling stuff other than g_type_string. */
	if (uic.coltype == G_TYPE_STRING)
		{
		if (title)
			column =
			    gtk_tree_view_column_new_with_attributes(title, renderer, "text",
			            uic.colnumber, NULL);
		else
			column =
			    gtk_tree_view_column_new_with_attributes("", renderer, "text",
			            uic.colnumber, NULL);
		}
	else
		{
		warningmsg("Unsupported column type.");
		return NULL;
		}
	/* set the attributes */
	if (uic.fixedwidth > 0)
		gtk_tree_view_column_set_fixed_width(column, uic.fixedwidth);

	if (uic.minwidth > 0)
		gtk_tree_view_column_set_min_width(column, uic.minwidth);

	if (uic.maxwidth > 0)
		gtk_tree_view_column_set_max_width(column, uic.maxwidth);
	gtk_tree_view_column_set_visible(column, visible);
	gtk_tree_view_column_set_clickable(column, uic.clickable);
	if (uic.clickable == TRUE)
		gtk_tree_view_column_clicked(column);
	gtk_tree_view_column_set_resizable(column, uic.resizable);
	gtk_tree_view_column_set_reorderable(column, uic.reorderable);
	if (uic.sortable == TRUE)
		{
		gtk_tree_view_column_set_sort_indicator(column, TRUE);
		gtk_tree_view_column_set_sort_column_id(column, uic.colnumber);
		}
	gtk_tree_view_column_set_spacing(column, uic.spacing);
	gtk_tree_view_column_set_sizing(column, uic.sizing);
	/* setup name for widget, so bondxml sees it */

	/* if (title != NULL) xmlNewProp(widgetnode, "name", title); */
	/* Warning, because a column isn't a widget it can not be properly
	   extracted from the hash tables and treated as a widget. */

	uixml_parsename(widgetnode, (UiWidget *) column, bondxml);
	/* done */

	return column;
	}

static void
uigtk_colfree(void *ptr)
	{
	mem_free(ptr);
	}

gint
uigtk_blockemit(GtkWidget * widget, gint state)
	{
	UiXML *bondxml;

	/* Function for callback to bond library */
	typedef gint(*function) (UiXML * uixml, gint state);
	function func;

	bondxml = g_object_get_data(G_OBJECT(widget), "bondxml");
	if (bondxml == NULL)
		{
		errormsg("Unable to find bondxml object in widget.");
		return 0;
		}

	/* Call back to bond the changes */
	func = bondxml->plugin->func_blockemit->ptr;
	return func(bondxml, state);
	}

static void
uigtk_sig_cursorchange(GtkTreeView * view, gpointer user_data)
	{
	GtkTreePath *path;
	GtkTreeViewColumn *column;	  /* *othercolumn; */
	GtkTreeModel *model = NULL;
	GtkTreeSelection *selection;
	GtkTreeIter iter, *iterptr = NULL;

	if (uigtk_blockemit(GTK_WIDGET(view), -1))
		return ;
	uigtk_blockemit(GTK_WIDGET(view), 1);
	column = (GtkTreeViewColumn *) user_data;
	/* Get selection made by user */
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	iterptr = &iter;
	if (!iterptr || !model)
		return ;
	path = gtk_tree_model_get_path(model, iterptr);
	/* m.essage("Path is %s",gtk_tree_path_to_string (path)); */

	/* gtk_tree_view_get_cursor (view,&path,&othercolumn); message("othercol is
	   %s", gtk_tree_view_column_get_title(othercolumn)); */

	gtk_tree_view_set_cursor(view, path, column, FALSE);
	/* gtk_tree_view_get_cursor (view,&path,&othercolumn); message("othercol is
	   %s", gtk_tree_view_column_get_title(othercolumn)); */

	/* m.essage( "Focus set %s %s!",gtk_tree_path_to_string (path),
	   gtk_tree_view_column_get_title(column)); */

	uigtk_blockemit(GTK_WIDGET(view), 0);
	return ;
	}

/**
 * uigtk_list_create:
 * @widgetnode: a xml node in the bond XML file.
 * @parentwidget: widget that will be the parent of this.
 *
 * Creates a label based on properties specified in the XML
 * @node. It makes the widget visible by default.  
 *
 * Options:
 * <source>
 * <column> See details in uigtk_list_createcolumn() on options here.
 * <sortable> specify if the columns can be sortable.
 * <clickable> If the columns can be clicked on or not.
 * <resizable> If the columns is resiable or not
 * <reorderable> If columns can be moved around or not 
 * <spacing> Spacing in pixels between columns
 * <sizing> Can be growonly, autoresize, or fixed.
 * 
 * Example:
 * <label>label txt</label>
 *
 * Returns: Pointer to the widget created.
 */
GtkWidget *
uigtk_list_create(xmlNode * widgetnode, GtkWidget * parentwidget,
                  GtkWidget ** scrolledwindow, UiXML * bondxml)
	{
	struct UiColumn uic;
	gboolean visible = TRUE, isscrolledwindow = TRUE;
	GType colcurtype, *coltypes = NULL;
	guint colcount = 1, editable = 0;
	xmlNode *node;
	guint border = 0;
	gchar *content = NULL;
	GtkListStore *store;
	GtkWidget *widget;
	GtkTreeViewColumn *column, *focuscolumn = NULL;

	memset(&uic, 0, sizeof(struct UiColumn));
	uic.sortable = TRUE;
	uic.clickable = FALSE;
	uic.resizable = TRUE;
	uic.reorderable = TRUE;
	uic.fixedwidth = -1;
	uic.maxwidth = -1;
	uic.minwidth = -1;

	coltypes = (GType *) mem_realloc(coltypes, sizeof(GType) * (colcount));
	coltypes[0] = G_TYPE_ULONG;

	*scrolledwindow = NULL;
	debuglog(50, "Creating a list %s", widgetnode->name);

	content = xmlGetProp(widgetnode, "sortable");
	if (content && IsTrue(content))
		uic.sortable = TRUE;
	else
		uic.sortable = FALSE;

	content = xmlGetProp(widgetnode, "visible");
	if (content && IsFalse(content))
		visible = FALSE;

	content = xmlGetProp(widgetnode, "scrolledwindow");
	if (content && IsFalse(content))
		isscrolledwindow = FALSE;

	content = xmlGetProp(widgetnode, "clickable");
	if (content && IsFalse(content))
		uic.clickable = FALSE;
	else
		uic.clickable = TRUE;

	content = xmlGetProp(widgetnode, "resizable");
	if (content && IsFalse(content))
		uic.resizable = FALSE;
	else
		uic.resizable = TRUE;

	content = xmlGetProp(widgetnode, "reorderable");
	if (content && IsTrue(content))
		uic.reorderable = TRUE;
	else
		uic.reorderable = FALSE;

	content = xmlGetProp(widgetnode, "spacing");
	if (content)
		uic.spacing = atoi(content);

	content = xmlGetProp(widgetnode, "sizing");
	if (content)
		{
		if (IsEqual(content, "growonly"))
			uic.sizing = GTK_TREE_VIEW_COLUMN_GROW_ONLY;
		else if (IsEqual(content, "autosize"))
			uic.sizing = GTK_TREE_VIEW_COLUMN_AUTOSIZE;
		else if (IsEqual(content, "fixed"))
			uic.sizing = GTK_TREE_VIEW_COLUMN_FIXED;
		}

	content = xmlGetProp(widgetnode, "editable");
	if (content && IsFalse(content))
		uic.editable = FALSE;
	else if (content)
		uic.editable = TRUE;

	content = xmlGetProp(widgetnode, "border");
	if (content)
		border = atoi(content);

	/* Initial parse to work out what data to store */
	/* uic is set twice, once to decide the defaults, then again for the
	   individual values */
	for (node = widgetnode->children; node; node = node->next)
		{
		if (IsEqual(node->name, "column"))
			{
			colcurtype = uigtk_list_getcolumn_gtype(node);
			coltypes = (GType *) mem_realloc(coltypes,
			                                 sizeof(GType) * (colcount + 1));
			coltypes[colcount] = colcurtype;
			colcount++;
			continue;
			}
		}

	if (colcount == 1)
		return NULL;
	bondxml->garbage = mem_addtogarbage(bondxml->garbage, colcount,
	                                    uigtk_colfree, coltypes);
	debuglog(100, "Creating liststore with %d columns", colcount);
	store = gtk_list_store_newv(colcount, coltypes);

	widget = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection
	                            (GTK_TREE_VIEW(widget)), GTK_SELECTION_BROWSE);
	g_object_set_data(G_OBJECT(widget), "bondxml", bondxml);

	/* Reparse, this time do some more */
	colcount = 1;
	column = uigtk_list_createsecretcolumn();
	gtk_tree_view_append_column(GTK_TREE_VIEW(widget), column);
	for (node = widgetnode->children; node; node = node->next)
		{
		if (IsEqual(node->name, "column"))
			{
			uic.colnumber = colcount;
			uic.coltype = coltypes[colcount];

			column = uigtk_list_createcolumn(node, uic, bondxml,
			                                 widget, &editable);
			/* If there is one editable column, default to always have that
			   one selected. */
			if (editable == 1)
				{
				focuscolumn = column;
				editable++;
				}

			gtk_tree_view_append_column(GTK_TREE_VIEW(widget), column);
			colcount++;
			}
		}
	if (focuscolumn != NULL && editable == 2)
		g_signal_connect(G_OBJECT(widget),
		                 "cursor_changed",
		                 G_CALLBACK(uigtk_sig_cursorchange), (void *)focuscolumn);

	if (visible == TRUE)
		gtk_widget_show(widget);
	if (isscrolledwindow)
		{
		*scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(*scrolledwindow),
		                               GTK_POLICY_AUTOMATIC,
		                               GTK_POLICY_AUTOMATIC);

		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW
		                                      (*scrolledwindow), widget);
		/* if (border) { attention("setting border to %d",border);
		   gtk_container_set_border_width( GTK_CONTAINER(*scrolledwindow),
		   border); } */
		if (visible == TRUE)
			gtk_widget_show(*scrolledwindow);
		}
	else if (border)
		gtk_container_set_border_width(GTK_CONTAINER(widget), border);
	return widget;
	}

static GtkTreeModel *
uigtk_list_getstore(GtkWidget * widget)
	{
	g_assert(widget);
	return gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	}

void
uigtk_list_clear(UiXML * uixml, GtkWidget * widget)
	{
	GtkListStore *store;

	store = GTK_LIST_STORE(uigtk_list_getstore(widget));
	g_assert(store);
	gtk_list_store_clear(store);
	}

GList *
uigtk_list_getrow(UiXML * uixml, GtkWidget * widget, gulong secretid)
	{
	GtkTreeIter iter;
	GtkTreeModel *model;

	warningmsg("This isn't coded yet");
	model = GTK_TREE_MODEL(uigtk_list_getstore(widget));
	/* move to root item */
	gtk_tree_model_get_iter_first(model, &iter);
	/* Get a column value. */
	gtk_tree_model_get(model, &iter, -1);
	/* get the next row */
	gtk_tree_model_iter_next(model, &iter);
	return NULL;
	}

gint
uigtk_list_setcellvalue(UiXML * uixml, GtkWidget * widget, gint col,
                        gulong secretid, gchar * value)
	{
	gboolean found = FALSE;
	gulong gotvalue = 0;
	GValue *a;
	GValue *b;
	GtkTreeIter iter;
	GtkListStore *store;
	GtkTreeModel *model;

	model = uigtk_list_getstore(widget);
	store = GTK_LIST_STORE(model);
	/* move to secretid item */
	gtk_tree_model_get_iter_first(model, &iter);
	b = g_malloc0(sizeof(GValue));

	do
		{
		gtk_tree_model_get_value(model, &iter, 0, b);
		gotvalue = g_value_get_ulong(b);

		if (gotvalue == secretid)
			{
			found = TRUE;
			g_value_unset(b);
			break;
			}
		g_value_unset(b);
		}
	while (gtk_tree_model_iter_next(model, &iter));

	if (found == FALSE)
		return -1;

	a = g_malloc0(sizeof(GValue));
	g_value_init(a, G_TYPE_STRING);
	g_value_set_string(a, value);

	gtk_list_store_set_value(store, &iter, col + 1, a);
	g_value_unset(a);
	g_free(a);
	return 0;
	}

/* Please ensure GList length is less than num columns */
void
uigtk_list_append(UiXML * uixml, GtkWidget * widget, GList * values,
                  gulong secretid, gboolean append)
	{
	GList *walk;
	GtkTreeIter iter;
	GtkListStore *store;
	GValue *a;
	gint i = 1;

	/* Initial work to build widget information */
	g_assert(widget);
	store = GTK_LIST_STORE(uigtk_list_getstore(widget));
	g_assert(store);
	/* if append is true then we add the row onto the end of the list */
	if (append == FALSE)
		gtk_list_store_prepend(store, &iter);
	else
		gtk_list_store_append(store, &iter);
	/* add secret column */
	a = g_malloc0(sizeof(GValue));
	g_value_init(a, G_TYPE_ULONG);
	g_value_set_ulong(a, secretid);
	gtk_list_store_set_value(store, &iter, 0, a);
	g_value_unset(a);
	g_free(a);
	/* add rest of columns */
	for (walk = g_list_first(values); walk; walk = walk->next)
		{
		a = g_malloc0(sizeof(GValue));
		g_value_init(a, G_TYPE_STRING);
		g_value_set_string(a, walk->data);

		gtk_list_store_set_value(store, &iter, i, a);
		g_value_unset(a);
		g_free(a);
		i++;
		}
	}

static gint
uigtk_list_numcol(GtkWidget * widget)
	{
	return
	    gtk_tree_model_get_n_columns(GTK_TREE_MODEL(uigtk_list_getstore(widget)));
	}

gboolean
uigtk_list_setselection(UiXML * uixml, GtkWidget * widget, gulong secretid)
	{
	guint secret;
	GtkTreeIter iter;
	GtkTreePath *path;
	GtkTreeSelection *selection;
	GtkTreeModel *model;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
	model = uigtk_list_getstore(widget);
	if (gtk_tree_model_get_iter_first(model, &iter))
		while (1)
			{
			gtk_tree_model_get(model, &iter, 0, &secret, -1);
			if (secret == secretid)
				{
				path = gtk_tree_model_get_path(model, &iter);
				gtk_tree_selection_select_path(selection, path);
				gtk_tree_path_free(path);
				return TRUE;
				}
			if (!gtk_tree_model_iter_next(model, &iter))
				break;
			}
	return FALSE;
	}

gulong
uigtk_list_getselection(UiXML * uixml, GtkWidget * widget, gboolean getvalues,
                        GList ** values)
	{
	gint numcol, i, a;
	gulong retsecret = -1;
	GList *retlist = NULL;
	gchar *tmpstr;
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
	if (gtk_tree_selection_get_selected(selection, &model, &iter) == TRUE)
		{
		gtk_tree_model_get(model, &iter, 0, &a, -1);
		retsecret = a;
		if (getvalues == TRUE)
			{
			*values = NULL;
			numcol = uigtk_list_numcol(widget);
			for (i = 1; i < numcol; i++)
				{
				gtk_tree_model_get(model, &iter, i, &tmpstr, -1);
				retlist = g_list_append(retlist, g_strdup(tmpstr));
				}
			*values = retlist;
			}
		}
	else
		return -2;
	return retsecret;
	}
