#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <gtk/gtk.h>

#include "uimessage.h"

void
uigtk_message_close(GtkWidget * widget, void *toclose)
	{
	gtk_widget_hide(GTK_WIDGET(toclose));
	}
/**
 * uigtk_message_display:
 * message: Message to display in dialog box.
 * title: title of dialog box.
 * type: Type of message box. 0 for Ok. 1 for Cancel, Ok. 2 for No, Yes
 *
 * Returns: negitive on cancel or no, else 0.
 */
gint
uigtk_message_display(gchar * message, gchar * title, gint type)
	{
	gint retval;
	static gchar notitle[40] = "BOND Message";
	GtkWidget *window;
	GtkWidget *dialog_vbox;
	GtkWidget *label;
	GtkWidget *dialog_action_area;
	GtkWidget *closebutton;
	GtkWidget *cancelbutton;
	GtkWidget *okbutton;

	g_assert(message);
	if (title == NULL)
		title = notitle;

	/* create a new window */
	window = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(window), title);

	/* sets the border width of the window. */
	gtk_container_set_border_width(GTK_CONTAINER(window), 10);

	/* set up vbox */
	dialog_vbox = GTK_DIALOG(window)->vbox;
	gtk_widget_show(dialog_vbox);

	/* display message */
	label = gtk_label_new(message);
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(dialog_vbox), label, TRUE, TRUE, 0);
  	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
  	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);	
	
	/* dialog area at bottom of box. */
	dialog_action_area = GTK_DIALOG(window)->action_area;
	gtk_widget_show(dialog_action_area);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area),
	                          GTK_BUTTONBOX_END);

	if (type ==0)
		{
		closebutton = gtk_button_new_from_stock("gtk-close");
		gtk_widget_show(closebutton);
		gtk_dialog_add_action_widget(GTK_DIALOG(window), closebutton,
	                             	GTK_RESPONSE_CLOSE);
		GTK_WIDGET_SET_FLAGS(closebutton, GTK_CAN_DEFAULT);
	
		/* When the button receives the "clicked" signal, it will call the func$ The
		   hello() function is defined above. */
		g_signal_connect(G_OBJECT(closebutton), "clicked",
	                 G_CALLBACK(uigtk_message_close), window);
		}
	else
	if (type==1)
		{
	  	cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  		gtk_widget_show (cancelbutton);
		gtk_dialog_add_action_widget (GTK_DIALOG (window), cancelbutton, GTK_RESPONSE_CANCEL);
		GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

	  	okbutton = gtk_button_new_from_stock ("gtk-ok");
  		gtk_widget_show (okbutton);
  		gtk_dialog_add_action_widget (GTK_DIALOG (window), okbutton, GTK_RESPONSE_OK);
  		GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

		g_signal_connect(G_OBJECT(okbutton), "clicked",
	                 G_CALLBACK(uigtk_message_close), window);
		g_signal_connect(G_OBJECT(cancelbutton), "clicked", 
	                 G_CALLBACK(uigtk_message_close), window); 
		}
	else
	if (type==2)
		{
	  	okbutton = gtk_button_new_from_stock ("gtk-yes");
  		gtk_widget_show (okbutton);
  		gtk_dialog_add_action_widget (GTK_DIALOG (window), okbutton, GTK_RESPONSE_YES);
  		GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);
	  	
		cancelbutton = gtk_button_new_from_stock ("gtk-no");
  		gtk_widget_show (cancelbutton);
		gtk_dialog_add_action_widget (GTK_DIALOG (window), cancelbutton, GTK_RESPONSE_NO);
		GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

		g_signal_connect(G_OBJECT(okbutton), "clicked",
	                 G_CALLBACK(uigtk_message_close), window);
		g_signal_connect(G_OBJECT(cancelbutton), "clicked",
	                 G_CALLBACK(uigtk_message_close), window);
		}

	g_signal_connect(G_OBJECT(window), "delete_event",
	                 G_CALLBACK(gtk_widget_destroy), window);
	g_signal_connect(G_OBJECT(window), "close", G_CALLBACK(gtk_widget_destroy),
					window); 
	/* display the widget */
	retval = gtk_dialog_run(GTK_DIALOG(window));
	if (type == 0)
			return 0;
	if (type == 1 && retval == GTK_RESPONSE_CANCEL)
			return -1;
	if (type == 2 && retval == GTK_RESPONSE_NO)
			return -1;
	return 0;
	}
