#include <stdio.h>
#include <glib.h>

#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "uigtk.h"
#include "bc.h"

#include "uigeneral.h"
#include "uisignal.h"

/**
 * uigtk_signalconnect:
 * bondxml: XML document
 * object: Widget to add signal
 * name: Name of the signal (ie clicked)
 * argfunc: Function to be called on @name event.
 * func_data: Custom data to passed to @argfunc.
 * 
 * Returns: Signal id.
 */
guint
uigtk_signalconnect(UiXML * bondxml, UiWidget * object, const gchar * name,
                    void (*argfunc) (), void *func_data)
	{
	GtkTextBuffer *buffer;
	GtkTreeSelection *select;

	g_assert(object);
	if (uigtk_gettype(bondxml, object) == UiDropDown)
		object = uigtk_dropdown_getentrybox(bondxml, object);

	if (uigtk_gettype(bondxml, object) == UiGrid)
		{
		select = gtk_tree_view_get_selection(GTK_TREE_VIEW(object));
		return g_signal_connect(G_OBJECT(select), name, G_CALLBACK(argfunc),
		                        func_data);
		}
	   
	if (uigtk_gettype(bondxml, object) == UiTextBox)
		{
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(object));
		return g_signal_connect(G_OBJECT(buffer), name, G_CALLBACK(argfunc),
		                        func_data);

		}

	return g_signal_connect(G_OBJECT(object), name, G_CALLBACK(argfunc),
	                        func_data);
	}

/**
 * uigtk_signaldisconnectbyfunc:
 * 
 * Suspect bug in gtk stops this function from working correctly. 
 */
void
uigtk_signaldisconnectbyfunc(UiXML * bondxml, UiWidget * object,
                             void (*argfunc) (), void *data)
	{
	GtkTreeSelection *select;
	GtkTextBuffer *buffer;

	g_assert(object);
	if (uigtk_gettype(bondxml, object) == UiDropDown)
		object = uigtk_dropdown_getentrybox(bondxml, object);

	if (uigtk_gettype(bondxml, object) == UiGrid)
		{
		select = gtk_tree_view_get_selection(GTK_TREE_VIEW(object));
		g_signal_handlers_disconnect_by_func(G_OBJECT(select),
		                                     G_CALLBACK(argfunc), data);
		}
	else if (uigtk_gettype(bondxml, object) == UiTextBox)
		{
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(object));
		g_signal_handlers_disconnect_by_func(G_OBJECT(buffer),
		                                     G_CALLBACK(argfunc), data);
		}
	else
		{

		/* int nu; */
		g_signal_handlers_disconnect_by_func(G_OBJECT(object),
		                                     G_CALLBACK(argfunc), data);
		/* if (nu) debugmsg("%d signals disconnected on
		   %s",nu,uigtk_getname(bondxml,object)); */
		}

	}

/**
 * uigtk_signaldisconnect:
 * bondxml: Bond uixml documentent.
 * object: widget to disconnect signals on.
 * signalid: Signal id returned by uigtk_signalconncet().
 * 
 * 
 */
void
uigtk_signaldisconnect(UiXML * bondxml, UiWidget * object, gint signalid)
	{
	GtkTreeSelection *select;
	GtkTextBuffer *buffer;

	g_assert(object);
	if (uigtk_gettype(bondxml, object) == UiDropDown)
		object = uigtk_dropdown_getentrybox(bondxml, object);
	if (uigtk_gettype(bondxml, object) == UiGrid)
		{
		select = gtk_tree_view_get_selection(GTK_TREE_VIEW(object));
		g_signal_handler_disconnect(G_OBJECT(select), signalid);
		}
	else if (uigtk_gettype(bondxml, object) == UiTextBox)
		{
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(object));
		g_signal_handler_disconnect(G_OBJECT(buffer), signalid);
		}

	else
		g_signal_handler_disconnect(G_OBJECT(object), signalid);
	}

static void
uigtk_signalconnect_keypress_func(UiWidget * widget, GdkEventKey * event,
                                  void *ptr)
	{
	gint tostop;
	UiSignalKey *key;
	typedef gint(*function) (void *, gchar * str);
	function func;

	key = ptr;
	func = key->func;
	tostop = func(key->userdata, event->string);
	/* stop event key press */
	if (tostop == 1)
		g_signal_stop_emission_by_name(G_OBJECT(widget), "key_press_event");
	}

gint
uigtk_signalconnect_keypress(UiXML * bondxml, UiWidget * widget,
                             UiSignalKey * uisignalkey)
	{
	g_assert(bondxml);
	g_assert(widget);

	uisignalkey->uixml = bondxml;
	uisignalkey->parentcallback = uigtk_signalconnect_keypress_func;
	uisignalkey->signal_id = g_signal_connect(G_OBJECT(widget),
	                         "key_press_event",
	                         G_CALLBACK
	                         (uigtk_signalconnect_keypress_func),
	                         uisignalkey);

	return 0;
	}

gint
uigtk_signaldisconnect_keypress(UiXML * bondxml, UiWidget * widget,
                                UiSignalKey * signalkey)
	{
	g_assert(bondxml);
	g_assert(widget);
	g_signal_handler_disconnect(G_OBJECT(widget), signalkey->signal_id);
	return 0;
	}
