/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_funcs.h"
#include "bc.h"
#include "uitoolbar.h"
#include "uigtk.h"
#include "uigeneral.h"
#include "parse.h"
#include "uigeneral.h"

/*
#include "uibutton.h"
#include "uiradiobutton.h"
#include "uitoggle.h"
#include "uimenu.h"
#include "uicheckbutton.h"
*/

/**
 * uigtk_toolbar_create:
 * @widgetnode: an xmlNodePtr in the bond XML file.
 * @parentwidget: widget that will be the parent of this.
 * 
 * Creates a toolbar.  After creating a toolbar, you can append,
 * prepend and insert items or elements into the toolbar.
 *
 * Properties:
 * orientation - "horizontal | vertical" 
 *     Sets the  orientation of  the toolbar either horizontally
 *     or vertically.
 *
 * style - "both | icons | text | both-beside"
 *     Sets    the toolbar style to    both icons and text, only
 *     icons, only  test or both  icons  and test alongside each
 *     other.
 *
 * tooltips - "true | false"
 *     Sets tooltips to be on or off.
 *
 * Children: radio, check, button, entry, label, menu.
 */
GtkWidget *
uigtk_toolbar_create(xmlNodePtr widgetnode, GtkWidget * parentwidget)
	{
	GtkWidget *widget = NULL;
	gchar *content = NULL;
	GtkOrientation orientation = GTK_ORIENTATION_HORIZONTAL;
	GtkToolbarStyle style = GTK_TOOLBAR_BOTH_HORIZ;
	gboolean tooltips = FALSE;

	content = xmlGetProp(widgetnode, "orientation");
	if (content && IsEqual(content, "vertical"))
		orientation = GTK_ORIENTATION_VERTICAL;

	content = xmlGetProp(widgetnode, "style");
	if (content)
		{
		if (IsEqual(content, "both"))
			style = GTK_TOOLBAR_BOTH;
		else if (IsEqual(content, "icons"))
			style = GTK_TOOLBAR_ICONS;
		else if (IsEqual(content, "text"))
			style = GTK_TOOLBAR_TEXT;
		}

	content = xmlGetProp(widgetnode, "tooltips");
	if (content && IsTrue(content))
		tooltips = TRUE;

	debuglog(50, "Creating toolbar", content);
	widget = gtk_widget_new(GTK_TYPE_TOOLBAR, 
			   "orientation", orientation,
	                   "toolbar-style", style, NULL);
	gtk_toolbar_set_tooltips(GTK_TOOLBAR(widget), tooltips);

	gtk_widget_show(widget);

	return widget;
	}

/**
 * uigtk_toolbar_add:
 * @widget: The current widget.
 * @parentwidget: The toolbar widget.
 *
 * Adds @widget to  the  toolbar, @parentwidget. @widget can  be
 * one of either radio, check, button, entry, label or menu.
 *
 */
gint
uigtk_toolbar_add(GtkWidget * widget, GtkWidget * parentwidget)
	{
	g_assert(widget);
	g_assert(parentwidget);

	/*
	   tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-add",
	   gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1))); button24 =
	   gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
	   GTK_TOOLBAR_CHILD_BUTTON, NULL, _("_Add Employee"), NULL, NULL,
	   tmp_toolbar_icon, NULL, NULL); */
	/* GTK 2.0 has gtk_toolbar_append while GTK 2.4 has gtk_container_add */
	if (GTK_WIDGET_TYPE(widget) != GTK_TYPE_TOOL_ITEM)
	     {	
	     gtk_toolbar_append_widget(GTK_TOOLBAR(parentwidget), widget, NULL, NULL);
	     gtk_widget_show(widget);
	     }	   
	else
	     gtk_container_add (GTK_CONTAINER (parentwidget), GTK_WIDGET(widget));

	return 0;
	}

/**
 * uigtk_toolbar_add:
 * @parentwidget: The toolbar widget.
 *
 * Add a space in the toolbar.
 */
gint
uigtk_toolbar_add_space(GtkWidget * parentwidget)
	{
	g_assert(parentwidget);
	gtk_toolbar_append_space(GTK_TOOLBAR(parentwidget));
	return 0;
	}

GtkWidget*
uigtk_toolbar_toolbutton(xmlNode *widgetnode, GtkWidget * parentwidget)
	{
	GtkToolItem *widget = NULL;
	GtkWidget *image = NULL;
	gchar *content = NULL;
	gint toolbar_icon_size;
	   
   	g_assert(parentwidget);
	g_assert(widgetnode);
	   
	toolbar_icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (parentwidget));
	   
	content = xmlGetProp(widgetnode, "stock");
	if (content)
	     widget = gtk_tool_button_new_from_stock (content);
	else
	     {
	     content = xmlGetProp(widgetnode, "image");
	     if (content)
		  image = gtk_image_new_from_stock (content, toolbar_icon_size);
	     else
		  {
		  content = xmlGetProp(widgetnode, "file");
		  if (content)
		       image = gtk_image_new_from_file(content);
		  }
	     if (image)
		  gtk_widget_show(image);
	     content = xmlGetProp(widgetnode, "label");
	     widget = gtk_tool_button_new(image,content);
	     }
	   
	content = xmlGetProp(widgetnode, "homogeneous");
	if (content && strcmp(content,"true")==0)
	     gtk_tool_item_set_homogeneous(widget,TRUE);
	   
	content = xmlGetProp(widgetnode, "visible");
	if (content && strcmp(content,"false")==0)
	     ;
	else
	     gtk_widget_show(GTK_WIDGET(widget));
	   
	return GTK_WIDGET(widget);
	}

