/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "bc.h"
#include "parse.h"
#include "xml_funcs.h"
#include "xml_display.h"

#include "uidropdown.h"
#include "uilist.h"
#include "uigeneral.h"
#include "uibox.h"
#include "uispinbutton.h"

#include "uiprogressbar.h"
#include "uiimage.h"
#include "uinotebook.h"
#include "uiframe.h"
#include "uiseparator.h"
#include "uibutton.h"
#include "uimenu.h"
#include "uiradiobutton.h"
#include "uicheckbutton.h"
#include "uitoolbar.h"
#include "uitextview.h"
/**
 * uihtml_vhbox_cellcompare:
 * used in uihtml_box_add by g_list_sort to sort cells
 */
gint
uihtml_vhbox_cellcompare(UiHtml * cell1, UiHtml * cell2)
{
  if(cell1->order < cell2->order)
	return -1;
  if(cell1->order > cell2->order)
	return 1;
  return 0;
}

static gchar *
uixml_generateshortname(xmlNode * widgetnode, gchar * widgetname, UiXML * bond)
{
  /* xmlNode *child; */
  gchar *retstr = NULL;

  retstr = xmlGetProp(widgetnode, "name");
  if (!retstr)
    {
      retstr =
		g_strdup_printf("%s%d", widgetnode->name, bond->widgetcounter++);
      xmlNewProp(widgetnode, "name", retstr);
    }
  return retstr;
}

static void
uixml_generatepath(xmlNode * widgetnode, UiXML * bond, gchar ** longname,
                   gchar ** shortname)
{
  xmlNodePtr node;
  gchar *content = NULL;

  *shortname = uixml_generateshortname(widgetnode, *shortname, bond);

  node = widgetnode;
  while (!content && node)
    {
      content = xmlGetProp(node, "longname");
      node = node->parent;
    }
  if (content)
    *longname = g_strdup_printf("%s/%s", content, *shortname);
  else
    *longname = g_strdup(*shortname);
  xmlNewProp(widgetnode, "longname", *longname);

  debuglog(100, "Shortname %s, Longname: %s", *shortname,
		   xmlGetProp(widgetnode, "longname"));
}

static gint
uixml_parsewidget_attribute(xmlNode * node, UiHtml * widget, gchar ** name,
                            UiXML * uixml)
{
  gint minheight = -1, minwidth = -1;
  gchar *content;
  gboolean isscrolledwindow = TRUE;

  /*  g_assert(widget);
   */
  *name = NULL;

  content = xmlGetProp(node, "name");
  if (content)
    *name = content;

  content = xmlGetProp(node, "minwidth");
  if (content)
    minwidth = atoi(content);

  content = xmlGetProp(node, "minheight");
  if (content)
    minheight = atoi(content);

  content = xmlGetProp(node, "scrolledwindow");
  if (content && IsFalse(content))
    isscrolledwindow = FALSE;

  if (minheight > 0 || minwidth > 0)
    {
      if (uihtml_gettype(uixml, widget) == UiGrid && isscrolledwindow == TRUE)
		{
		  minheight -= 5;
		  minwidth -= 3;
		}
      uihtml_setwidgetsize(uixml, widget, minwidth, minheight);
    }

  return 0;
}

static void
uixml_parse_project(UiXML * uixml, xmlNodePtr parentnode)
{
  gchar *content;

  g_assert(uixml);

  content = xmlGetProp(parentnode, "defaultvisible");
  if (content && IsTrue(content))
    uixml->defaultvisible = TRUE;
  else
    uixml->defaultvisible = FALSE;

  content = xmlGetProp(parentnode, "defaultentrywidget");
  if (content)
    uixml->defaultentrywidth = atoi(content);
}

int
uixml_node(xmlNode * node, gchar * name)
{
  if (IsEqual(node->name, name))
    return 1;
  return 0;
}

int
uixml_parseparent(UiXML * bondxml)
{
  xmlNodePtr node;

  node = xmlDocGetRootElement(bondxml->doc);
  /* xml_display_doc(bondxml->doc); */
  node = node->children;
  printf("1321324413241324132413241324132413241325143251432\n\n\n");
  uixml_parse(node, 0, NULL, bondxml);
  printf("1321324413241324132413241324132413241325143251432\n\n\n");
  uixml_showfooter();
  return 0;
}

UiXML *
uixml_init(gchar * filename)
{
  gchar *dtdvalue;
  xmlDoc *doc;
  xmlParserCtxt *ctxt;			  /* the parser context */

  UiXML *bondxml;

  /* create a parser context */
  ctxt = xmlNewParserCtxt();
  if (!ctxt)
    {
      fprintf(stderr, "Failed to allocate parser context\n");
      return NULL;
    }
  /* Check if the file needs to be validated with dtd */
  dtdvalue = bcconfig_getvalue("dtd_validation");
  /* parse the file, activating the DTD validation option */
  if (dtdvalue && IsTrue(dtdvalue))
    doc = xmlCtxtReadFile(ctxt, filename, NULL, XML_PARSE_DTDVALID);
  /* otherwise, parse without validation */
  else
    doc = xmlCtxtReadFile(ctxt, filename, NULL, 0);

  /* check if parsing suceeded */
  if (!doc)
    {
      errormsg("Error parsing xmlfile %s.\n", filename);
      return NULL;
    }
  else
    {
      /* check if validation suceeded */
      if (ctxt->valid == 0)
		{
		  errormsg("Failed to validate %s\n", filename);
		  return NULL;
		}
    }

  bondxml = mem_calloc(sizeof(UiXML));
  bondxml->doc = doc;
  bondxml->defaultvisible = TRUE;
  bondxml->defaultentrywidth = -1;
  /* Set up hash tables which contain widgets and lookups */
  bondxml->widgets =
    g_hash_table_new_full(&g_direct_hash, &g_direct_equal, NULL, NULL);
  bondxml->shortname =
    g_hash_table_new_full(&g_str_hash, &g_str_equal, &g_free, NULL);
  bondxml->longname =
    g_hash_table_new_full(&g_str_hash, &g_str_equal, &g_free, NULL);
  bondxml->garbage = NULL;
  /* This should free up the context (but not the ctxt->doc) */
  xmlFreeParserCtxt(ctxt);
  debugmsg("XML context freed (except the XML tree)");
  
  bondxml->sessionid=NULL;
  
  return bondxml;
}

UiHtml *
uixml_parse(xmlNode * node, ParseState state, UiHtml * parentwidget,
            UiXML * bondxml)
{
  UiHtml *widget = NULL, *tmpwidget, *scrolledwindow = NULL;
  gchar *tmpstr1 = NULL, *tmpstr2 = NULL;
  gboolean visible;
  gchar *name = NULL;

  for (node = node->children; node; node = node->next)
    {
      /* The name of the current node */
      name = (char*)node->name;
      switch (state)
		{
		case ParseStart:
		  if (IsEqual(name, "project"))
			{
			  uixml_parse_project(bondxml, node);
 /*			  widget = uihtml_project_create(bondxml, node);*/
			  uixml_parsename(node, widget, bondxml);
			  tmpwidget = uixml_parse(node, ParseStart, widget, bondxml);
			  
			  if (tmpwidget)
				uihtml_addcontainer(tmpwidget, widget);

			  uixml_showheader(bondxml, node);
			  break;
			}
		  if (IsEqual(name, "window"))
			{
			  widget =  uihtml_window_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  tmpwidget = uixml_parse(node, ParseGeneral, widget, bondxml);
			  if (tmpwidget)
				uihtml_addcontainer(tmpwidget, widget);
			  if (visible == TRUE)
				uihtml_showwidget(bondxml, widget);
			  else
				uihtml_hidewidget(bondxml, widget);
			  
			  break;
			}
		  break;
		case ParseToolBar:
		case ParseHandleBox:
		case ParseGeneral:
		  if (IsEqual(name, "label"))
			{
			  widget = uihtml_label_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "entry"))
			{
			  widget =
				uihtml_entry_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "dropdown"))
			{
			  widget = uihtml_dropdown_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "box"))
			{
			  widget = uihtml_box_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  uixml_parse(node, ParseBox, widget, bondxml);
			  
			  g_list_sort (widget->children, (void *)uihtml_vhbox_cellcompare);
			  break;
			}
		  if (IsEqual(name, "vbox"))
			{
			  widget = uihtml_vhbox_create(bondxml, node, parentwidget, 1, &visible);
			  uixml_parsename(node, widget, bondxml);
			  uixml_parse(node, ParseVHBox, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "hbox"))
			{
			  widget = uihtml_vhbox_create(bondxml, node, parentwidget, 0, &visible);
			  uixml_parsename(node, widget, bondxml);
			  uixml_parse(node, ParseVHBox, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "button"))
			{
			  widget = uihtml_button_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "list"))
			{
			  widget =
				uihtml_list_create(node, parentwidget,
								   bondxml, &visible);
			  if (scrolledwindow)
				uixml_parsewidget_attribute(node, scrolledwindow,
											&tmpstr1, bondxml);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "spin"))
			{
			  widget = uihtml_spinbutton_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
/*		  if (IsEqual(name, "toggle"))
			{
			  widget = uihtml_toggle_button_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
*/
		  if (IsEqual(name, "check"))
			{
			  widget = uihtml_check_button_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "scale"))
			{
			  widget = uihtml_scale_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "separator"))
			{
			  widget = uihtml_separator_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "frame"))
			{
			  widget = uihtml_frame_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "scrollbar"))
			{
			  widget = uihtml_scroll_bar_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "image"))
			{
			  widget = uihtml_image_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "progressbar"))
			{
			  widget = uihtml_progress_bar_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "statusbar"))
			{
			  widget = uihtml_status_bar_create(bondxml, node, parentwidget, &visible);
			  if (!widget)
				break;
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "radio"))
			{
			  widget = uihtml_radio_button_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  uixml_parse(node, ParseRadio, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "notebook"))
			{
			  widget = uihtml_notebook_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  uixml_parse(node, ParseNotebook, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "menubar"))
			{
			  widget = uihtml_menubar_create(bondxml, node, parentwidget);
			  uixml_parsename(node, widget, bondxml);
			  uixml_parse(node, ParseMenuBar, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "toolbar"))
			{
			  widget = uihtml_toolbar_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  uixml_parse(node, ParseToolBar, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "handlebox"))
			{
			  widget =
				uihtml_handlebox_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  uixml_parse(node, ParseHandleBox, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "textview"))
			{
			  widget =
				uihtml_textview_create(bondxml, node, parentwidget);
			  if (scrolledwindow)
				uixml_parsewidget_attribute(node, scrolledwindow,
											&tmpstr1, bondxml);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  if (IsEqual(name, "toolbarspace") && state == ParseToolBar)
			{
			  uihtml_toolbar_add_space(parentwidget);
			  break;
			}

		  break;

		case ParseRadio:
		  if (IsEqual(name, "radio"))
			{
			  widget = uihtml_radio_button_create(bondxml, node, parentwidget, &visible);
			  uixml_parsename(node, widget, bondxml);
			  break;
			}
		  break;
		case ParseBox:
		  if (IsEqual(name, "cell"))
			{
			  widget = uixml_parse(node, ParseGeneral, parentwidget,
								   bondxml);
			  uixml_generatepath(node, bondxml, &tmpstr1, &tmpstr2);
			  if (widget)
				{
				uihtml_box_add(node, widget, parentwidget);
				}
			  break;
			}
		  break;
		case ParseVHBox:
		  if (IsEqual(name, "cell"))
			{
			  widget =
				uixml_parse(node, ParseGeneral, parentwidget,
							bondxml);
			  g_assert(widget);
			  uixml_generatepath(node, bondxml, &tmpstr1, &tmpstr2);
			  if (widget)
				uihtml_vhbox_add(node, widget, parentwidget);
			  break;
			}
		  break;
		case ParseNotebook:
		  if (IsEqual(name, "page"))
			{
			  widget = uixml_parse(node, ParseGeneral, parentwidget, bondxml);
			  uixml_generatepath(node, bondxml, &tmpstr1, &tmpstr2);
			  if (widget)
				uihtml_notebook_addpage(node, widget, parentwidget);
			  break;
			}
		  break;
		case ParseMenuBar:
		  if (IsEqual(name, "menu"))
			{
			  widget = uihtml_menu_create(node, parentwidget, bondxml);
			  uixml_parsename(node, widget, bondxml);
			  if (widget)
				uihtml_menubar_add(widget, parentwidget);
			  break;
			}
		  break;
		default:
		  /* Unknown state */
		  errormsg("Unknown state %d\n", state);
		}

      /* add the widget to its parent if its a special type of container
		 widget */
      if (state == ParseToolBar && widget)
		{
		  uihtml_toolbar_add(node, widget, parentwidget);
		  widget = NULL;
		}
      if (state == ParseHandleBox && widget)
		{
		  uihtml_addcontainer(widget, parentwidget);
		  widget = NULL;
		}
    }
  /* possible memory leak if for example 2 cells occur in same table */
  if (tmpstr1)
    g_free(tmpstr1);
  if (tmpstr2)
    g_free(tmpstr2);
  if (scrolledwindow)
    widget = scrolledwindow;

  return widget;
}
void
uixml_showheader(UiXML * uixml, xmlNode * project)
{
  char * content;
  content = xmlGetProp(project, "name");
  if(!content)
	content = mem_strdup("");
   
  printf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"\n\"http://www.w3.org/TR/html4/strict.dtd\">\n<html>\n<head>\n<title>%s</title>\n<SCRIPT language=JavaScript src=\"jsUtils.js\" type=text/javascript></SCRIPT>\n<link type=\"text/css\" rel=\"stylesheet\" href=\"uihtml_basic.css\">\n<link type=\"text/css\" rel=\"stylesheet\" href=\"uihtml_user.css\">\n<link type=\"text/css\" rel=\"stylesheet\" href=\"menu.css\">\n<meta name=\"generator\" content=\"Bond UIHTML0.1, www.treshna.com\">\n</head>\n<body>\n<table class=\"pagewrapper\">\n<tr>\n<td class=\"pagewrapper\">\n", content);
}
void
uixml_showfooter()
{
  printf("</td></tr>\n</table><!--pagewrapper-->\n</body>\n</html>");
}


/**
 * 
 *
 *
 * Work out the name of widgets and add to hash table.
 */
gint
uixml_parsename(xmlNode * node, UiHtml * widget, UiXML * uixml)
{
  gchar *longname, *shortname = NULL;

  /*  g_assert(widget);
   */
  uixml_parsewidget_attribute(node, widget, &shortname, uixml);

  uixml_generatepath(node, uixml, &longname, &shortname);
  g_hash_table_insert(uixml->widgets, widget, longname);
  g_hash_table_insert(uixml->shortname, shortname, widget);
  g_hash_table_insert(uixml->longname, longname, widget);
  return 0;
}

static void
uixml_buildglistonmatch(void *_key, void *_widget, void *_retlist)
{
  UiHtml *widget;
  gchar *key;
  gchar *comparestr;
  guint i = 0;
  gboolean fail = FALSE;
  GList *retlist;

  retlist = _retlist;
  widget = _widget;
  key = _key;
  comparestr = (g_list_first(retlist))->data;

  while (comparestr[i] != 0)
    {
      if (comparestr[i] != key[i])
		{
		  fail = TRUE;
		  break;
		}
      i++;
    }
  if (fail == FALSE)
    retlist = g_list_append(retlist, widget);

  /* Andru, you are norty.  You don't want to return that! */
  /* return retlist; */
}

GList *
uixml_widgetchildren(UiXML * bondxml, gchar * longname)
{
  GList *retlist;

  /* GList *walk; */

  g_assert(longname);
  retlist = g_list_append(NULL, longname);
  g_hash_table_foreach(bondxml->longname, uixml_buildglistonmatch, retlist);
  retlist = g_list_remove_link(retlist, retlist);
  /* for(walk=g_list_first(retlist);walk!=NULL;walk=walk->next)
     debugmsg("%s",g_hash_table_lookup(bond->widgets, walk->data)); */ 
  return retlist;
}

gchar *
uixml_getwidgetshortname_fromname(gchar * name)
{
  gint i, slen;
  
  if (!name)
    return NULL;
  slen = strlen(name);
  for (i = slen - 1; i >= 0; i++)
    if (name[i] == '/')
      return &name[i];
  return name;
}
