#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <glob.h>
#include <dlfcn.h>

#include <glib.h>

#include "uihtml.h"
#include "bc.h"
#include "parse.h"
#include "uigeneral.h"
#include "uidropdown.h"
#include "uilist.h"
#include "uisignal.h"
#include "uimessage.h"

#include "uicommonplugin.h"

/**
 * plugin_exportfunctions:
 * @plugin: plugin to add function information
 * 
 * 
 * Returns: non-zero on error
 */
static gint
plugin_exportfunctions(Plugin * plugin)
	{
	gint retval = 0;

	/* Top Level */
	retval -= plugin_addfunction(plugin, "init", (void *)uihtml_init);
	retval -= plugin_addfunction(plugin, "parse", (void *)uihtml_parse);
	retval -= plugin_addfunction(plugin, "main", (void *)uihtml_main);
	retval -= plugin_addfunction(plugin, "mainquit", (void *)uihtml_mainquit);
	retval -= plugin_addfunction(plugin, "cleanup", (void *)uihtml_cleanup);
	/* Common functions */
	retval -= plugin_addfunction(plugin, "getname", (void *)uihtml_getname);
	retval -= plugin_addfunction(plugin, "getname_short",
	                             (void *)uihtml_getwidgetshortname);
	retval -= plugin_addfunction(plugin, "getwidget", (void *)uihtml_getwidget);
	retval -= plugin_addfunction(plugin, "gettype", (void *)uihtml_gettype);
	retval -= plugin_addfunction(plugin, "getchildren",
	                             (void *)uihtml_getchildren);
	retval -= plugin_addfunction(plugin, "getparent", (void *)uihtml_getparent);
	retval -= plugin_addfunction(plugin, "widgetlongname",
	                             (void *)uihtml_widgetlongname);
	retval -=
	    plugin_addfunction(plugin, "showwidget", (void *)uihtml_showwidget);
	retval -=
	    plugin_addfunction(plugin, "hidewidget", (void *)uihtml_hidewidget);
	retval -= plugin_addfunction(plugin, "getvalue", (void *)uihtml_getvalue);
	retval -= plugin_addfunction(plugin, "setvalue", (void *)uihtml_setvalue);
	retval -= plugin_addfunction(plugin, "isvisible", (void *)uihtml_isvisible);

	/*  retval -= plugin_addfunction(plugin, "seteditable",
								   (void *)uihtml_seteditable);
	  retval -= plugin_addfunction(plugin, "geteditable", 
								   (void *)uihtml_geteditable);*/
	/* dropdown stuff */
	retval -= plugin_addfunction(plugin, "dropdown_populate",
	                             (void *)uihtml_dropdown_populate);

	/*  retval -= plugin_addfunction(plugin, "dropdown_getentrybox",
								   (void *)uihtml_dropdown_getentrybox);*/
	retval -= plugin_addfunction(plugin, "dropdown_clear",
	                             (void *)uihtml_dropdown_clear);
	retval -= plugin_addfunction(plugin, "dropdown_getpopulatelist",
	                             (void *)uihtml_dropdown_populatelist);

	/* list stuff */
	retval -= plugin_addfunction(plugin, "list_getselection",
	                             (void *)uihtml_list_getselection);
	retval -= plugin_addfunction(plugin, "list_setselection",
	                             (void *)uihtml_list_setselection);
	retval -= plugin_addfunction(plugin, "list_append",
	                             (void *)uihtml_list_append);

	/*  retval -= plugin_addfunction(plugin, "list_clear",
								   (void *)uihtml_list_clear);*/
	retval -= plugin_addfunction(plugin, "list_setcellvalue",
	                             (void *)uihtml_list_setcellvalue);

	/* signal stuff */
	retval -= plugin_addfunction(plugin, "signalconnect",
	                             (void *)uihtml_signalconnect);
	retval -= plugin_addfunction(plugin, "signaldisconnect",
	                             (void *)uihtml_signaldisconnect);
	retval -= plugin_addfunction(plugin, "signaldisconnectbyfunc",
	                             (void *)uihtml_signaldisconnectbyfunc);
	retval -= plugin_addfunction(plugin, "signalconnect_keypress",
	                             (void *)uihtml_signalconnect_keypress);
	retval -= plugin_addfunction(plugin, "signaldisconnect_keypress",
	                             (void *)uihtml_signaldisconnect_keypress);
	/* retval -= plugin_addfunction(plugin, "signalemitstopbyname", (void
	   )uihtml_signalemitstopbyname); */
	/* retval -= plugin_addfunction(plugin, "issignalattached",
	   uihtml_issignalattached); */

	retval -=
	    plugin_addfunction(plugin, "setuserdata", (void *)uihtml_setuserdata);
	retval -=
	    plugin_addfunction(plugin, "getcontainerchildren",
	                       (void *)uihtml_getcontainerchildren);
	retval -=
	    plugin_addfunction(plugin, "message_display",
	                       (void *)uihtml_message_display);

	return retval;
	}

/**
 * plugin_init:
 * @plugin: plugin structure passed in
 *
 * Will build a list of functions in the library and return them to 
 * parent library.
 *
 * Returns: 0 on success
 */
gint
plugin_init(Plugin * plugin)
	{
	if (!IsEqual(plugin->name, "html"))
		{
		errormsg("Not the correct library, can not continue.");
		return -1;
		}

	debug_output("Loading html user interface library.\n");

	plugin_exportfunctions(plugin);
	return 0;
	}

/**
 * plugin_addfunction:
 * @plugin:
 * 
 * Function used by plugin libraries to export functions. 
 */
gint
plugin_addfunction(Plugin * plugin, gchar * funcname, void *(*funcptr) ())
	{
	g_assert(plugin);
	plugin->func = mem_realloc(plugin->func,
	                           (sizeof(PluginFunction *) *
	                            (plugin->numfunctions + 1)));
	plugin->func[plugin->numfunctions] = mem_alloc(sizeof(PluginFunction));
	plugin->func[plugin->numfunctions]->name = mem_strdup(funcname);
	plugin->func[plugin->numfunctions]->ptr = funcptr;
	plugin->numfunctions++;
	return 0;
	}

void
plugin_cleanup(Plugin * plugin)
	{
	gint i;

	mem_verify(plugin);
	for (i = 0; i < plugin->numfunctions; i++)
		{
		mem_free(plugin->func[i]->name);
		mem_free(plugin->func[i]);
		}

	mem_free(plugin->func);
	}

/**
 * ui_init:
 * @xmlfile: Bond XML which contains the UI descripition
 * 
 * Run all the gtk and XML initilising including gtk_init();
 * Create the global instance of xmlwindow, which will contain a list of all the windows.
 * This will make it nice and easy for window creating.
 *
 * Returns: non-zero on error
 */
UiXML *
uihtml_init(gchar * xmlfile, gint argc, gchar * argv[])
	{
	UiXML *bondxml;

	gtk_init(&argc, &argv);
	bondxml = uixml_init(xmlfile);

	if (!bondxml || !bondxml->doc)
		return NULL;
	return bondxml;
	}

void
uihtml_parse(UiXML * bondxml)
	{
	xmlNodePtr root = NULL;

	root = xmlDocGetRootElement(bondxml->doc);
	uixml_parse(root, 0, NULL, bondxml);
	uixml_showfooter();

	}

void
uihtml_cleanup(UiXML * bondxml)
	{
	plugin_cleanup(bondxml->plugin);
	}

void
uihtml_main(UiXML * bondxml)
	{
	gtk_main();
	}

void
uihtml_mainquit(UiXML * bondxml)
	{
	gtk_main_quit();
	}

UiHtml *
uihtml_create(xmlNode * node)
	{
	UiHtml *uihtml;

	g_assert(node);

	uihtml = mem_calloc(sizeof(UiHtml));
	uihtml->editable = TRUE;
	uihtml->node = node;
	uihtml->openinghtml = NULL;
	uihtml->closinghtml = NULL;
	uihtml->children = NULL;
	uihtml->parent = NULL;

	return uihtml;
	}

gint
uihtml_output(int indent, UiXML * uixml, UiHtml * uihtml)
	{
	GList *walk;

	if (uihtml->openinghtml)
		printf("%*s%s\n", indent * 2, "", uihtml->openinghtml);
	for (walk = g_list_first(uihtml->children); walk != NULL; walk = walk->next)
		{
		uihtml_output(indent + 1, uixml, walk->data);
		}
	if (uihtml->closinghtml)
		printf("%*s%s\n", indent * 2, "", uihtml->closinghtml);

	return 0;
	}
