/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_funcs.h"
#include "bc.h"
#include "uimenu.h"
#include "uihtml.h"
#include "uigeneral.h"
#include "parse.h"

static UiHtml *
uihtml_menuitem_create(UiXML * uixml, xmlNode * parentnode, UiHtml * parentwidget)
{
  gchar  *content, *name, *label;
  UiHtml *menuitem;
  int i, j;
  
  menuitem = uihtml_create(parentnode);
  menuitem->widgettype = UiGeneral;
  menuitem->openinghtml = mem_strdup("<input type=\"submit\" class=\"menuitem\"");
  
  content = xmlGetProp(parentnode, "label");
  if(content)
	{
	 label = content;
	  
	  /* remove underscores for label */
	  j=0; i=0;
	  while(label[j] != 0)
		{
		  if (label[j] == '_')
			j++;
		  label[i] = label[j];
		  i++;
		  j++;
		}
	  label[i] = 0;
	  
	   name = mem_strdup(label);
	  /* replace spaces for 'button' names*/
	  j=0; i=0;
	  while(name[j] != 0)
		{
		  if(isspace(name[j]))
			name[j] = '_';
		  name[i] = tolower(name[j]);
		  i++;
		  j++;
		}
	  name[i] = 0;
	}
  else
	{
	  name = xmlGetProp(parentnode, "stock");
	  if(name && !strcmp(name, "gtk-quit"))
		{
		  label = mem_strdup("Quit");
		  name = mem_strdup("quit");
		}
	}
  menuitem->openinghtml = mem_strdup_printf("%s name=\"%s\" value=\"%s\">", 
											menuitem->openinghtml,
											name, label);
  
  menuitem->closinghtml = mem_strdup("");
  
  parentwidget->children = g_list_append(parentwidget->children, menuitem);
  menuitem->parent = parentwidget;
  
  uixml_parsename(parentnode, menuitem, uixml);
  
  /*	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	gtk_widget_show(menuitem);*/
  return menuitem;
}

UiHtml *
uihtml_menu_create(xmlNode * parentnode, UiHtml * menubar, UiXML *bondxml)
{
	gchar *content;
	UiHtml *menu;
	xmlNode *node;
	
	menu = uihtml_create(parentnode);
	menu->widgettype = UiGeneral;

	content = xmlGetProp(parentnode, "label");
	
	menu->openinghtml = mem_strdup_printf("<input type=\"submit\"  class=\"menubutton\" href=\"\" onclick=\"return buttonClick(this,'%s');\" onmouseover=\"buttonMouseover(this,'%s');\" value=\"%s &nabla;\">\n<div id=\"%s\" class=\"menu\">", content, content, content, content);

	menu->closinghtml = mem_strdup("</div>");

	/* Parse the menu items */
	for (node = parentnode->children; node != NULL; node = node->next) {
		if (IsEqual(node->name, "menuitem"))
		  uihtml_menuitem_create(bondxml, node, menu);
		else if (IsEqual(node->name, "separator")) {
		  uihtml_menuseparator_create(bondxml, node, menu);
		  /* do something...maybe? */
		  ;
		}
	}
	return menu;
}

UiHtml *
uihtml_menubar_create(UiXML * uixml, xmlNode * parentnode, 
					 UiHtml * parentwidget)
{
	gchar *content = NULL;
	UiHtml *menubar;
	
	menubar = uihtml_create(parentnode);
	menubar->widgettype = UiGeneral;
	
	menubar->openinghtml = mem_strdup("<div id=\"menubar\">");
	menubar->closinghtml = mem_strdup("</div>");

	debuglog(50,"Creating menubar which shall have many options");
     
	content = xmlGetProp(parentnode, "visible");
	if (content && IsFalse(content))
	  menubar->visible = FALSE;
	else
	  menubar->visible=TRUE;

	return menubar;
}

gint
uihtml_menubar_add(UiHtml *menuitem, UiHtml *menubar)
{
  menubar->children = g_list_append(menubar->children, menuitem);
  menuitem->parent = menubar;
  
  debugmsg("Adding %s to %s", menuitem->openinghtml, menubar->openinghtml);
  
  return 0;
}

static UiHtml *
uihtml_menuseparator_create(UiXML * uixml, xmlNode * parentnode, UiHtml * parentwidget)
{
  UiHtml *menuseparator;
  
  menuseparator = uihtml_create(parentnode);
  menuseparator->widgettype = UiGeneral;
  menuseparator->openinghtml = mem_strdup("<div class=\"sep\"></div>");
  
  parentwidget->children = g_list_append(parentwidget->children, menuseparator);
  menuseparator->parent = parentwidget;
  
  uixml_parsename(parentnode, menuseparator, uixml);
  
  return menuseparator;
}
