/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_display.h"
#include "xml_funcs.h"
#include "bc.h"
#include "parse.h"
#include "uinotebook.h"

/**
 * uihtml_notebook_create:
 * @widgetnode: a xml node in the bond XML file.
 * @parentwidget: widget that will be the parent of this.
 * 
 * creates an empty notebook widget from the values specified in the XML
 *
 * Options:
 * <scrollable> indicates wether the notebook should be scrollable. set to non-scrollable by default 
 * <border> sets a border around the notebook if specifeid. By default the notebook does not contain a border.
 */

UiHtml *
uihtml_notebook_create(UiXML * uixml, xmlNode * widgetnode, UiHtml * parentwidget,
		 gboolean * visible)
{
  UiHtml *notebook;
  gchar *content;

	debuglog(50, "Creating a notebook");
	notebook = gtk_notebook_new();

	content = xmlGetProp(widgetnode, "scrollable");
	if (content && IsTrue(content))
		gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);
	else 
		gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), FALSE);
		
	content = xmlGetProp(widgetnode, "show_border");
	if (content && IsTrue(content))
		gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), TRUE);
	else 
		gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);

	/*	gtk_widget_show(notebook); */
	return notebook;
}

/*uihtml_notebook_addpage:
 * @widgetnode: a xml node in the bond XML file.
 * @parentwidget: widget that will be the parent of this.
 * 
 * creates a new page with the sepecified details in the XML and adds it to the notebook.
 *
 * Options:
 * title: the name label of the pagename. 
 */
gint
uihtml_notebook_addpage(xmlNode * widgetnode, UiHtml * widget, UiHtml * parentwidget)
{
	UiHtml *page = NULL;
	gchar *content;

	debuglog(50, "In notebook about to add a add page");

	content = xmlGetProp(widgetnode, "title");
	if (content) {
		page = gtk_label_new(content);
		gtk_notebook_append_page(GTK_NOTEBOOK(parentwidget), widget, page);
	}
	else {
		page = gtk_label_new("No title specified");
		gtk_notebook_append_page(GTK_NOTEBOOK(parentwidget), widget, page);
	}
	
	gtk_widget_show(parentwidget);
	return 0;
}
