#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_display.h"
#include "bc.h"
#include "uiprogressbar.h"
#include "parse.h"

/*uihtml_progress_bar_create:
 * @widgetnode: a xml node in the bond XML file.
 * @parentwidget: widget that will be the parent of this.
 * 
 * creates a progress bar from the values specified in the XML
 *
 * Options:
 * <textvalue> sets the specified text on the background of the progress bar 
 * <progressvalue> indaication of the amount of progress made. The number should be wihin the range (0.0-1.0)
 * <alignment> whhich way the progress bar should be. it can either be top-bottom, bottom-top,
 * left-right or right-left
 */

UiHtml * uihtml_progress_bar_create(UiXML * uixml, xmlNode * widgetnode, 
								   UiHtml * parentwidget, 
								   gboolean * visible)
{
	GtkProgressBar *progress_bar;
	gchar *content = NULL;
	xmlNode *node;

	progress_bar = GTK_PROGRESS_BAR(gtk_progress_bar_new());
	gtk_widget_show(GTK_WIDGET(progress_bar));

	content = xmlGetProp(node, "alignment");
	if (content) {
		if (IsEqual(content, "bottom-top"))
			gtk_progress_bar_set_orientation(progress_bar, GTK_PROGRESS_BOTTOM_TO_TOP);
		else if (IsEqual(content, "top-bottom"))
			gtk_progress_bar_set_orientation(progress_bar, GTK_PROGRESS_TOP_TO_BOTTOM);
		else if (IsEqual(content, "left-right"))
			gtk_progress_bar_set_orientation(progress_bar, GTK_PROGRESS_LEFT_TO_RIGHT);
		else if (IsEqual(content, "right-left"))
			gtk_progress_bar_set_orientation(progress_bar, GTK_PROGRESS_RIGHT_TO_LEFT);
	}

	for (node = widgetnode->children; node; node = node->next)
	{
		content = xmlNodeGetContent(node);
		if (!content)
			continue;
		if (uixml_node(node, "textvalue"))
			gtk_progress_bar_set_text(progress_bar, content);
		else if (uixml_node(node, "progressvalue"))
			gtk_progress_bar_set_fraction(progress_bar, atof(content));
	}

	return GTK_WIDGET(progress_bar);
}
