/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "xml_display.h"
#include "bc.h"
#include "uitextview.h"
#include "parse.h"

/*Creates a text view and buffer widget*/

/*XML notation:
  <textview editable="false" wrap="char" cursorvisible="false">
      Some text to put in the textview...
  </textview>
*/

UiHtml *
uihtml_textview_create(UiXML * uixml, xmlNode * widgetnode, UiHtml * parentwidget)
{
	UiHtml *textarea;
	xmlChar *content = NULL;
	gchar *style = NULL, *text = NULL;

	textarea = uihtml_create(widgetnode);
	textarea->openinghtml = mem_strdup("<textarea");

	content = xmlGetProp(widgetnode, "name");
	if(content)
	  textarea->openinghtml = mem_strdup_printf("%s name=\"%s\" rows=\"5\" cols=\"30\" ",
												textarea->openinghtml, content);

	content = xmlGetProp(widgetnode, "editable");
	if (content && IsFalse(content))
	  textarea->openinghtml = mem_strdup_printf("%s disabled", 
												textarea->openinghtml);
		
	content = xmlGetProp(widgetnode, "justify");
	if (content) {
	  if (IsEqual(content, "right"))
		style = mem_strdup("text-align:right;");
	  else if (IsEqual(content, "center"))
		style = mem_strdup("text-align:center;");
	}

	content = xmlNodeGetContent(widgetnode);
	if (content) {
	  text = mem_strdup(content);
	  g_strstrip(text);
	  debuglog(50, "Retrieved textual content: %s", text);
	}
	else
	  debugmsg("Failed to get content from %s",widgetnode->name);
	
	debuglog(50, "Creating a textview");

	if(style)
	  textarea->openinghtml = 
		mem_strdup_printf("%s style=\"%s\">", textarea->openinghtml, style);
	else
	  textarea->openinghtml = 
		mem_strdup_printf("%s>", textarea->openinghtml);
	
	if(text)
	  textarea->openinghtml = 
		mem_strdup_printf("%s%s", textarea->openinghtml, text);
	
	/*TO DO size of textarea - rows= and cols=*/
	textarea->closinghtml = mem_strdup("</textarea>");

	free(style);
	free(text);

	return textarea;
}
